@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -30,18 +30,24 @@ import sys
 from nuitka.build.DataComposerInterface import runDataComposer
 from nuitka.build.SconsUtils import getSconsReportValue
 from nuitka.constants.Serialization import ConstantAccessor
+from nuitka.freezer.IncludedDataFiles import (
+    addIncludedDataFilesFromFileOptions,
+    addIncludedDataFilesFromPackageOptions,
+    addIncludedDataFilesFromPlugins,
+    copyDataFiles,
+)
 from nuitka.freezer.IncludedEntryPoints import (
+    addExtensionModuleEntryPoint,
     addIncludedEntryPoints,
-    addShlibEntryPoint,
     getStandaloneEntryPoints,
     setMainEntryPoint,
 )
-from nuitka.freezer.Standalone import copyDataFiles
 from nuitka.importing import Importing, Recursion
 from nuitka.Options import (
     getPythonPgoInput,
     hasPythonFlagNoAsserts,
     hasPythonFlagNoWarnings,
+    hasPythonFlagUnbuffered,
 )
 from nuitka.plugins.Plugins import Plugins
 from nuitka.PostProcessing import executePostProcessing
@@ -51,17 +57,15 @@ from nuitka.Progress import (
     setupProgressBar,
 )
 from nuitka.PythonFlavors import (
-    isAnacondaPython,
     isApplePython,
     isDebianPackagePython,
-    isMSYS2MingwPython,
+    isFedoraPackagePython,
     isNuitkaPython,
     isPyenvPython,
 )
 from nuitka.PythonVersions import (
     getPythonABI,
     getSupportedPythonVersions,
-    getSystemPrefixPath,
     python_version,
     python_version_str,
 )
@@ -70,34 +74,46 @@ from nuitka.tree import SyntaxErrors
 from nuitka.utils import InstanceCounters, MemoryUsage
 from nuitka.utils.Execution import (
     callProcess,
-    withEnvironmentVarOverriden,
+    withEnvironmentVarOverridden,
     wrapCommandForDebuggerForExec,
 )
 from nuitka.utils.FileOperations import (
+    changeFilenameExtension,
     deleteFile,
-    getDirectoryRealPath,
     getExternalUsePath,
-    makePath,
-    putTextFileContents,
+    openTextFile,
     removeDirectory,
+    resetDirectory,
 )
 from nuitka.utils.Importing import getSharedLibrarySuffix
 from nuitka.utils.ModuleNames import ModuleName
 from nuitka.utils.ReExecute import callExecProcess, reExecuteNuitka
 from nuitka.utils.StaticLibraries import getSystemStaticLibPythonPath
-from nuitka.utils.Utils import getArchitecture, getOS, isWin32Windows
+from nuitka.utils.Utils import isMacOS, isWin32Windows
 from nuitka.Version import getCommercialVersion, getNuitkaVersion
 
-from . import ModuleRegistry, Options, OutputDirectories, TreeXML
-from .build import SconsInterface
-from .codegen import CodeGeneration, LoaderCodes, Reports
+from . import ModuleRegistry, Options, OutputDirectories
+from .build.SconsInterface import (
+    asBoolStr,
+    cleanSconsDirectory,
+    getSconsDataPath,
+    runScons,
+    setCommonSconsOptions,
+)
+from .code_generation import CodeGeneration, LoaderCodes, Reports
 from .finalizations import Finalization
 from .freezer.Onefile import packDistFolderToOnefile
-from .freezer.Standalone import copyUsedDLLs
+from .freezer.Standalone import (
+    checkFreezingModuleSet,
+    copyDllsUsed,
+    detectUsedDLLs,
+)
 from .optimizations.Optimization import optimizeModules
 from .pgo.PGO import readPGOInputFile
 from .Reports import writeCompilationReport
-from .tree import Building
+from .tree.Building import buildMainModuleTree
+from .tree.SourceHandling import writeSourceCode
+from .TreeXML import dumpTreeXMLToFile
 
 
 def _createNodeTree(filename):
@@ -112,7 +128,7 @@ def _createNodeTree(filename):
     # Many cases to deal with, pylint: disable=too-many-branches
 
     # First, build the raw node tree from the source code.
-    main_module = Building.buildMainModuleTree(
+    main_module = buildMainModuleTree(
         filename=filename,
         is_main=not Options.shallMakeModule(),
     )
@@ -123,13 +139,17 @@ def _createNodeTree(filename):
     source_dir = OutputDirectories.getSourceDirectoryPath()
 
     if not Options.shallOnlyExecCCompilerCall():
-        SconsInterface.cleanSconsDirectory(source_dir)
+        cleanSconsDirectory(source_dir)
 
     # Prepare the ".dist" directory, throwing away what was there before.
     if Options.isStandaloneMode():
-        standalone_dir = OutputDirectories.getStandaloneDirectoryPath()
-        removeDirectory(path=standalone_dir, ignore_errors=True)
-        makePath(standalone_dir)
+        standalone_dir = OutputDirectories.getStandaloneDirectoryPath(bundle=False)
+        resetDirectory(path=standalone_dir, ignore_errors=True)
+
+        if Options.shallCreateAppBundle():
+            resetDirectory(
+                path=changeFilenameExtension(standalone_dir, ".app"), ignore_errors=True
+            )
 
     # Delete result file, to avoid confusion with previous build and to
     # avoid locking issues after the build.
@@ -157,8 +177,8 @@ def _createNodeTree(filename):
 
         if kind != "absolute":
             inclusion_logger.sysexit(
-                "Error, failed to locate package %r you asked to include."
-                % package_name.asString()
+                "Error, failed to locate package '%s' you asked to include."
+                % package_name
             )
 
         Recursion.checkPluginPath(
@@ -189,6 +209,10 @@ def _createNodeTree(filename):
     # Then optimize the tree and potentially recursed modules.
     optimizeModules(main_module.getOutputFilename())
 
+    # Freezer may have concerns for some modules.
+    if Options.isStandaloneMode():
+        checkFreezingModuleSet()
+
     # Allow plugins to comment on final module set.
     Plugins.onModuleCompleteSet()
 
@@ -203,9 +227,14 @@ def _createNodeTree(filename):
         return main_module
 
 
-def dumpTreeXML(tree):
-    xml_root = tree.asXml()
-    TreeXML.dump(xml_root)
+def dumpTreeXML():
+    filename = Options.getXMLDumpOutputFilename()
+
+    if filename is not None:
+        with openTextFile(filename, "w") as output_file:
+            # XML output only.
+            for module in ModuleRegistry.getDoneModules():
+                dumpTreeXMLToFile(tree=module.asXml(), output_file=output_file)
 
 
 def pickSourceFilenames(source_dir, modules):
@@ -245,7 +274,7 @@ def pickSourceFilenames(source_dir, modules):
 
     # First pass, check for collisions.
     for module in modules:
-        if module.isPythonShlibModule():
+        if module.isPythonExtensionModule():
             continue
 
         _base_filename, collision_filename = _getModuleFilenames(module)
@@ -264,7 +293,7 @@ def pickSourceFilenames(source_dir, modules):
     # Second pass, this time sorted, so we get deterministic results. We will
     # apply an "@1"/"@2",... to disambiguate the filenames.
     for module in sorted(modules, key=lambda x: x.getFullName()):
-        if module.isPythonShlibModule():
+        if module.isPythonExtensionModule():
             continue
 
         base_filename, collision_filename = _getModuleFilenames(module)
@@ -280,9 +309,6 @@ def pickSourceFilenames(source_dir, modules):
     return module_filenames
 
 
-standalone_entry_points = []
-
-
 def makeSourceDirectory():
     """Get the full list of modules imported, create code for all of them."""
     # We deal with a lot of details here, but rather one by one, and split makes
@@ -322,10 +348,9 @@ def makeSourceDirectory():
         if "*" in any_case_module or "{" in any_case_module:
             continue
 
-        for module in ModuleRegistry.getDoneModules():
-            if module.getFullName() == any_case_module:
-                break
-        else:
+        if not ModuleRegistry.hasDoneModule(
+            any_case_module
+        ) and not ModuleRegistry.hasRootModule(any_case_module):
             general.warning(
                 "Did not follow import to unused '%s', consider include options."
                 % any_case_module
@@ -347,8 +372,8 @@ def makeSourceDirectory():
                 inclusion_logger.info(
                     "Included compiled module '%s'." % module.getFullName()
                 )
-        elif module.isPythonShlibModule():
-            addShlibEntryPoint(module)
+        elif module.isPythonExtensionModule():
+            addExtensionModuleEntryPoint(module)
 
             if Options.isShowInclusion():
                 inclusion_logger.info(
@@ -386,7 +411,7 @@ def makeSourceDirectory():
 
         source_code = CodeGeneration.generateModuleCode(
             module=module,
-            data_filename=os.path.basename(c_filename + "onst"),  # Really .const
+            data_filename=os.path.basename(c_filename[:-2] + ".const"),
         )
 
         writeSourceCode(filename=c_filename, source_code=source_code)
@@ -446,11 +471,11 @@ def _wasMsvcMode():
 def _deleteMsvcPGOFiles(pgo_mode):
     assert _wasMsvcMode()
 
-    msvc_pgc_filename = OutputDirectories.getResultBasepath(onefile=False) + "!1.pgc"
+    msvc_pgc_filename = OutputDirectories.getResultBasePath(onefile=False) + "!1.pgc"
     deleteFile(msvc_pgc_filename, must_exist=False)
 
     if pgo_mode == "use":
-        msvc_pgd_filename = OutputDirectories.getResultBasepath(onefile=False) + ".pgd"
+        msvc_pgd_filename = OutputDirectories.getResultBasePath(onefile=False) + ".pgd"
         deleteFile(msvc_pgd_filename, must_exist=False)
 
     return msvc_pgc_filename
@@ -468,7 +493,7 @@ def _runCPgoBinary():
     if _wasMsvcMode():
         msvc_pgc_filename = _deleteMsvcPGOFiles(pgo_mode="generate")
 
-        with withEnvironmentVarOverriden(
+        with withEnvironmentVarOverridden(
             "PATH",
             getSconsReportValue(
                 source_dir=OutputDirectories.getSourceDirectoryPath(), key="PATH"
@@ -480,6 +505,7 @@ def _runCPgoBinary():
     else:
         _exit_code = _runPgoBinary()
 
+        # gcc file suffix, spell-checker: ignore gcda
         gcc_constants_pgo_filename = os.path.join(
             OutputDirectories.getSourceDirectoryPath(), "__constants.gcda"
         )
@@ -501,7 +527,7 @@ def _runPythonPgoBinary():
 
     pgo_filename = OutputDirectories.getPgoRunInputFilename()
 
-    with withEnvironmentVarOverriden("NUITKA_PGO_OUTPUT", pgo_filename):
+    with withEnvironmentVarOverridden("NUITKA_PGO_OUTPUT", pgo_filename):
         _exit_code = _runPgoBinary()
 
     if not os.path.exists(pgo_filename):
@@ -512,34 +538,24 @@ def _runPythonPgoBinary():
     return pgo_filename
 
 
-def runSconsBackend(quiet):
+def runSconsBackend():
     # Scons gets transported many details, that we express as variables, and
     # have checks for them, leading to many branches and statements,
     # pylint: disable=too-many-branches,too-many-statements
 
-    asBoolStr = SconsInterface.asBoolStr
-
     options = {
-        "result_name": OutputDirectories.getResultBasepath(onefile=False),
+        "result_name": OutputDirectories.getResultBasePath(onefile=False),
         "source_dir": OutputDirectories.getSourceDirectoryPath(),
         "nuitka_python": asBoolStr(isNuitkaPython()),
         "debug_mode": asBoolStr(Options.is_debug),
-        "python_debug": asBoolStr(Options.isPythonDebug()),
-        "unstripped_mode": asBoolStr(Options.isUnstripped()),
+        "python_debug": asBoolStr(Options.shallUsePythonDebug()),
         "module_mode": asBoolStr(Options.shallMakeModule()),
-        "full_compat": asBoolStr(Options.is_fullcompat),
+        "full_compat": asBoolStr(Options.is_full_compat),
         "experimental": ",".join(Options.getExperimentalIndications()),
         "trace_mode": asBoolStr(Options.shallTraceExecution()),
         "python_version": python_version_str,
-        "target_arch": getArchitecture(),
-        "python_prefix": getDirectoryRealPath(getSystemPrefixPath()),
-        "nuitka_src": SconsInterface.getSconsDataPath(),
-        "module_count": "%d"
-        % (
-            1
-            + len(ModuleRegistry.getDoneModules())
-            + len(ModuleRegistry.getUncompiledNonTechnicalModules())
-        ),
+        "nuitka_src": getSconsDataPath(),
+        "module_count": "%d" % len(ModuleRegistry.getDoneModules()),
     }
 
     if Options.isLowMemory():
@@ -561,11 +577,8 @@ def runSconsBackend(quiet):
     if isDebianPackagePython():
         options["debian_python"] = asBoolStr(True)
 
-    if isMSYS2MingwPython():
-        options["msys2_mingw_python"] = asBoolStr(True)
-
-    if isAnacondaPython():
-        options["anaconda_python"] = asBoolStr(True)
+    if isFedoraPackagePython():
+        options["fedora_python"] = asBoolStr(True)
 
     if isApplePython():
         options["apple_python"] = asBoolStr(True)
@@ -596,9 +609,6 @@ def runSconsBackend(quiet):
             len(ModuleRegistry.getUncompiledTechnicalModules())
         )
 
-    if getOS() == "Windows":
-        options["noelf_mode"] = asBoolStr(True)
-
     if Options.isProfile():
         options["profile_mode"] = asBoolStr(True)
 
@@ -634,6 +644,9 @@ def runSconsBackend(quiet):
     if python_version >= 0x370 and sys.flags.utf8_mode:
         options["python_sysflag_utf8"] = asBoolStr(True)
 
+    if hasPythonFlagUnbuffered():
+        options["python_sysflag_unbuffered"] = asBoolStr(True)
+
     abiflags = getPythonABI()
     if abiflags:
         options["abiflags"] = abiflags
@@ -641,14 +654,20 @@ def runSconsBackend(quiet):
     if Options.shallMakeModule():
         options["module_suffix"] = getSharedLibrarySuffix(preferred=True)
 
-    SconsInterface.setCommonOptions(options)
+    env_values = setCommonSconsOptions(options)
+
+    # Allow plugins to build definitions.
+    env_values.update(Plugins.getBuildDefinitions())
 
     if Options.shallCreatePgoInput():
         options["pgo_mode"] = "python"
 
-        result = SconsInterface.runScons(
-            options=options, quiet=quiet, scons_filename="Backend.scons"
+        result = runScons(
+            options=options,
+            env_values=env_values,
+            scons_filename="Backend.scons",
         )
+
         if not result:
             return result, options
 
@@ -665,8 +684,11 @@ def runSconsBackend(quiet):
         # there, which currently are not yet there, so it won't run.
         if Options.isPgoMode():
             options["pgo_mode"] = "generate"
-            result = SconsInterface.runScons(
-                options=options, quiet=quiet, scons_filename="Backend.scons"
+
+            result = runScons(
+                options=options,
+                env_values=env_values,
+                scons_filename="Backend.scons",
             )
 
             if not result:
@@ -678,37 +700,21 @@ def runSconsBackend(quiet):
             options["pgo_mode"] = "use"
 
     result = (
-        SconsInterface.runScons(
-            options=options, quiet=quiet, scons_filename="Backend.scons"
+        runScons(
+            options=options,
+            env_values=env_values,
+            scons_filename="Backend.scons",
         ),
         options,
     )
 
+    # Delete PGO files if asked to do that.
     if options.get("pgo_mode") == "use" and _wasMsvcMode():
         _deleteMsvcPGOFiles(pgo_mode="use")
 
     return result
 
 
-def writeSourceCode(filename, source_code):
-    # Prevent accidental overwriting. When this happens the collision detection
-    # or something else has failed.
-    assert not os.path.isfile(filename), filename
-
-    putTextFileContents(filename=filename, contents=source_code, encoding="latin1")
-
-
-def writeBinaryData(filename, binary_data):
-    # Prevent accidental overwriting. When this happens the collision detection
-    # or something else has failed.
-    assert not os.path.isfile(filename), filename
-
-    assert type(binary_data) is bytes
-
-    with open(filename, "wb") as output_file:
-        output_file.write(binary_data)
-
-
 def callExecPython(args, clean_path, add_path):
     old_python_path = os.environ.get("PYTHONPATH")
 
@@ -721,10 +727,6 @@ def callExecPython(args, clean_path, add_path):
         else:
             os.environ["PYTHONPATH"] = Options.getOutputDir()
 
-    # We better flush these, "os.execl" won't do it anymore.
-    sys.stdout.flush()
-    sys.stderr.flush()
-
     # Add the main arguments, previous separated.
     args += Options.getPositionalArgs()[1:] + Options.getMainArgs()
 
@@ -732,7 +734,8 @@ def callExecPython(args, clean_path, add_path):
 
 
 def executeMain(binary_filename, clean_path):
-    if Options.shallRunInDebugger():
+    # Wrap in debugger, unless the CMD file contains that call already.
+    if Options.shallRunInDebugger() and not Options.shallCreateCmdFileForExecution():
         args = wrapCommandForDebuggerForExec(binary_filename)
     else:
         args = (binary_filename, binary_filename)
@@ -741,6 +744,7 @@ def executeMain(binary_filename, clean_path):
 
 
 def executeModule(tree, clean_path):
+    """Execute the extension module just created."""
 
     if python_version < 0x340:
         python_command_template = """\
@@ -788,6 +792,7 @@ def compileTree():
                     memory=MemoryUsage.getHumanReadableProcessMemoryUsage()
                 )
             )
+
         # Now build the target language code for the whole tree.
         makeSourceDirectory()
 
@@ -797,7 +802,7 @@ def compileTree():
 
         # This should take all bytecode values, even ones needed for frozen or
         # not produce anything.
-        loader_code = LoaderCodes.getMetapathLoaderBodyCode(bytecode_accessor)
+        loader_code = LoaderCodes.getMetaPathLoaderBodyCode(bytecode_accessor)
 
         writeSourceCode(
             filename=os.path.join(source_dir, "__loader.c"), source_code=loader_code
@@ -827,23 +832,14 @@ def compileTree():
 
     general.info("Running data composer tool for optimal constant value handling.")
 
-    blob_filename = runDataComposer(source_dir)
-    Plugins.onDataComposerResult(blob_filename)
+    runDataComposer(source_dir)
 
-    for filename, source_code in Plugins.getExtraCodeFiles().items():
-        target_dir = os.path.join(source_dir, "plugins")
-
-        if not os.path.isdir(target_dir):
-            makePath(target_dir)
-
-        writeSourceCode(
-            filename=os.path.join(target_dir, filename), source_code=source_code
-        )
+    Plugins.writeExtraCodeFiles(onefile=False)
 
     general.info("Running C compilation via Scons.")
 
     # Run the Scons to build things.
-    result, options = runSconsBackend(quiet=not Options.isShowScons())
+    result, options = runSconsBackend()
 
     return result, options
 
@@ -854,7 +850,7 @@ def handleSyntaxError(e):
     # versions he wants, tell him about the potential version problem.
     error_message = SyntaxErrors.formatOutput(e)
 
-    if not Options.is_fullcompat:
+    if not Options.is_full_compat:
         if python_version < 0x300:
             suggested_python_version_str = getSupportedPythonVersions()[-1]
         else:
@@ -894,11 +890,14 @@ def main():
     if pgo_filename is not None:
         readPGOInputFile(pgo_filename)
 
-    if not Options.shallDumpBuiltTreeXML():
-        general.info(
-            "Starting Python compilation with Nuitka %r on Python %r commercial %r."
-            % (getNuitkaVersion(), python_version_str, getCommercialVersion())
+    general.info(
+        "Starting Python compilation with Nuitka %r on Python %r commercial grade %r."
+        % (
+            getNuitkaVersion(),
+            python_version_str,
+            getCommercialVersion() or "not installed",
         )
+    )
 
     filename = Options.getPositionalArgs()[0]
 
@@ -908,120 +907,145 @@ def main():
         main_dir=os.path.dirname(os.path.abspath(filename))
     )
 
+    addIncludedDataFilesFromFileOptions()
+    addIncludedDataFilesFromPackageOptions()
+
     # Turn that source code into a node tree structure.
     try:
         main_module = _createNodeTree(filename=filename)
     except (SyntaxError, IndentationError) as e:
         handleSyntaxError(e)
 
-    if Options.shallDumpBuiltTreeXML():
-        # XML output only.
+    addIncludedDataFilesFromPlugins()
+
+    dumpTreeXML()
+
+    # Make the actual compilation.
+    result, options = compileTree()
+
+    # Exit if compilation failed.
+    if not result:
+        sys.exit(1)
+
+    # Relaunch in case of Python PGO input to be produced.
+    if Options.shallCreatePgoInput():
+        # Will not return.
+        pgo_filename = OutputDirectories.getPgoRunInputFilename()
+        general.info(
+            "Restarting compilation using collected information from '%s'."
+            % pgo_filename
+        )
+        reExecuteNuitka(pgo_filename=pgo_filename)
+
+    if Options.shallNotDoExecCCompilerCall():
+        if Options.isShowMemory():
+            MemoryUsage.showMemoryTrace()
+
+        sys.exit(0)
+
+    executePostProcessing()
+
+    copyDataFiles()
+
+    if Options.isStandaloneMode():
+        binary_filename = options["result_exe"]
+
+        setMainEntryPoint(binary_filename)
+
         for module in ModuleRegistry.getDoneModules():
-            dumpTreeXML(module)
-    else:
-        # Make the actual compilation.
-        result, options = compileTree()
+            addIncludedEntryPoints(Plugins.considerExtraDlls(module))
 
-        # Exit if compilation failed.
-        if not result:
-            sys.exit(1)
-
-        # Relaunch in case of Python PGO input to be produced.
-        if Options.shallCreatePgoInput():
-            # Will not return.
-            pgo_filename = OutputDirectories.getPgoRunInputFilename()
-            general.info(
-                "Restarting compilation using collected information from '%s'."
-                % pgo_filename
-            )
-            reExecuteNuitka(pgo_filename=pgo_filename)
-
-        if Options.shallNotDoExecCCompilerCall():
-            if Options.isShowMemory():
-                MemoryUsage.showMemoryTrace()
-
-            sys.exit(0)
-
-        executePostProcessing()
-
-        if Options.isStandaloneMode():
-            binary_filename = options["result_exe"]
-
-            setMainEntryPoint(binary_filename)
-
-            dist_dir = OutputDirectories.getStandaloneDirectoryPath()
-
-            for module in ModuleRegistry.getDoneModules():
-                addIncludedEntryPoints(Plugins.considerExtraDlls(dist_dir, module))
-
-            copyUsedDLLs(
-                source_dir=OutputDirectories.getSourceDirectoryPath(),
-                dist_dir=dist_dir,
-                standalone_entry_points=getStandaloneEntryPoints(),
-            )
-
-            copyDataFiles(dist_dir=dist_dir)
-
-            Plugins.onStandaloneDistributionFinished(dist_dir)
-
-            if Options.isOnefileMode():
-                packDistFolderToOnefile(dist_dir, binary_filename)
-
-                if Options.isRemoveBuildDir():
-                    general.info("Removing dist folder %r." % dist_dir)
-
-                    removeDirectory(path=dist_dir, ignore_errors=False)
-                else:
-                    general.info(
-                        "Keeping dist folder %r for inspection, no need to use it."
-                        % dist_dir
-                    )
-
-        # Remove the source directory (now build directory too) if asked to.
-        source_dir = OutputDirectories.getSourceDirectoryPath()
-
-        if Options.isRemoveBuildDir():
-            general.info("Removing build directory %r." % source_dir)
-
-            removeDirectory(path=source_dir, ignore_errors=False)
-            assert not os.path.exists(source_dir)
-        else:
-            general.info("Keeping build directory %r." % source_dir)
-
-        final_filename = OutputDirectories.getResultFullpath(
-            onefile=Options.isOnefileMode()
+        detectUsedDLLs(
+            standalone_entry_points=getStandaloneEntryPoints(),
+            source_dir=OutputDirectories.getSourceDirectoryPath(),
         )
 
-        if "macos_minversion" in options:
-            general.info(
-                "Created binary that runs on macOS %s or higher."
-                % options["macos_minversion"]
-            )
+        dist_dir = OutputDirectories.getStandaloneDirectoryPath()
 
-        Plugins.onFinalResult(final_filename)
+        copyDllsUsed(
+            dist_dir=dist_dir,
+            standalone_entry_points=getStandaloneEntryPoints(),
+        )
 
-        general.info("Successfully created %r." % final_filename)
+        Plugins.onStandaloneDistributionFinished(dist_dir)
 
-        report_filename = Options.getCompilationReportFilename()
+        if Options.isOnefileMode():
+            packDistFolderToOnefile(dist_dir)
 
-        if report_filename:
-            writeCompilationReport(report_filename)
+            if Options.isRemoveBuildDir():
+                general.info("Removing dist folder '%s'." % dist_dir)
 
-        # Execute the module immediately if option was given.
-        if Options.shallExecuteImmediately():
-            run_filename = OutputDirectories.getResultRunFilename(
-                onefile=Options.isOnefileMode()
-            )
-
-            general.info("Launching %r." % run_filename)
-
-            if Options.shallMakeModule():
-                executeModule(
-                    tree=main_module,
-                    clean_path=Options.shallClearPythonPathEnvironment(),
-                )
+                removeDirectory(path=dist_dir, ignore_errors=False)
             else:
-                executeMain(
-                    binary_filename=run_filename,
-                    clean_path=Options.shallClearPythonPathEnvironment(),
+                general.info(
+                    "Keeping dist folder '%s' for inspection, no need to use it."
+                    % dist_dir
                 )
+
+    # Remove the source directory (now build directory too) if asked to.
+    source_dir = OutputDirectories.getSourceDirectoryPath()
+
+    if Options.isRemoveBuildDir():
+        general.info("Removing build directory '%s'." % source_dir)
+
+        removeDirectory(path=source_dir, ignore_errors=False)
+        assert not os.path.exists(source_dir)
+    else:
+        general.info("Keeping build directory '%s'." % source_dir)
+
+    final_filename = OutputDirectories.getResultFullpath(
+        onefile=Options.isOnefileMode()
+    )
+
+    if Options.isStandaloneMode() and isMacOS():
+        general.info(
+            "Created binary that runs on macOS %s (%s) or higher."
+            % (options["macos_min_version"], options["macos_target_arch"])
+        )
+
+    Plugins.onFinalResult(final_filename)
+
+    if Options.shallMakeModule():
+        base_path = OutputDirectories.getResultBasePath(onefile=False)
+
+        if os.path.isdir(base_path) and os.path.isfile(
+            os.path.join(base_path, "__init__.py")
+        ):
+            general.warning(
+                """\
+The compilation result is hidden by package directory '%s'. Importing will \
+not use compiled code while it exists."""
+                % base_path
+            )
+
+    general.info("Successfully created '%s'." % final_filename)
+
+    report_filename = Options.getCompilationReportFilename()
+
+    if report_filename:
+        writeCompilationReport(report_filename)
+
+    run_filename = OutputDirectories.getResultRunFilename(
+        onefile=Options.isOnefileMode()
+    )
+
+    # Execute the module immediately if option was given.
+    if Options.shallExecuteImmediately():
+        general.info("Launching '%s'" % run_filename)
+
+        if Options.shallMakeModule():
+            executeModule(
+                tree=main_module,
+                clean_path=Options.shallClearPythonPathEnvironment(),
+            )
+        else:
+            executeMain(
+                binary_filename=run_filename,
+                clean_path=Options.shallClearPythonPathEnvironment(),
+            )
+    else:
+        if run_filename != final_filename:
+            general.info(
+                "Execute it by launching '%s', the batch file needs to set environment."
+                % run_filename
+            )
