@@ -37,12 +37,31 @@ def getSupportedPythonVersions():
     return ("2.6", "2.7", "3.3", "3.4", "3.5", "3.6", "3.7", "3.8", "3.9", "3.10")
 
 
+def getNotYetSupportedPythonVersions():
+    """Versions known to not work at all (yet)."""
+    return ("3.11",)
+
+
 def getPartiallySupportedPythonVersions():
     """Partially supported Python versions for Nuitka."""
 
     return ()
 
 
+# Make somewhat sure we keep these ones consistent
+assert len(
+    set(
+        getPartiallySupportedPythonVersions()
+        + getNotYetSupportedPythonVersions()
+        + getSupportedPythonVersions()
+    )
+) == len(
+    getPartiallySupportedPythonVersions()
+    + getNotYetSupportedPythonVersions()
+    + getSupportedPythonVersions()
+)
+
+
 def getSupportedPythonVersionStr():
     supported_python_versions = getSupportedPythonVersions()
 
@@ -108,6 +127,21 @@ def getComplexCallSequenceErrorTemplate():
     return getComplexCallSequenceErrorTemplate.result
 
 
+def getUnboundLocalErrorErrorTemplate():
+    if not hasattr(getUnboundLocalErrorErrorTemplate, "result"):
+        try:
+            # We are doing this on purpose, to get the exception.
+            # pylint: disable=undefined-variable
+            del _f
+        except UnboundLocalError as e:
+            result = e.args[0].replace("_f", "%s")
+            getUnboundLocalErrorErrorTemplate.result = result
+        else:
+            sys.exit("Error, cannot detect expected error message.")
+
+    return getUnboundLocalErrorErrorTemplate.result
+
+
 _needs_set_literal_reverse_insertion = None
 
 
@@ -140,6 +174,7 @@ def getRunningPythonDLLPath():
     )
 
     # We trust ctypes internals here, pylint: disable=protected-access
+    # spell-checker: ignore pythonapi
     return getWindowsRunningProcessModuleFilename(ctypes.pythonapi._handle)
 
 
@@ -189,6 +224,7 @@ def getPythonABI():
         # Cyclic dependency here.
         from nuitka.Options import shallUsePythonDebug
 
+        # spell-checker: ignore getobjects
         if shallUsePythonDebug() or hasattr(sys, "getobjects"):
             if not abiflags.startswith("d"):
                 abiflags = "d" + abiflags
@@ -258,6 +294,12 @@ def getSystemPrefixPath():
 
             sys_prefix = os.path.normpath(os.path.join(python_binary, "..", ".."))
 
+        # Resolve symlinks on Windows manually.
+        if os.name == "nt":
+            from nuitka.utils.FileOperations import getDirectoryRealPath
+
+            sys_prefix = getDirectoryRealPath(sys_prefix)
+
         _the_sys_prefix = sys_prefix
 
     return _the_sys_prefix
