@@ -24,7 +24,13 @@ import subprocess
 import sys
 
 from nuitka import Options, OutputDirectories
-from nuitka.build import SconsInterface
+from nuitka.build.SconsInterface import (
+    asBoolStr,
+    cleanSconsDirectory,
+    getSconsDataPath,
+    runScons,
+    setCommonSconsOptions,
+)
 from nuitka.Options import getOnefileTempDirSpec, isOnefileTempDirMode
 from nuitka.OutputDirectories import getResultFullpath
 from nuitka.plugins.Plugins import Plugins
@@ -35,7 +41,7 @@ from nuitka.utils.Execution import withEnvironmentVarsOverridden
 from nuitka.utils.FileOperations import areSamePaths, removeDirectory
 from nuitka.utils.InstalledPythons import findInstalledPython
 from nuitka.utils.Signing import addMacOSCodeSignature
-from nuitka.utils.Utils import getArchitecture, isMacOS, isWin32Windows
+from nuitka.utils.Utils import isMacOS, isWin32Windows
 
 
 def packDistFolderToOnefile(dist_dir):
@@ -48,11 +54,11 @@ def packDistFolderToOnefile(dist_dir):
     Plugins.onOnefileFinished(onefile_output_filename)
 
 
-def _runOnefileScons(quiet, onefile_compression):
+def _runOnefileScons(onefile_compression):
     source_dir = OutputDirectories.getSourceDirectoryPath(onefile=True)
-    SconsInterface.cleanSconsDirectory(source_dir)
 
-    asBoolStr = SconsInterface.asBoolStr
+    # Let plugins do their thing for onefile mode too.
+    Plugins.writeExtraCodeFiles(onefile=True)
 
     options = {
         "result_name": OutputDirectories.getResultBasePath(onefile=True),
@@ -61,40 +67,40 @@ def _runOnefileScons(quiet, onefile_compression):
         "debug_mode": asBoolStr(Options.is_debug),
         "experimental": ",".join(Options.getExperimentalIndications()),
         "trace_mode": asBoolStr(Options.shallTraceExecution()),
-        "target_arch": getArchitecture(),
-        "python_prefix": sys.prefix,
-        "nuitka_src": SconsInterface.getSconsDataPath(),
+        "nuitka_src": getSconsDataPath(),
         "compiled_exe": OutputDirectories.getResultFullpath(onefile=False),
         "onefile_splash_screen": asBoolStr(
             Options.getWindowsSplashScreen() is not None
         ),
     }
 
-    if Options.isClang():
-        options["clang_mode"] = "true"
-
-    env_values = SconsInterface.setCommonOptions(options)
+    env_values = setCommonSconsOptions(options)
 
     env_values["_NUITKA_ONEFILE_TEMP_SPEC"] = getOnefileTempDirSpec()
-    env_values["_NUITKA_ONEFILE_TEMP"] = "1" if isOnefileTempDirMode() else "0"
-    env_values["_NUITKA_ONEFILE_COMPRESSION"] = "1" if onefile_compression else "0"
+    env_values["_NUITKA_ONEFILE_TEMP_BOOL"] = "1" if isOnefileTempDirMode() else "0"
+    env_values["_NUITKA_ONEFILE_COMPRESSION_BOOL"] = "1" if onefile_compression else "0"
+    env_values["_NUITKA_ONEFILE_BUILD_BOOL"] = "1" if onefile_compression else "0"
 
-    with withEnvironmentVarsOverridden(env_values):
-        result = SconsInterface.runScons(
-            options=options, quiet=quiet, scons_filename="Onefile.scons"
-        )
+    # Allow plugins to build definitions.
+    env_values.update(Plugins.getBuildDefinitions())
+
+    result = runScons(
+        options=options,
+        env_values=env_values,
+        scons_filename="Onefile.scons",
+    )
 
     # Exit if compilation failed.
     if not result:
         onefile_logger.sysexit("Error, onefile bootstrap binary build failed.")
 
     if Options.isRemoveBuildDir():
-        onefile_logger.info("Removing onefile build directory %r." % source_dir)
+        onefile_logger.info("Removing onefile build directory '%s'." % source_dir)
 
         removeDirectory(path=source_dir, ignore_errors=False)
         assert not os.path.exists(source_dir)
     else:
-        onefile_logger.info("Keeping onefile build directory %r." % source_dir)
+        onefile_logger.info("Keeping onefile build directory '%s'." % source_dir)
 
 
 def getCompressorPython():
@@ -177,12 +183,27 @@ def packDistFolderToOnefileBootstrap(onefile_output_filename, dist_dir):
 
     onefile_logger.info("Running bootstrap binary compilation via Scons.")
 
+    # Cleanup first.
+    source_dir = OutputDirectories.getSourceDirectoryPath(onefile=True)
+    cleanSconsDirectory(source_dir)
+
     # Now need to append to payload it, potentially compressing it.
     compressor_python = getCompressorPython()
 
-    # First need to create the bootstrap binary for unpacking.
+    # Decide if we need the payload during build already, or if it should be
+    # attached.
+    payload_used_in_build = isMacOS()
+
+    if payload_used_in_build:
+        runOnefileCompressor(
+            compressor_python=compressor_python,
+            dist_dir=dist_dir,
+            onefile_output_filename=os.path.join(source_dir, "__payload.bin"),
+            start_binary=getResultFullpath(onefile=False),
+        )
+
+    # Create the bootstrap binary for unpacking.
     _runOnefileScons(
-        quiet=not Options.isShowScons(),
         onefile_compression=compressor_python is not None,
     )
 
@@ -194,9 +215,10 @@ def packDistFolderToOnefileBootstrap(onefile_output_filename, dist_dir):
     if isMacOS():
         addMacOSCodeSignature(filenames=[onefile_output_filename])
 
-    runOnefileCompressor(
-        compressor_python=compressor_python,
-        dist_dir=dist_dir,
-        onefile_output_filename=onefile_output_filename,
-        start_binary=getResultFullpath(onefile=False),
-    )
+    if not payload_used_in_build:
+        runOnefileCompressor(
+            compressor_python=compressor_python,
+            dist_dir=dist_dir,
+            onefile_output_filename=onefile_output_filename,
+            start_binary=getResultFullpath(onefile=False),
+        )
