@@ -54,7 +54,10 @@ from nuitka.utils.AppDirs import getCacheDir
 from nuitka.utils.FileOperations import listDir, removeDirectory
 from nuitka.utils.Importing import getSharedLibrarySuffixes
 from nuitka.utils.ModuleNames import ModuleName
-from nuitka.utils.Utils import getOS, isMacOS
+from nuitka.utils.SharedLibraries import (
+    hasUniversalOrMatchingMacOSArchitecture,
+)
+from nuitka.utils.Utils import isMacOS, isWin32OrPosixWindows
 
 from .IgnoreListing import isIgnoreListedNotExistingModule
 from .PreloadedPackages import getPreloadedPackagePath, isPreloadedPackagePath
@@ -64,7 +67,7 @@ _debug_module_finding = Options.shallExplainImports()
 warned_about = set()
 
 # Directory where the main script lives. Should attempt to import from there.
-main_path = None
+_main_path = None
 
 
 def setMainScriptDirectory(main_dir):
@@ -74,12 +77,12 @@ def setMainScriptDirectory(main_dir):
     """
     # We need to set this from the outside, pylint: disable=global-statement
 
-    global main_path
-    main_path = main_dir
+    global _main_path
+    _main_path = main_dir
 
 
 def getMainScriptDirectory():
-    return main_path
+    return _main_path
 
 
 def isPackageDir(dirname):
@@ -329,10 +332,11 @@ def findModule(module_name, parent_package, level):
 
 
 # Some platforms are case insensitive.
-case_sensitive = not isMacOS() and getOS() != "Windows"
+case_sensitive = not isMacOS() and not isWin32OrPosixWindows()
 
 ImportScanFinding = collections.namedtuple(
-    "ImportScanFinding", ("found_in", "priority", "full_path", "search_order")
+    "ImportScanFinding",
+    ("found_in", "module_type", "priority", "full_path", "search_order"),
 )
 
 # We put here things that are not worth it (Cython is not really used by
@@ -368,6 +372,18 @@ existing '%s' extension module by default. Candidates were: %s <-> %s."""
                     )
 
 
+_list_dir_cache = {}
+
+
+def listDirCached(path):
+    """Cached listing of a directory."""
+
+    if path not in _list_dir_cache:
+        _list_dir_cache[path] = tuple(listDir(path))
+
+    return _list_dir_cache[path]
+
+
 def _findModuleInPath2(package_name, module_name, search_path):
     """This is out own module finding low level implementation.
 
@@ -417,6 +433,7 @@ def _findModuleInPath2(package_name, module_name, search_path):
                     candidates.add(
                         ImportScanFinding(
                             found_in=entry,
+                            module_type=module_type,
                             priority=priority_map[module_type],
                             full_path=package_directory,
                             search_order=count,
@@ -428,6 +445,7 @@ def _findModuleInPath2(package_name, module_name, search_path):
                 candidates.add(
                     ImportScanFinding(
                         found_in=entry,
+                        module_type=10,
                         priority=10,
                         full_path=package_directory,
                         search_order=count + len(search_path),
@@ -447,6 +465,7 @@ def _findModuleInPath2(package_name, module_name, search_path):
                 candidates.add(
                     ImportScanFinding(
                         found_in=entry,
+                        module_type=module_type,
                         priority=4 + priority_map[module_type],
                         full_path=full_path,
                         search_order=count,
@@ -457,36 +476,49 @@ def _findModuleInPath2(package_name, module_name, search_path):
     if _debug_module_finding:
         my_print("Candidates:", candidates)
 
+    found_candidate = None
+
     if candidates:
         # Sort by priority, with entries from same path element coming first, then desired type.
-        candidates = sorted(candidates, key=lambda c: (c.search_order, c.priority))
+        candidates = tuple(
+            sorted(candidates, key=lambda c: (c.search_order, c.priority))
+        )
 
         # On case sensitive systems, no resolution needed.
         if case_sensitive:
-            _reportCandidates(
-                package_name=package_name,
-                module_name=module_name,
-                candidate=candidates[0],
-                candidates=candidates,
-            )
-            return candidates[0].full_path
+            found_candidate = candidates[0]
         else:
             for candidate in candidates:
-                for fullname, _filename in listDir(candidate[0]):
+                for fullname, _filename in listDirCached(candidate.found_in):
                     if fullname == candidate.full_path:
-                        _reportCandidates(
-                            package_name=package_name,
-                            module_name=module_name,
-                            candidate=candidate,
-                            candidates=candidates,
-                        )
-                        return candidate.full_path
+                        found_candidate = candidate
+                        break
+
+                if found_candidate:
+                    break
 
             # Only exact case matches matter, all candidates were ignored,
             # lets just fall through to raising the import error.
 
-    # Nothing found.
-    raise ImportError
+    if found_candidate is None:
+        # Nothing found.
+        raise ImportError
+    if (
+        found_candidate.module_type == imp.C_EXTENSION
+        and isMacOS()
+        and not hasUniversalOrMatchingMacOSArchitecture(found_candidate.full_path)
+    ):
+        # Not usable for target architecture.
+        raise ImportError
+
+    _reportCandidates(
+        package_name=package_name,
+        module_name=module_name,
+        candidate=found_candidate,
+        candidates=candidates,
+    )
+
+    return found_candidate.full_path
 
 
 _egg_files = {}
@@ -521,10 +553,10 @@ def _unpackPathElement(path_entry):
 
 
 def getPackageSearchPath(package_name):
-    assert main_path is not None
+    assert _main_path is not None
 
     if package_name is None:
-        result = [os.getcwd(), main_path] + [
+        result = [os.getcwd(), _main_path] + [
             _unpackPathElement(path_element) for path_element in sys.path
         ]
     elif "." in package_name:
@@ -552,10 +584,8 @@ def getPackageSearchPath(package_name):
         def getPackageDirCandidates(element):
             yield os.path.join(element, package_name.asPath()), False
 
-            # Hack for PyWin32. TODO: Move this "__path__" extensions to be
-            # plug-in decisions.
-            if package_name == "win32com":
-                yield os.path.join(element, "win32comext"), True
+            for extra_path in Plugins.getPackageExtraScanPaths(package_name, element):
+                yield extra_path, True
 
         result = []
         for element in getPackageSearchPath(None):
@@ -637,7 +667,11 @@ def _findModule(module_name):
 
         return result
 
-    module_search_cache[key] = _findModuleInPath(module_name)
+    try:
+        module_search_cache[key] = _findModuleInPath(module_name)
+    except ImportError:
+        module_search_cache[key] = ImportError
+        raise
 
     return module_search_cache[key]
 
@@ -668,7 +702,7 @@ def locateModule(module_name, parent_package, level):
 
         module_name, module_kind = getModuleNameAndKindFromFilename(module_filename)
 
-        assert module_kind is not None, module_filename
+        assert module_kind is not None, (module_filename, finding)
 
         module_name = ModuleName.makeModuleNameInPackage(module_name, module_package)
 
