@@ -26,20 +26,22 @@ that is the child of the dictionary creation.
 from nuitka import Constants
 
 from .AttributeNodes import makeExpressionAttributeLookup
+from .ChildrenHavingMixins import ChildrenHavingPairsTupleMixin
 from .ConstantRefNodes import (
     ExpressionConstantDictEmptyRef,
     makeConstantRefNode,
 )
 from .ExpressionBases import (
+    ExpressionBase,
     ExpressionChildHavingBase,
     ExpressionChildrenHavingBase,
-    ExpressionChildTupleHavingBase,
 )
 from .ExpressionShapeMixins import (
     ExpressionBoolShapeExactMixin,
     ExpressionDictShapeExactMixin,
     ExpressionListShapeExactMixin,
     ExpressionNoneShapeExactMixin,
+    ExpressionTupleShapeExactMixin,
 )
 from .NodeBases import (
     SideEffectsFromChildrenMixin,
@@ -99,19 +101,23 @@ def makeExpressionMakeDictOrConstant(pairs, user_provided, source_ref):
 class ExpressionMakeDict(
     ExpressionDictShapeExactMixin,
     SideEffectsFromChildrenMixin,
-    ExpressionChildTupleHavingBase,
+    ChildrenHavingPairsTupleMixin,
+    ExpressionBase,
 ):
     kind = "EXPRESSION_MAKE_DICT"
 
-    named_child = "pairs"
+    named_children = ("pairs",)
 
     def __init__(self, pairs, source_ref):
         assert pairs
 
-        ExpressionChildTupleHavingBase.__init__(
-            self, value=tuple(pairs), source_ref=source_ref
+        ChildrenHavingPairsTupleMixin.__init__(
+            self,
+            pairs=tuple(pairs),
         )
 
+        ExpressionBase.__init__(self, source_ref=source_ref)
+
     def computeExpression(self, trace_collection):
         pairs = self.subnode_pairs
 
@@ -492,6 +498,47 @@ class ExpressionDictOperationPop3(ExpressionChildrenHavingBase):
             )
 
 
+class ExpressionDictOperationPopitem(
+    ExpressionTupleShapeExactMixin, ExpressionChildHavingBase
+):
+    """This operation represents d.popitem()."""
+
+    kind = "EXPRESSION_DICT_OPERATION_POPITEM"
+
+    named_child = "dict_arg"
+
+    def __init__(self, dict_arg, source_ref):
+        assert dict_arg is not None
+
+        ExpressionChildHavingBase.__init__(
+            self,
+            value=dict_arg,
+            source_ref=source_ref,
+        )
+
+    def computeExpression(self, trace_collection):
+        dict_arg = self.subnode_dict_arg
+
+        # TODO: Check if dict_arg is not empty.
+
+        # TODO: Until we have proper dictionary tracing, do this.
+        trace_collection.removeKnowledge(dict_arg)
+
+        # TODO: Until we can know KeyError won't happen, but then we should change into
+        # something else.
+        trace_collection.onExceptionRaiseExit(BaseException)
+
+        return self, None, None
+
+    # TODO: These turn this into dictionary item removal, as value is unused.
+    # def computeExpressionDrop(self, statement, trace_collection):
+
+    # TODO: Might raise KeyError depending on dictionary.
+    @staticmethod
+    def mayRaiseException(exception_type):
+        return True
+
+
 class ExpressionDictOperationSetdefault2(ExpressionChildrenHavingBase):
     """This operation represents d.setdefault(key), i.e. default None."""
 
