@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -140,6 +140,11 @@ class StatementExec(StatementChildrenHavingBase):
 
     named_children = ("source", "globals_arg", "locals_arg")
 
+    checkers = {
+        "globals_arg": convertNoneConstantToNone,
+        "locals_arg": convertNoneConstantToNone,
+    }
+
     def __init__(self, source_code, globals_arg, locals_arg, source_ref):
         StatementChildrenHavingBase.__init__(
             self,
@@ -151,14 +156,8 @@ class StatementExec(StatementChildrenHavingBase):
             source_ref=source_ref,
         )
 
-    def setChild(self, name, value):
-        if name in ("globals_arg", "locals_arg"):
-            value = convertNoneConstantToNone(value)
-
-        return StatementChildrenHavingBase.setChild(self, name, value)
-
     def computeStatement(self, trace_collection):
-        source_code = trace_collection.onExpression(expression=self.subnode_source)
+        source_code = trace_collection.onExpression(self.subnode_source)
 
         if source_code.mayRaiseException(BaseException):
             trace_collection.onExceptionRaiseExit(BaseException)
