@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -37,7 +37,12 @@ from .NodeMakingHelpers import (
     wrapExpressionWithNodeSideEffects,
     wrapExpressionWithSideEffects,
 )
-from .shapes.BuiltinTypeShapes import tshape_dict, tshape_str, tshape_unicode
+from .shapes.BuiltinTypeShapes import (
+    tshape_bytes,
+    tshape_dict,
+    tshape_str,
+    tshape_unicode,
+)
 from .shapes.StandardShapes import tshape_unknown
 
 
@@ -45,6 +50,18 @@ class ExpressionBase(NodeBase):
     # TODO: Maybe we can do this only for debug mode.
     __slots__ = ("code_generated",)
 
+    @staticmethod
+    def isExpression():
+        return True
+
+    @staticmethod
+    def isStatement():
+        return False
+
+    @staticmethod
+    def isStatementsSequence():
+        return False
+
     @staticmethod
     def getTypeShape():
         return tshape_unknown
@@ -352,6 +369,26 @@ class ExpressionBase(NodeBase):
 
         return call_node, None, None
 
+    def computeExpressionCallViaVariable(
+        self, call_node, variable_ref_node, call_args, call_kw, trace_collection
+    ):
+        # Virtual method, pylint: disable=unused-argument
+
+        # The called and the arguments escape for good.
+        self.onContentEscapes(trace_collection)
+        if call_args is not None:
+            call_args.onContentEscapes(trace_collection)
+        if call_kw is not None:
+            call_kw.onContentEscapes(trace_collection)
+
+        # Any code could be run, note that.
+        trace_collection.onControlFlowEscape(self)
+
+        # Any exception may be raised.
+        trace_collection.onExceptionRaiseExit(BaseException)
+
+        return call_node, None, None
+
     def computeExpressionLen(self, len_node, trace_collection):
         shape = self.getValueShape()
 
@@ -481,7 +518,7 @@ class ExpressionBase(NodeBase):
 
             return makeRaiseTypeErrorExceptionReplacementFromTemplateAndValue(
                 "float() argument must be a string or a number"
-                if Options.is_fullcompat and python_version < 0x300
+                if Options.is_full_compat and python_version < 0x300
                 else "float() argument must be a string or a number, not '%s'",
                 operation="long",
                 original_node=float_node,
@@ -535,7 +572,7 @@ class ExpressionBase(NodeBase):
 
             return makeRaiseTypeErrorExceptionReplacementFromTemplateAndValue(
                 "complex() argument must be a string or a number"
-                if Options.is_fullcompat and python_version < 0x300
+                if Options.is_full_compat and python_version < 0x300
                 else "complex() argument must be a string or a number, not '%s'",
                 operation="complex",
                 original_node=complex_node,
@@ -583,12 +620,13 @@ class ExpressionBase(NodeBase):
         self.onContentEscapes(trace_collection)
 
         # Any code could be run, note that.
-        trace_collection.onControlFlowEscape(self)
+        if self.mayHaveSideEffectsNext():
+            trace_collection.onControlFlowEscape(self)
 
         # Any exception may be raised.
         trace_collection.onExceptionRaiseExit(BaseException)
 
-        return False, (next_node, None, None)
+        return True, (next_node, None, None)
 
     def computeExpressionAsyncIter(self, iter_node, trace_collection):
         self.onContentEscapes(trace_collection)
@@ -679,10 +717,17 @@ class ExpressionBase(NodeBase):
         ):
             trace_collection.onExceptionRaiseExit(BaseException)
 
+        # None indicates no replacement action.
+        return None, None, None
+
     @staticmethod
     def onContentEscapes(trace_collection):
         pass
 
+    @staticmethod
+    def onContentIteratedEscapes(trace_collection):
+        pass
+
     @staticmethod
     def mayRaiseExceptionBool(exception_type):
         """Unless we are told otherwise, everything may raise being checked."""
@@ -773,6 +818,11 @@ class ExpressionBase(NodeBase):
 
         return True
 
+    def mayHaveSideEffectsNext(self):
+        """The type shape tells us, if "next" may execute code."""
+
+        return self.getTypeShape().hasShapeSlotNextCode()
+
     def hasShapeSlotLen(self):
         """The type shape tells us, if "len" is available."""
         return self.getTypeShape().hasShapeSlotLen()
@@ -845,6 +895,15 @@ class ExpressionBase(NodeBase):
         def hasShapeStrOrUnicodeExact(self):
             return self.getTypeShape() is tshape_str
 
+    def hasShapeBytesExact(self):
+        """Does an expression have exactly a bytes shape."""
+        return self.getTypeShape() is tshape_bytes
+
+    @staticmethod
+    def hasVeryTrustedValue():
+        """Trust that value will not be overwritten from the outside."""
+        return False
+
 
 class ExpressionNoSideEffectsMixin(object):
     __slots__ = ()
@@ -1131,17 +1190,15 @@ Predicted '%s' on compiled time constant values."""
     def computeExpressionBool(self, trace_collection):
         constant = self.getCompileTimeConstant()
 
+        # Dealt with through dedicated nodes.
         assert type(constant) is not bool
+        truth_value = bool(constant)
 
-        self.parent.replaceChild(
-            self, makeConstantReplacementNode(bool(constant), self, user_provided=False)
+        result = makeConstantReplacementNode(
+            constant=truth_value, node=self, user_provided=False
         )
 
-        trace_collection.signalChange(
-            tags="new_constant",
-            source_ref=self.source_ref,
-            message="Predicted compile time constant truth value.",
-        )
+        return truth_value, result, "Predicted compile time constant truth value."
 
 
 class ExpressionChildrenHavingBase(ChildrenHavingMixin, ExpressionBase):
@@ -1179,7 +1236,7 @@ class ExpressionChildrenHavingBase(ChildrenHavingMixin, ExpressionBase):
                 )
 
         # Then ask ourselves to work on it.
-        return self.computeExpression(trace_collection=trace_collection)
+        return self.computeExpression(trace_collection)
 
 
 class ExpressionChildHavingBase(ExpressionBase):
@@ -1220,7 +1277,7 @@ class ExpressionChildHavingBase(ExpressionBase):
         value = getattr(self, attr_name)
 
         if value is not None:
-            expression = trace_collection.onExpression(expression=value)
+            expression = trace_collection.onExpression(value)
 
             if expression.willRaiseException(BaseException):
                 return (
@@ -1231,7 +1288,7 @@ class ExpressionChildHavingBase(ExpressionBase):
                 )
 
         # Then ask ourselves to work on it.
-        return self.computeExpression(trace_collection=trace_collection)
+        return self.computeExpression(trace_collection)
 
     def setChild(self, name, value):
         """Set a child value.
@@ -1330,147 +1387,6 @@ class ExpressionChildHavingBase(ExpressionBase):
         return values
 
 
-class ExpressionChildTupleHavingBase(ExpressionBase):
-    checker = None
-
-    def __init__(self, value, source_ref):
-        ExpressionBase.__init__(self, source_ref=source_ref)
-
-        if self.checker is not None:
-            value = self.checker(value)  # False alarm, pylint: disable=not-callable
-
-        assert type(value) is tuple, self.named_child
-        assert None not in value, self.named_child
-
-        for expression in value:
-            expression.parent = self
-
-        attr_name = "subnode_" + self.named_child
-        setattr(self, attr_name, value)
-
-    def finalize(self):
-        del self.parent
-
-        for c in self.getVisitableNodes():
-            c.finalize()
-
-    # TODO: De-duplicate this with multiple child variant.
-    def computeExpressionRaw(self, trace_collection):
-        """Compute an expression.
-
-        Default behavior is to just visit the child expressions first, and
-        then the node "computeExpression". For a few cases this needs to
-        be overloaded, e.g. conditional expressions.
-        """
-        # First apply the sub-expressions, as they are evaluated before.
-        sub_expressions = self.getVisitableNodes()
-
-        for count, sub_expression in enumerate(sub_expressions):
-            assert sub_expression.isExpression(), (self, sub_expression)
-
-            expression = trace_collection.onExpression(expression=sub_expression)
-
-            if expression.willRaiseException(BaseException):
-                wrapped_expression = wrapExpressionWithSideEffects(
-                    side_effects=sub_expressions[:count],
-                    old_node=sub_expression,
-                    new_node=expression,
-                )
-
-                return (
-                    wrapped_expression,
-                    "new_raise",
-                    lambda: "For '%s' the child expression '%s' will raise."
-                    % (self.getChildNameNice(), expression.getChildNameNice()),
-                )
-
-        # Then ask ourselves to work on it.
-        return self.computeExpression(trace_collection=trace_collection)
-
-    def setChild(self, name, value):
-        """Set a child value.
-
-        Do not overload, provide self.checkers instead.
-        """
-        # Only accept legal child name
-        assert name == self.named_child, name
-
-        # Lists as inputs are OK, but turn them into tuples.
-        if type(value) is list:
-            value = tuple(value)
-
-        if self.checker is not None:
-            value = self.checker(value)  # False alarm, pylint: disable=not-callable
-
-        # Re-parent value to us.
-        for val in value:
-            val.parent = self
-
-        attr_name = "subnode_" + name
-        setattr(self, attr_name, value)
-
-    def clearChild(self, name):
-        # We do not do this for tuples, pylint: disable=unused-argument
-        assert False, self.named_child
-
-    def getChild(self, name):
-        # Only accept legal child names
-        assert name == self.named_child, name
-
-        attr_name = "subnode_" + name
-        return getattr(self, attr_name)
-
-    def getVisitableNodes(self):
-        attr_name = "subnode_" + self.named_child
-        return getattr(self, attr_name)
-
-    def getVisitableNodesNamed(self):
-        """Named children items.
-
-        For generic code to use in outputs and code generation.
-        """
-        attr_name = "subnode_" + self.named_child
-        value = getattr(self, attr_name)
-
-        yield self.named_child, value
-
-    def replaceChild(self, old_node, new_node):
-        if new_node is not None and not isinstance(new_node, NodeBase):
-            raise AssertionError(
-                "Cannot replace with", new_node, "old", old_node, "in", self
-            )
-
-        # Find the replaced node, as an added difficulty, what might be
-        # happening, is that the old node is an element of a tuple, in which we
-        # may also remove that element, by setting it to None.
-        attr_name = "subnode_" + self.named_child
-        value = getattr(self, attr_name)
-
-        if old_node not in value:
-            raise AssertionError("Didn't find child", old_node, "in", self)
-
-        if new_node is not None:
-            new_value = tuple(
-                (val if val is not old_node else new_node) for val in value
-            )
-        else:
-            new_value = tuple(val for val in value if val is not old_node)
-
-        new_node.parent = self
-
-        setattr(self, attr_name, new_value)
-
-    def getCloneArgs(self):
-        # Make clones of child nodes too.
-        attr_name = "subnode_" + self.named_child
-        value = getattr(self, attr_name)
-
-        values = {self.named_child: tuple(v.makeClone() for v in value)}
-        values.update(self.getDetails())
-
-        return values
-
-
 class ExpressionSpecBasedComputationMixin(object):
     # Mixins are not allow to specify slots.
     __slots__ = ()
@@ -1490,6 +1406,7 @@ class ExpressionSpecBasedComputationMixin(object):
             computation=lambda: self.builtin_spec.simulateCall(given_values),
             description="Built-in call to '%s' pre-computed."
             % (self.builtin_spec.getName()),
+            user_provided=self.builtin_spec.isUserProvided(given_values),
         )
 
 
