@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -26,6 +26,8 @@ them.
 
 """
 
+from abc import abstractmethod
+
 from nuitka.PythonVersions import python_version
 
 from .CodeObjectSpecs import CodeObjectSpec
@@ -34,7 +36,7 @@ from .StatementNodes import StatementsSequence
 
 
 def checkFrameStatements(value):
-    """ Check that frames statements list value proper.
+    """Check that frames statements list value proper.
 
     Must not be None, must not contain None, and of course only statements
     sequences, or statements, may be empty.
@@ -44,26 +46,18 @@ def checkFrameStatements(value):
     assert None not in value
 
     for statement in value:
-        assert statement.isStatement() or statement.isStatementsFrame(), \
-          statement
+        assert statement.isStatement() or statement.isStatementsFrame(), statement
 
     return tuple(value)
 
 
-
 class StatementsFrameBase(StatementsSequence):
-    checkers = {
-        "statements" : checkFrameStatements
-    }
+    checkers = {"statements": checkFrameStatements}
 
     __slots__ = ("guard_mode", "code_object", "needs_frame_exception_preserve")
 
     def __init__(self, statements, guard_mode, code_object, source_ref):
-        StatementsSequence.__init__(
-            self,
-            statements = statements,
-            source_ref = source_ref
-        )
+        StatementsSequence.__init__(self, statements=statements, source_ref=source_ref)
 
         # TODO: Why not have multiple classes for this.
         self.guard_mode = guard_mode
@@ -76,9 +70,7 @@ class StatementsFrameBase(StatementsSequence):
         return True
 
     def getDetails(self):
-        result = {
-            "code_object" : self.code_object
-        }
+        result = {"code_object": self.code_object}
 
         result.update(StatementsSequence.getDetails(self))
 
@@ -107,17 +99,13 @@ class StatementsFrameBase(StatementsSequence):
 
         code_object = CodeObjectSpec(**code_object_args)
 
-        return cls(
-            code_object = code_object,
-            source_ref  = source_ref,
-            **other_args
-        )
+        return cls(code_object=code_object, source_ref=source_ref, **other_args)
 
     def getGuardMode(self):
         return self.guard_mode
 
     def needsExceptionFramePreservation(self):
-        if python_version < 300:
+        if python_version < 0x300:
             return self.guard_mode != "generator"
         else:
             return True
@@ -126,48 +114,50 @@ class StatementsFrameBase(StatementsSequence):
         return self.code_object.getVarNames()
 
     def updateLocalNames(self):
-        """ For use during variable closure phase. Finalize attributes.
-
-        """
+        """For use during variable closure phase. Finalize attributes."""
         provider = self.getParentVariableProvider()
 
         if not provider.isCompiledPythonModule():
+            if (
+                provider.isExpressionGeneratorObjectBody()
+                or provider.isExpressionCoroutineObjectBody()
+                or provider.isExpressionAsyncgenObjectBody()
+            ):
+                closure_provider = provider.getParentVariableProvider()
+            else:
+                closure_provider = provider
+
+            if closure_provider.isExpressionFunctionBody():
+                closure_variables = closure_provider.getClosureVariables()
+            else:
+                closure_variables = ()
+
             self.code_object.updateLocalNames(
+                [variable.getName() for variable in provider.getLocalVariables()],
                 [
-                    variable.getName() for
-                    variable in
-                    provider.getLocalVariables()
-                ]
+                    variable.getName()
+                    for variable in closure_variables
+                    if variable.getOwner() is not closure_provider
+                ],
             )
 
         entry_point = provider.getEntryPoint()
 
-        is_optimized = not entry_point.isCompiledPythonModule() and \
-                       not entry_point.isExpressionClassBody() and \
-                       not entry_point.isUnoptimized()
+        is_optimized = (
+            not entry_point.isCompiledPythonModule()
+            and not entry_point.isExpressionClassBody()
+            and not entry_point.isUnoptimized()
+        )
 
         self.code_object.setFlagIsOptimizedValue(is_optimized)
 
-        new_locals = not provider.isCompiledPythonModule() and \
-                     (python_version < 340 or (
-                     not provider.isExpressionClassBody() and \
-                     not provider.isUnoptimized()))
+        new_locals = not provider.isCompiledPythonModule() and (
+            python_version < 0x340
+            or (not provider.isExpressionClassBody() and not provider.isUnoptimized())
+        )
 
         self.code_object.setFlagNewLocalsValue(new_locals)
 
-        if provider.isExpressionGeneratorObjectBody() or \
-           provider.isExpressionCoroutineObjectBody() or \
-           provider.isExpressionAsyncgenObjectBody():
-            closure_provider = provider.getParentVariableProvider()
-        else:
-            closure_provider = provider
-
-        has_closure  = closure_provider.isExpressionFunctionBody() and \
-                       closure_provider.getClosureVariables() != () and \
-                       not closure_provider.isExpressionClassBody()
-
-        self.code_object.setFlagHasClosureValue(has_closure)
-
     def markAsFrameExceptionPreserving(self):
         self.needs_frame_exception_preserve = True
 
@@ -182,32 +172,34 @@ class StatementsFrameBase(StatementsSequence):
         # the frame scope, takes it toll to complexity, pylint: disable=too-many-branches
         new_statements = []
 
-        statements = self.getStatements()
+        statements = self.subnode_statements
 
         for count, statement in enumerate(statements):
             # May be frames embedded.
             if statement.isStatementsFrame():
                 new_statement = statement.computeStatementsSequence(
-                    trace_collection = trace_collection
+                    trace_collection=trace_collection
                 )
             else:
-                new_statement = trace_collection.onStatement(
-                    statement = statement
-                )
+                new_statement = trace_collection.onStatement(statement=statement)
 
             if new_statement is not None:
-                if new_statement.isStatementsSequence() and \
-                   not new_statement.isStatementsFrame():
-                    new_statements.extend(new_statement.getStatements())
+                if (
+                    new_statement.isStatementsSequence()
+                    and not new_statement.isStatementsFrame()
+                ):
+                    new_statements.extend(new_statement.subnode_statements)
                 else:
                     new_statements.append(new_statement)
 
-                if statement is not statements[-1] and \
-                   new_statement.isStatementAborting():
+                if (
+                    statement is not statements[-1]
+                    and new_statement.isStatementAborting()
+                ):
                     trace_collection.signalChange(
                         "new_statements",
-                        statements[count+1].getSourceReference(),
-                        "Removed dead statements."
+                        statements[count + 1].getSourceReference(),
+                        "Removed dead statements.",
                     )
 
                     break
@@ -215,8 +207,9 @@ class StatementsFrameBase(StatementsSequence):
         if not new_statements:
             trace_collection.signalChange(
                 "new_statements",
-                self.getSourceReference(),
-                "Removed empty frame object of '%s'." % self.code_object.getCodeObjectName()
+                self.source_ref,
+                "Removed empty frame object of '%s'."
+                % self.code_object.getCodeObjectName(),
             )
 
             return None
@@ -225,51 +218,52 @@ class StatementsFrameBase(StatementsSequence):
         # so do this in two steps. Next time we can reduce the frame scope just
         # as well.
         if statements != tuple(new_statements):
-            self.setStatements(new_statements)
+            self.setChild("statements", new_statements)
             return self
 
         # Determine statements inside the frame, that need not be in a frame,
         # because they wouldn't raise an exception.
         outside_pre = []
-        while new_statements and \
-              not new_statements[0].needsFrame():
+        while new_statements and not new_statements[0].needsFrame():
             outside_pre.append(new_statements[0])
             del new_statements[0]
 
         outside_post = []
-        while new_statements and \
-              not new_statements[-1].needsFrame():
+        while new_statements and not new_statements[-1].needsFrame():
             outside_post.insert(0, new_statements[-1])
             del new_statements[-1]
 
         if outside_pre or outside_post:
-            from .NodeMakingHelpers import makeStatementsSequenceReplacementNode
+            from .NodeMakingHelpers import (
+                makeStatementsSequenceReplacementNode,
+            )
 
             if new_statements:
-                self.setStatements(new_statements)
+                self.setChild("statements", new_statements)
 
                 return makeStatementsSequenceReplacementNode(
-                    statements = outside_pre + [self] + \
-                                 outside_post,
-                    node       = self
+                    statements=outside_pre + [self] + outside_post, node=self
                 )
             else:
                 trace_collection.signalChange(
                     "new_statements",
-                    self.getSourceReference(),
-                    "Removed useless frame object of '%s'." % self.code_object.getCodeObjectName()
+                    self.source_ref,
+                    "Removed useless frame object of '%s'."
+                    % self.code_object.getCodeObjectName(),
                 )
 
                 return makeStatementsSequenceReplacementNode(
-                    statements = outside_pre + outside_post,
-                    node       = self
+                    statements=outside_pre + outside_post, node=self
                 )
         else:
             if statements != new_statements:
-                self.setStatements(new_statements)
+                self.setChild("statements", new_statements)
 
             return self
 
+    @abstractmethod
+    def hasStructureMember(self):
+        """Does the frame have a structure associated, like e.g. generator objects need."""
 
 
 class StatementsFrameModule(StatementsFrameBase):
@@ -278,10 +272,10 @@ class StatementsFrameModule(StatementsFrameBase):
     def __init__(self, statements, code_object, source_ref):
         StatementsFrameBase.__init__(
             self,
-            statements  = statements,
-            code_object = code_object,
-            guard_mode  = "once",
-            source_ref  = source_ref
+            statements=statements,
+            code_object=code_object,
+            guard_mode="once",
+            source_ref=source_ref,
         )
 
     @staticmethod
@@ -295,10 +289,10 @@ class StatementsFrameFunction(StatementsFrameBase):
     def __init__(self, statements, code_object, source_ref):
         StatementsFrameBase.__init__(
             self,
-            statements  = statements,
-            code_object = code_object,
-            guard_mode  = "full",
-            source_ref  = source_ref
+            statements=statements,
+            code_object=code_object,
+            guard_mode="full",
+            source_ref=source_ref,
         )
 
     @staticmethod
@@ -306,16 +300,42 @@ class StatementsFrameFunction(StatementsFrameBase):
         return False
 
 
+class StatementsFrameClass(StatementsFrameBase):
+    kind = "STATEMENTS_FRAME_CLASS"
+
+    __slots__ = ("locals_scope",)
+
+    def __init__(self, statements, code_object, locals_scope, source_ref):
+        StatementsFrameBase.__init__(
+            self,
+            statements=statements,
+            code_object=code_object,
+            # TODO: Guard mode should be derived from where and how it's used, and
+            # should not be static.
+            guard_mode="full",
+            source_ref=source_ref,
+        )
+
+        self.locals_scope = locals_scope
+
+    @staticmethod
+    def hasStructureMember():
+        return False
+
+    def getLocalsScope(self):
+        return self.locals_scope
+
+
 class StatementsFrameGenerator(StatementsFrameBase):
     kind = "STATEMENTS_FRAME_GENERATOR"
 
     def __init__(self, statements, code_object, source_ref):
         StatementsFrameBase.__init__(
             self,
-            statements  = statements,
-            code_object = code_object,
-            guard_mode  = "generator",
-            source_ref  = source_ref
+            statements=statements,
+            code_object=code_object,
+            guard_mode="generator",
+            source_ref=source_ref,
         )
 
     @staticmethod
@@ -329,10 +349,10 @@ class StatementsFrameCoroutine(StatementsFrameBase):
     def __init__(self, statements, code_object, source_ref):
         StatementsFrameBase.__init__(
             self,
-            statements  = statements,
-            code_object = code_object,
-            guard_mode  = "generator",
-            source_ref  = source_ref
+            statements=statements,
+            code_object=code_object,
+            guard_mode="generator",
+            source_ref=source_ref,
         )
 
     @staticmethod
@@ -346,10 +366,10 @@ class StatementsFrameAsyncgen(StatementsFrameBase):
     def __init__(self, statements, code_object, source_ref):
         StatementsFrameBase.__init__(
             self,
-            statements  = statements,
-            code_object = code_object,
-            guard_mode  = "generator",
-            source_ref  = source_ref
+            statements=statements,
+            code_object=code_object,
+            guard_mode="generator",
+            source_ref=source_ref,
         )
 
     @staticmethod
