@@ -200,6 +200,14 @@ class PythonModuleBase(NodeBase):
 
             return result
 
+    @staticmethod
+    def isStatement():
+        return False
+
+    @staticmethod
+    def isExpression():
+        return False
+
 
 class CompiledPythonModule(
     ChildrenHavingMixin,
@@ -384,6 +392,7 @@ class CompiledPythonModule(
         if self.source_code is not None:
             return self.source_code
         else:
+            # This should of course give same result as before.
             return readSourceCodeFromFilename(
                 module_name=self.getFullName(),
                 source_filename=self.getCompileTimeFilename(),
@@ -445,6 +454,10 @@ class CompiledPythonModule(
 
         return encodePythonIdentifierToC(self.getFullName())
 
+    @staticmethod
+    def getChildQualname(function_name):
+        return function_name
+
     def addFunction(self, function_body):
         functions = self.subnode_functions
         assert function_body not in functions
@@ -457,6 +470,7 @@ class CompiledPythonModule(
 
     def restartTraversal(self):
         self.visited_functions = set()
+        self.used_modules = None
 
     def getUsedModules(self):
         if self.used_modules is None:
