@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -29,10 +29,7 @@ from nuitka import PythonOperators
 from nuitka.Errors import NuitkaAssumptionError
 from nuitka.PythonVersions import python_version
 
-from .ExpressionBases import (
-    ExpressionChildHavingBase,
-    ExpressionChildrenHavingBase,
-)
+from .ExpressionBases import ExpressionChildrenHavingBase
 from .NodeMakingHelpers import (
     makeRaiseExceptionReplacementExpressionFromInstance,
     wrapExpressionWithSideEffects,
@@ -41,6 +38,7 @@ from .shapes.BuiltinTypeShapes import tshape_bool, tshape_int_or_long
 from .shapes.StandardShapes import (
     ShapeLargeConstantValue,
     ShapeLargeConstantValuePredictable,
+    tshape_unknown,
     vshape_unknown,
 )
 
@@ -48,6 +46,9 @@ from .shapes.StandardShapes import (
 class ExpressionPropertiesFromTypeShapeMixin(object):
     """Given a self.type_shape, this can derive default properties from there."""
 
+    # Mixins are required to slots
+    __slots__ = ()
+
     def isKnownToBeHashable(self):
         return self.type_shape.hasShapeSlotHash()
 
@@ -57,22 +58,24 @@ class ExpressionOperationBinaryBase(
 ):
     """Base class for all binary operation expression."""
 
+    __slots__ = ("type_shape", "escape_desc", "inplace_suspect", "shape")
+
     named_children = ("left", "right")
     nice_children = tuple(child_name + " operand" for child_name in named_children)
 
-    shape = vshape_unknown
-
     def __init__(self, left, right, source_ref):
         ExpressionChildrenHavingBase.__init__(
             self, values={"left": left, "right": right}, source_ref=source_ref
         )
 
-        self.type_shape = None
+        # Question might be asked early on, later this is cached from last computation.
+        self.type_shape = tshape_unknown
+
         self.escape_desc = None
 
-    @staticmethod
-    def getDetails():
-        return {}
+        self.inplace_suspect = False
+
+        self.shape = vshape_unknown
 
     @staticmethod
     def isExpressionOperationBinary():
@@ -81,12 +84,10 @@ class ExpressionOperationBinaryBase(
     def getOperator(self):
         return self.operator
 
-    inplace_suspect = False
-
     def markAsInplaceSuspect(self):
         self.inplace_suspect = True
 
-    def unmarkAsInplaceSuspect(self):
+    def removeMarkAsInplaceSuspect(self):
         self.inplace_suspect = False
 
     def isInplaceSuspect(self):
@@ -96,9 +97,7 @@ class ExpressionOperationBinaryBase(
         return (self.subnode_left, self.subnode_right)
 
     def mayRaiseExceptionOperation(self):
-        return (
-            self.escape_desc is None or self.escape_desc.getExceptionExit() is not None
-        )
+        return self.escape_desc.getExceptionExit() is not None
 
     def mayRaiseException(self, exception_type):
         # TODO: Match getExceptionExit() more precisely against exception type given
@@ -110,12 +109,6 @@ class ExpressionOperationBinaryBase(
         )
 
     def getTypeShape(self):
-        # Question might be asked early on, later this is cached from last computation.
-        if self.type_shape is None:
-            self.type_shape, self.escape_desc = self._getOperationShape(
-                self.subnode_left.getTypeShape(), self.subnode_right.getTypeShape()
-            )
-
         return self.type_shape
 
     @abstractmethod
@@ -239,6 +232,9 @@ class ExpressionOperationBinaryBase(
 
 
 class ExpressionOperationAddMixin(object):
+    # Mixins are not allow to specify slots, pylint: disable=assigning-non-slot
+    __slots__ = ()
+
     def getValueShape(self):
         return self.shape
 
@@ -268,6 +264,11 @@ class ExpressionOperationBinaryAdd(
 ):
     kind = "EXPRESSION_OPERATION_BINARY_ADD"
 
+    def __init__(self, left, right, source_ref):
+        ExpressionOperationBinaryBase.__init__(
+            self, left=left, right=right, source_ref=source_ref
+        )
+
     operator = "Add"
     simulator = PythonOperators.binary_operator_functions[operator]
 
@@ -288,6 +289,9 @@ class ExpressionOperationBinarySub(ExpressionOperationBinaryBase):
 
 
 class ExpressionOperationMultMixin(object):
+    # Mixins are not allow to specify slots, pylint: disable=assigning-non-slot
+    __slots__ = ()
+
     def getValueShape(self):
         return self.shape
 
@@ -359,6 +363,11 @@ class ExpressionOperationBinaryMult(
     operator = "Mult"
     simulator = PythonOperators.binary_operator_functions[operator]
 
+    def __init__(self, left, right, source_ref):
+        ExpressionOperationBinaryBase.__init__(
+            self, left=left, right=right, source_ref=source_ref
+        )
+
     @staticmethod
     def _getOperationShape(left_shape, right_shape):
         return left_shape.getOperationBinaryMultShape(right_shape)
@@ -481,7 +490,38 @@ class ExpressionOperationBinaryPow(ExpressionOperationBinaryBase):
         return left_shape.getOperationBinaryPowShape(right_shape)
 
 
-class ExpressionOperationBinaryLshift(ExpressionOperationBinaryBase):
+class ExpressionOperationLshiftMixin(object):
+    # Mixins are not allow to specify slots, pylint: disable=assigning-non-slot
+    __slots__ = ()
+
+    def getValueShape(self):
+        return self.shape
+
+    def _isTooLarge(self):
+        if self.subnode_right.isNumberConstant():
+            if self.subnode_left.isNumberConstant():
+                # Estimate with logarithm, if the result of number
+                # calculations is computable with acceptable effort,
+                # otherwise, we will have to do it at runtime.
+                left_value = self.subnode_left.getCompileTimeConstant()
+
+                if left_value != 0:
+                    right_value = self.subnode_right.getCompileTimeConstant()
+
+                    # More than a typical shift, most likely a stupid test.
+                    if right_value > 64:
+                        self.shape = ShapeLargeConstantValue(
+                            size=None, shape=tshape_int_or_long
+                        )
+
+                        return True
+
+        return False
+
+
+class ExpressionOperationBinaryLshift(
+    ExpressionOperationLshiftMixin, ExpressionOperationBinaryBase
+):
     kind = "EXPRESSION_OPERATION_BINARY_LSHIFT"
 
     operator = "LShift"
@@ -538,9 +578,7 @@ class ExpressionOperationBinaryBitXor(ExpressionOperationBinaryBase):
 
 if python_version >= 0x350:
 
-    class ExpressionOperationBinaryMatMult(
-        ExpressionOperationMultMixin, ExpressionOperationBinaryBase
-    ):
+    class ExpressionOperationBinaryMatMult(ExpressionOperationBinaryBase):
         kind = "EXPRESSION_OPERATION_BINARY_MAT_MULT"
 
         operator = "MatMult"
@@ -580,171 +618,17 @@ def makeBinaryOperationNode(operator, left, right, source_ref):
     return node_class(left=left, right=right, source_ref=source_ref)
 
 
-class ExpressionOperationUnaryBase(ExpressionChildHavingBase):
-    named_child = "operand"
-
-    __slots__ = ("operator", "simulator")
-
-    def __init__(self, operator, operand, source_ref):
-        ExpressionChildHavingBase.__init__(self, value=operand, source_ref=source_ref)
-
-        self.operator = operator
-
-        self.simulator = PythonOperators.unary_operator_functions[operator]
-
-    def getDetails(self):
-        return {"operator": self.operator}
-
-    def getOperator(self):
-        return self.operator
-
-    def computeExpression(self, trace_collection):
-        operator = self.getOperator()
-        operand = self.subnode_operand
-
-        if operand.isCompileTimeConstant():
-            operand_value = operand.getCompileTimeConstant()
-
-            return trace_collection.getCompileTimeComputationResult(
-                node=self,
-                computation=lambda: self.simulator(operand_value),
-                description="Operator '%s' with constant argument." % operator,
-            )
-        else:
-            # TODO: May go down to MemoryError for compile time constant overflow
-            # ones.
-            trace_collection.onExceptionRaiseExit(BaseException)
-
-            # The value of that node escapes and could change its contents.
-            # TODO: The unary operations don't do much to an operator, add
-            # methods, that don't do stuff on common types though.
-            # trace_collection.onValueEscapeSomeUnaryOperator(operand)
-
-            # Any code could be run, note that.
-            trace_collection.onControlFlowEscape(self)
-
-            return self, None, None
-
-    def getOperands(self):
-        return (self.subnode_operand,)
-
-    @staticmethod
-    def isExpressionOperationUnary():
-        return True
-
-
-class ExpressionOperationUnary(ExpressionOperationUnaryBase):
-    kind = "EXPRESSION_OPERATION_UNARY"
-
-    def __init__(self, operator, operand, source_ref):
-        assert operand.isExpression(), operand
-
-        ExpressionOperationUnaryBase.__init__(
-            self, operator=operator, operand=operand, source_ref=source_ref
-        )
-
-
-class ExpressionOperationNot(ExpressionOperationUnaryBase):
-    kind = "EXPRESSION_OPERATION_NOT"
-
-    def __init__(self, operand, source_ref):
-        ExpressionOperationUnaryBase.__init__(
-            self, operator="Not", operand=operand, source_ref=source_ref
-        )
-
-    @staticmethod
-    def getTypeShape():
-        return tshape_bool
-
-    @staticmethod
-    def getDetails():
-        return {}
-
-    def computeExpression(self, trace_collection):
-        return self.subnode_operand.computeExpressionOperationNot(
-            not_node=self, trace_collection=trace_collection
-        )
-
-    def mayRaiseException(self, exception_type):
-        return self.subnode_operand.mayRaiseException(
-            exception_type
-        ) or self.subnode_operand.mayRaiseExceptionBool(exception_type)
-
-    def mayRaiseExceptionBool(self, exception_type):
-        return self.subnode_operand.mayRaiseExceptionBool(exception_type)
-
-    def getTruthValue(self):
-        result = self.subnode_operand.getTruthValue()
-
-        # Need to invert the truth value of operand of course here.
-        return None if result is None else not result
-
-    def mayHaveSideEffects(self):
-        operand = self.subnode_operand
-
-        if operand.mayHaveSideEffects():
-            return True
-
-        return operand.mayHaveSideEffectsBool()
-
-    def mayHaveSideEffectsBool(self):
-        return self.subnode_operand.mayHaveSideEffectsBool()
-
-    def extractSideEffects(self):
-        operand = self.subnode_operand
-
-        # TODO: Find the common ground of these, and make it an expression
-        # method.
-        if operand.isExpressionMakeSequence():
-            return operand.extractSideEffects()
-
-        if operand.isExpressionMakeDict():
-            return operand.extractSideEffects()
-
-        return (self,)
-
-
-class ExpressionOperationAbs(ExpressionOperationUnaryBase):
-    kind = "EXPRESSION_OPERATION_ABS"
-
-    def __init__(self, operand, source_ref):
-        ExpressionOperationUnaryBase.__init__(
-            self, operator="Abs", operand=operand, source_ref=source_ref
-        )
-
-    def computeExpression(self, trace_collection):
-        return self.subnode_operand.computeExpressionAbs(
-            abs_node=self, trace_collection=trace_collection
-        )
-
-    def mayRaiseException(self, exception_type):
-        operand = self.subnode_operand
-
-        if operand.mayRaiseException(exception_type):
-            return True
-
-        return operand.mayRaiseExceptionAbs(exception_type)
-
-    def mayHaveSideEffects(self):
-        operand = self.subnode_operand
-
-        if operand.mayHaveSideEffects():
-            return True
-
-        return operand.mayHaveSideEffectsAbs()
-
-
 class ExpressionOperationBinaryInplaceBase(ExpressionOperationBinaryBase):
     # Base classes can be abstract, pylint: disable=abstract-method
     """Base class for all inplace operations."""
 
-    inplace_suspect = True
-
     def __init__(self, left, right, source_ref):
         ExpressionOperationBinaryBase.__init__(
             self, left=left, right=right, source_ref=source_ref
         )
 
+        self.inplace_suspect = True
+
     @staticmethod
     def isExpressionOperationInplace():
         return True
@@ -824,6 +708,11 @@ class ExpressionOperationInplaceAdd(
     operator = "IAdd"
     simulator = PythonOperators.binary_operator_functions[operator]
 
+    def __init__(self, left, right, source_ref):
+        ExpressionOperationBinaryInplaceBase.__init__(
+            self, left=left, right=right, source_ref=source_ref
+        )
+
     @staticmethod
     def _getOperationShape(left_shape, right_shape):
         return left_shape.getOperationInplaceAddShape(right_shape)
