@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -364,3 +364,6 @@ class ExpressionOutlineFunction(ExpressionOutlineFunctionBase):
         self.locals_scope = getLocalsDictHandle(
             "locals_%s" % self.getCodeName(), "python_function", self
         )
+
+    def getChildQualname(self, function_name):
+        return self.provider.getChildQualname(function_name)
