@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -58,12 +58,18 @@ class StatementsSequence(StatementChildHavingBase):
         for s in self.subnode_statements:
             s.finalize()
 
-    # Overloading name based automatic check, so that derived ones know it too.
-    def isStatementsSequence(self):
-        # Virtual method, pylint: disable=no-self-use
-
+    @staticmethod
+    def isStatementsSequence():
         return True
 
+    @staticmethod
+    def isStatement():
+        return False
+
+    @staticmethod
+    def isExpression():
+        return False
+
     def trimStatements(self, statement):
         assert statement.parent is self
 
@@ -147,6 +153,9 @@ class StatementsSequence(StatementChildHavingBase):
     def isStatementAborting(self):
         return self.subnode_statements[-1].isStatementAborting()
 
+    def willRaiseException(self, exception_type):
+        return self.subnode_statements[-1].willRaiseException(exception_type)
+
     def computeStatement(self, trace_collection):
         # Don't want to be called like this.
         assert False, self
@@ -220,7 +229,7 @@ class StatementExpressionOnly(StatementChildHavingBase):
         return self.subnode_expression.mayRaiseException(exception_type)
 
     def computeStatement(self, trace_collection):
-        expression = trace_collection.onExpression(expression=self.subnode_expression)
+        expression = trace_collection.onExpression(self.subnode_expression)
 
         return expression.computeExpressionDrop(
             statement=self, trace_collection=trace_collection
