@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -49,6 +49,8 @@ class StatementTry(StatementChildrenHavingBase):
         "return_handler": checkStatementsSequenceOrNone,
     }
 
+    __slots__ = ("tried_may_raise",)
+
     def __init__(
         self,
         tried,
@@ -70,6 +72,11 @@ class StatementTry(StatementChildrenHavingBase):
             source_ref=source_ref,
         )
 
+        self.tried_may_raise = None
+
+    def getDetailsForDisplay(self):
+        return {"aborting": self.isStatementAborting()}
+
     def computeStatement(self, trace_collection):
         # This node has many children to handle, pylint: disable=I0021,too-many-branches,too-many-locals,too-many-statements
         tried = self.subnode_tried
@@ -111,9 +118,13 @@ class StatementTry(StatementChildrenHavingBase):
             return_collections = trace_collection.getFunctionReturnCollections()
             exception_collections = trace_collection.getExceptionRaiseCollections()
 
-        tried_may_raise = tried.mayRaiseException(BaseException)
+        # Not raising never turns into raising, but None (never calculated) and True
+        # may no longer be true, but not raising never becomes raising.
+        if self.tried_may_raise is not False:
+            self.tried_may_raise = tried.mayRaiseException(BaseException)
+
         # Exception handling is useless if no exception is to be raised.
-        if not tried_may_raise:
+        if not self.tried_may_raise:
             if except_handler is not None:
                 except_handler.finalize()
 
@@ -128,7 +139,7 @@ class StatementTry(StatementChildrenHavingBase):
 
         # If tried may raise, even empty exception handler has a meaning to
         # ignore that exception.
-        if tried_may_raise:
+        if self.tried_may_raise:
             collection_exception_handling = TraceCollectionBranch(
                 parent=collection_start, name="except handler"
             )
@@ -240,17 +251,22 @@ class StatementTry(StatementChildrenHavingBase):
 
         # Merge exception handler only if it is used. Empty means it is not
         # aborting, as it swallows the exception.
-        if tried_may_raise and (
+        if self.tried_may_raise and (
             except_handler is None or not except_handler.isStatementAborting()
         ):
-            trace_collection.mergeBranches(
-                collection_yes=collection_exception_handling, collection_no=None
-            )
+            if tried.isStatementAborting():
+                trace_collection.variable_actives = (
+                    collection_exception_handling.variable_actives
+                )
+            else:
+                trace_collection.mergeBranches(
+                    collection_yes=collection_exception_handling, collection_no=None
+                )
 
         # An empty exception handler means we have to swallow exception.
         if (
             (
-                not tried_may_raise
+                not self.tried_may_raise
                 or (
                     except_handler is not None
                     and except_handler.subnode_statements[
@@ -262,7 +278,11 @@ class StatementTry(StatementChildrenHavingBase):
             and continue_handler is None
             and return_handler is None
         ):
-            return tried, "new_statements", "Removed useless try, all handlers removed."
+            return (
+                tried,
+                "new_statements",
+                "Removed useless try, all handlers became empty.",
+            )
 
         tried_statements = tried.subnode_statements
 
@@ -353,10 +373,11 @@ class StatementTry(StatementChildrenHavingBase):
         if self.subnode_tried.mayReturn():
             return True
 
-        except_handler = self.subnode_except_handler
+        if self.tried_may_raise is not False:
+            except_handler = self.subnode_except_handler
 
-        if except_handler is not None and except_handler.mayReturn():
-            return True
+            if except_handler is not None and except_handler.mayReturn():
+                return True
 
         break_handler = self.subnode_break_handler
 
@@ -381,10 +402,11 @@ class StatementTry(StatementChildrenHavingBase):
         if self.subnode_tried.mayBreak():
             return True
 
-        except_handler = self.subnode_except_handler
+        if self.tried_may_raise is not False:
+            except_handler = self.subnode_except_handler
 
-        if except_handler is not None and except_handler.mayBreak():
-            return True
+            if except_handler is not None and except_handler.mayBreak():
+                return True
 
         break_handler = self.subnode_break_handler
 
@@ -409,10 +431,11 @@ class StatementTry(StatementChildrenHavingBase):
         if self.subnode_tried.mayContinue():
             return True
 
-        except_handler = self.subnode_except_handler
+        if self.tried_may_raise is not False:
+            except_handler = self.subnode_except_handler
 
-        if except_handler is not None and except_handler.mayContinue():
-            return True
+            if except_handler is not None and except_handler.mayContinue():
+                return True
 
         break_handler = self.subnode_break_handler
 
@@ -432,10 +455,11 @@ class StatementTry(StatementChildrenHavingBase):
         return False
 
     def isStatementAborting(self):
-        except_handler = self.subnode_except_handler
+        if self.tried_may_raise is not False:
+            except_handler = self.subnode_except_handler
 
-        if except_handler is None or not except_handler.isStatementAborting():
-            return False
+            if except_handler is None or not except_handler.isStatementAborting():
+                return False
 
         break_handler = self.subnode_break_handler
 
@@ -455,9 +479,7 @@ class StatementTry(StatementChildrenHavingBase):
         return self.subnode_tried.isStatementAborting()
 
     def mayRaiseException(self, exception_type):
-        tried = self.subnode_tried
-
-        if tried.mayRaiseException(exception_type):
+        if self.tried_may_raise is not False:
             except_handler = self.subnode_except_handler
 
             if except_handler is not None and except_handler.mayRaiseException(
@@ -489,10 +511,11 @@ class StatementTry(StatementChildrenHavingBase):
         return False
 
     def needsFrame(self):
-        except_handler = self.subnode_except_handler
+        if self.tried_may_raise is not False:
+            except_handler = self.subnode_except_handler
 
-        if except_handler is not None and except_handler.needsFrame():
-            return True
+            if except_handler is not None and except_handler.needsFrame():
+                return True
 
         break_handler = self.subnode_break_handler
 
