@@ -24,7 +24,10 @@ and its expressions, changing the meaning of course dramatically.
 
 from nuitka import Builtins, Variables
 from nuitka.ModuleRegistry import getOwnerFromCodeName
-from nuitka.PythonVersions import python_version
+from nuitka.PythonVersions import (
+    getUnboundLocalErrorErrorTemplate,
+    python_version,
+)
 from nuitka.tree.TreeHelpers import makeStatementsSequenceFromStatements
 
 from .ConstantRefNodes import makeConstantRefNode
@@ -419,7 +422,9 @@ class ExpressionVariableRef(ExpressionVariableRefBase):
             # could be decided from context.
             trace_collection.onExceptionRaiseExit(BaseException)
 
-        if variable.isModuleVariable() and variable.hasDefiniteWrites() is False:
+        if variable.isModuleVariable() and (
+            variable.hasDefiniteWrites() is False or variable.getName() == "super"
+        ):
             variable_name = self.variable.getName()
 
             if variable_name in Builtins.builtin_exception_names:
@@ -519,8 +524,7 @@ Replaced read-only module attribute '__spec__' with module attribute reference."
             result = makeRaiseExceptionReplacementExpression(
                 expression=self,
                 exception_type="UnboundLocalError",
-                exception_value="""local variable '%s' referenced before assignment"""
-                % variable_name,
+                exception_value=getUnboundLocalErrorErrorTemplate() % variable_name,
             )
 
             return (
@@ -581,7 +585,11 @@ Replaced read-only module attribute '__spec__' with module attribute reference."
                     attribute_node.isCompileTimeConstant()
                     and not attribute_node.isMutable()
                 ):
-                    return attribute_node.computeExpressionBool(trace_collection)
+                    return (
+                        bool(attribute_node.getCompileTimeConstant()),
+                        attribute_node.makeClone(),
+                        "Using very trusted constant truth value.",
+                    )
 
         # TODO: This is probably only default stuff here, that could be compressed.
         if not self.mayRaiseException(BaseException) and self.mayRaiseExceptionBool(
@@ -589,7 +597,7 @@ Replaced read-only module attribute '__spec__' with module attribute reference."
         ):
             trace_collection.onExceptionRaiseExit(BaseException)
 
-        return None, None
+        return None, None, None
 
     def hasShapeDictionaryExact(self):
         return (
