@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -180,6 +180,14 @@ class ShapeNumberMixin(object):
 
 
 class ShapeIteratorMixin(ShapeNotContainerMixin):
+    @staticmethod
+    def isShapeIterator():
+        return True
+
+    @staticmethod
+    def getIteratedShape():
+        return None
+
     @staticmethod
     def hasShapeSlotIter():
         return True
@@ -188,6 +196,15 @@ class ShapeIteratorMixin(ShapeNotContainerMixin):
     def hasShapeSlotNext():
         return True
 
+    @staticmethod
+    def hasShapeSlotNextCode():
+        """Does next execute code, i.e. control flow escaped.
+
+        For most known iterators that is not the case, only the generic
+        tshape_iterator needs to say "do not know", aka None.
+        """
+        return False
+
     @staticmethod
     def hasShapeSlotContains():
         return True
