@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -21,21 +21,18 @@ These are not the graphs you might be thinking of. This is for rending the
 progress of optimization into images.
 """
 
-from logging import warning
-
 from nuitka import Options
 from nuitka.ModuleRegistry import getDoneModules
+from nuitka.Tracing import general
 
 graph = None
 computation_counters = {}
 
 progressive = False
 
+
 def _addModuleGraph(module, desc):
-    module_graph = module.asGraph(
-        graph,
-        desc
-    )
+    module_graph = module.asGraph(graph, desc)
 
     return module_graph
 
@@ -55,11 +52,12 @@ def startGraph():
 
     if Options.shallCreateGraph():
         try:
-            from pygraphviz import AGraph # pylint: disable=I0021,import-error
-            graph = AGraph(name = "Optimization", directed = True)
+            from pygraphviz import AGraph  # pylint: disable=I0021,import-error
+
+            graph = AGraph(name="Optimization", directed=True)
             graph.layout()
         except ImportError:
-            warning("Cannot import pygraphviz module, no graphing capability.")
+            general.sysexit("Cannot import pygraphviz module, no graphing capability.")
 
 
 def endGraph(output_filename):
@@ -67,4 +65,4 @@ def endGraph(output_filename):
         for module in getDoneModules():
             _addModuleGraph(module, "final")
 
-        graph.draw(output_filename + ".dot", prog = "dot")
+        graph.draw(output_filename + ".dot", prog="dot")
