@@ -44,6 +44,7 @@ from nuitka.nodes.shapes.StandardShapes import (
     tshape_uninitialized,
     tshape_unknown,
 )
+from nuitka.Tracing import my_print
 from nuitka.utils.InstanceCounters import (
     counted_del,
     counted_init,
@@ -88,6 +89,9 @@ class ValueTraceBase(object):
     def __repr__(self):
         return "<%s of %s>" % (self.__class__.__name__, self.owner.getCodeName())
 
+    def dump(self, indent):
+        my_print("%s%s %s:" % (indent, self.__class__.__name__, id(self)))
+
     def getOwner(self):
         return self.owner
 
@@ -148,6 +152,10 @@ class ValueTraceBase(object):
     def isUnknownTrace():
         return False
 
+    @staticmethod
+    def isUnknownOrVeryTrustedTrace():
+        return False
+
     @staticmethod
     def isEscapeTrace():
         return False
@@ -272,9 +280,6 @@ class ValueTraceUninitialized(ValueTraceUnassignedBase):
     def isTraceThatNeedsEscape():
         return False
 
-    def dump(self, indent):
-        print("%s%s %s:" % (indent, self.__class__.__name__, id(self)))
-
 
 class ValueTraceDeleted(ValueTraceUnassignedBase):
     """Trace caused by a deletion."""
@@ -385,6 +390,10 @@ class ValueTraceUnknown(ValueTraceBase):
     def isUnknownTrace():
         return True
 
+    @staticmethod
+    def isUnknownOrVeryTrustedTrace():
+        return True
+
     @staticmethod
     def isTraceThatNeedsEscape():
         return False
@@ -444,6 +453,10 @@ class ValueTraceEscaped(ValueTraceUnknown):
     def isUnknownTrace():
         return False
 
+    @staticmethod
+    def isUnknownOrVeryTrustedTrace():
+        return False
+
     @staticmethod
     def isEscapeTrace():
         return True
@@ -528,11 +541,9 @@ class ValueTraceAssign(ValueTraceBase):
             return None
 
     def getAttributeNodeVeryTrusted(self):
-        source_node = self.assign_node.subnode_source
-
         # Hard imports typically.
-        if source_node.hasVeryTrustedValue():
-            return source_node
+        if self.assign_node.hasVeryTrustedValue():
+            return self.assign_node.subnode_source
         else:
             return None
 
@@ -546,6 +557,12 @@ class ValueTraceAssignUnescapable(ValueTraceAssign):
         return False
 
 
+class ValueTraceAssignVeryTrusted(ValueTraceAssignUnescapable):
+    @staticmethod
+    def isUnknownOrVeryTrustedTrace():
+        return True
+
+
 class ValueTraceAssignUnescapablePropagated(ValueTraceAssignUnescapable):
     """Assignment from value where it is not that escaping doesn't matter."""
 
@@ -588,7 +605,7 @@ class ValueTraceMergeBase(ValueTraceBase):
         self.merge_usage_count += 1
 
     def dump(self, indent):
-        print("%s%s %s:" % (indent, self.__class__.__name__, id(self)))
+        ValueTraceBase.dump(self, indent)
 
         for trace in self.previous:
             trace.dump(indent + "  ")
