@@ -29,12 +29,14 @@ it being used.
 import ast
 import functools
 import inspect
+import os
 import sys
 from collections import namedtuple
 
 from nuitka.__past__ import getMetaClassBase
 from nuitka.containers.OrderedSets import OrderedSet
 from nuitka.freezer.IncludedDataFiles import (
+    decodeDataFileTags,
     makeIncludedDataDirectory,
     makeIncludedDataFile,
     makeIncludedEmptyDirectory,
@@ -49,7 +51,12 @@ from nuitka.ModuleRegistry import (
     addModuleInfluencingCondition,
     getModuleInclusionInfoByName,
 )
-from nuitka.Options import isStandaloneMode, shallMakeModule
+from nuitka.Options import (
+    isStandaloneMode,
+    shallCreateAppBundle,
+    shallMakeModule,
+    shallShowExecutedCommands,
+)
 from nuitka.PythonFlavors import isAnacondaPython, isDebianPackagePython
 from nuitka.PythonVersions import getSupportedPythonVersions, python_version
 from nuitka.Tracing import plugins_logger
@@ -61,7 +68,12 @@ from nuitka.utils.ModuleNames import (
     pre_module_load_trigger_name,
 )
 from nuitka.utils.SharedLibraries import locateDLL, locateDLLsInDirectory
-from nuitka.utils.Utils import isLinux, isMacOS, isWin32Windows
+from nuitka.utils.Utils import (
+    getArchitecture,
+    isLinux,
+    isMacOS,
+    isWin32Windows,
+)
 
 _warned_unused_plugins = set()
 
@@ -77,6 +89,15 @@ def _convertVersionToTuple(version_str):
     return tuple(numberize(d) for d in version_str.split("."))
 
 
+def _getPackageNameFromDistributionName(distribution_name):
+    if distribution_name == "opencv-python":
+        return "cv2"
+    elif distribution_name == "pyobjc":
+        return "objc"
+    else:
+        return distribution_name
+
+
 def _getPackageVersion(distribution_name):
     if distribution_name not in _package_versions:
         try:
@@ -88,23 +109,40 @@ def _getPackageVersion(distribution_name):
             result = _convertVersionToTuple(version(distribution_name))
         except ImportError:
             try:
-                from pkg_resources import get_distribution
-
-                result = _convertVersionToTuple(
-                    get_distribution(distribution_name).version
+                from pkg_resources import (
+                    DistributionNotFound,
+                    get_distribution,
                 )
             except ImportError:
                 # Fallback if nothing is available, which may happen if no package is installed,
                 # but only source code is found.
-                result = _convertVersionToTuple(
-                    __import__(distribution_name).__version__.split(".")
-                )
+                try:
+                    result = _convertVersionToTuple(
+                        __import__(
+                            _getPackageNameFromDistributionName(distribution_name)
+                        ).__version__
+                    )
+                except ImportError:
+                    result = None
+            else:
+                try:
+                    result = _convertVersionToTuple(
+                        get_distribution(distribution_name).version
+                    )
+                except DistributionNotFound:
+                    result = None
 
         _package_versions[distribution_name] = result
 
     return _package_versions[distribution_name]
 
 
+def _isPluginActive(plugin_name):
+    from .Plugins import getUserActivatedPluginNames
+
+    return plugin_name in getUserActivatedPluginNames()
+
+
 class NuitkaPluginBase(getMetaClassBase("Plugin")):
     """Nuitka base class for all plugins.
 
@@ -188,26 +226,6 @@ class NuitkaPluginBase(getMetaClassBase("Plugin")):
         # Call group.add_option() here.
         pass
 
-    @classmethod
-    def getPluginDefaultOptionValues(cls):
-        """This method is used to get a values to use as defaults.
-
-        Since the defaults are in the command line options, we call
-        that and extract them.
-        """
-
-        from optparse import OptionGroup, OptionParser
-
-        parser = OptionParser()
-        group = OptionGroup(parser, "Pseudo Target")
-        cls.addPluginCommandLineOptions(group)
-
-        result = {}
-        for option in group.option_list:
-            result[option.dest] = option.default
-
-        return result
-
     def isRequiredImplicitImport(self, module, full_name):
         """Indicate whether an implicitly imported module should be accepted.
 
@@ -438,6 +456,39 @@ class NuitkaPluginBase(getMetaClassBase("Plugin")):
             prepare.
         """
 
+    def onModuleCompleteSetGUI(self, module_set, plugin_binding_name):
+        from .Plugins import getOtherGUIBindingNames, getQtBindingNames
+
+        for module in module_set:
+            module_name = module.getFullName()
+
+            if module_name == plugin_binding_name:
+                continue
+
+            if module_name in getOtherGUIBindingNames():
+                if plugin_binding_name in getQtBindingNames():
+                    recommendation = "Use '--nofollow-import-to=%s'" % module_name
+
+                    if module_name in getQtBindingNames():
+                        problem = "conflicts with"
+                    else:
+                        problem = "is redundant with"
+                else:
+                    recommendation = "Use '--enable-plugin=no-qt'"
+                    problem = "is redundant with"
+
+                self.warning(
+                    """\
+Unwanted import of '%(unwanted)s' that %(problem)s '%(binding_name)s' encountered. \
+%(recommendation)s or uninstall it for best compatibility with pure Python execution."""
+                    % {
+                        "unwanted": module_name,
+                        "binding_name": plugin_binding_name,
+                        "recommendation": recommendation,
+                        "problem": problem,
+                    }
+                )
+
     @staticmethod
     def locateModule(module_name):
         """Provide a filename / -path for a to-be-imported module.
@@ -509,7 +560,7 @@ class NuitkaPluginBase(getMetaClassBase("Plugin")):
             msg = "Found %d %s DLLs from %s%s installation." % (
                 count,
                 "file" if count < 2 else "files",
-                "" if not section else (" '%s' " % section),
+                "" if not section else ("'%s' " % section),
                 module_name.asString(),
             )
 
@@ -587,6 +638,24 @@ class NuitkaPluginBase(getMetaClassBase("Plugin")):
             tags=tags,
         )
 
+    def makeIncludedAppBundleResourceFile(
+        self, source_path, dest_path, reason, tags=""
+    ):
+        tags = decodeDataFileTags(tags)
+        tags.add("framework_resource")
+
+        assert isMacOS() and shallCreateAppBundle()
+
+        # The default dest path root is the "Contents" folder
+        dest_path = os.path.join("..", "Resources", dest_path)
+
+        return self.makeIncludedDataFile(
+            source_path=source_path,
+            dest_path=dest_path,
+            reason=reason,
+            tags=tags,
+        )
+
     def makeIncludedGeneratedDataFile(self, data, dest_path, reason, tags=""):
         return makeIncludedGeneratedDataFile(
             data=data, dest_path=dest_path, reason=reason, tracer=self, tags=tags
@@ -764,7 +833,20 @@ class NuitkaPluginBase(getMetaClassBase("Plugin")):
             None for no defines, otherwise dictionary of key to be
             defined, and non-None values if any, i.e. no "-Dkey" only
         """
+        # Virtual method, pylint: disable=no-self-use
+        # spell-checker: ignore -Dkey
+        return None
 
+    def getBuildDefinitions(self):
+        """Decide C source defines to be used in compilation.
+
+        Notes:
+            Make sure to use a namespace for your defines, and prefer
+            `getPreprocessorSymbols` if you can.
+
+        Returns:
+            dict or None for no values
+        """
         # Virtual method, pylint: disable=no-self-use
         return None
 
@@ -881,23 +963,29 @@ class NuitkaPluginBase(getMetaClassBase("Plugin")):
             query_codes.append('print("-" * 27)')
 
         if type(setup_codes) is str:
-            setup_codes = setup_codes.split("\n")
+            setup_codes = setup_codes.splitlines()
+
+        if not setup_codes:
+            setup_codes = ["pass"]
 
         cmd = r"""\
 from __future__ import print_function
 from __future__ import absolute_import
 
 try:
-    %(setup_codes)s
+%(setup_codes)s
 except ImportError:
     import sys
     sys.exit(38)
 %(query_codes)s
 """ % {
-            "setup_codes": "\n   ".join(setup_codes),
+            "setup_codes": "\n".join("   %s" % line for line in setup_codes),
             "query_codes": "\n".join(query_codes),
         }
 
+        if shallShowExecutedCommands():
+            self.info("Executing query command:\n%s" % cmd)
+
         try:
             feedback = check_output([sys.executable, "-c", cmd])
         except NuitkaCalledProcessError as e:
@@ -908,6 +996,9 @@ except ImportError:
         if str is not bytes:  # We want to work with strings, that's hopefully OK.
             feedback = feedback.decode("utf8")
 
+        if shallShowExecutedCommands():
+            self.info("Result of query command:\n%s" % feedback)
+
         # Ignore Windows newlines difference.
         feedback = [line.strip() for line in feedback.splitlines()]
 
@@ -948,6 +1039,11 @@ except ImportError:
         # Virtual method, pylint: disable=no-self-use,unused-argument
         return ()
 
+    @staticmethod
+    def getPackageVersion(distribution_name):
+        """Provide package version of a distribution."""
+        return _getPackageVersion(distribution_name)
+
     def evaluateCondition(self, full_name, condition, control_tags=()):
         # Note: Caching makes no sense yet, this should all be very fast and
         # cache themselves. TODO: Allow plugins to contribute their own control
@@ -971,13 +1067,21 @@ except ImportError:
                 "module_mode": shallMakeModule(),
                 # TODO: Allow to provide this.
                 "deployment": False,
-                # Module dependent
-                "main_module": full_name.getBasename() == "__main__",
                 # Querying package versions.
                 "version": _getPackageVersion,
+                "plugin": _isPluginActive,
             }
         )
 
+        if isWin32Windows():
+            context.update(
+                {
+                    "arch_x86": getArchitecture() == "x86",
+                    "arch_amd64": getArchitecture() == "x86_64",
+                    "arch_arm64": getArchitecture() == "arm64",
+                }
+            )
+
         versions = getSupportedPythonVersions()
 
         for version in versions:
@@ -992,7 +1096,13 @@ except ImportError:
         context["python3_or_higher"] = python_version >= 0x300
 
         # We trust the yaml files, pylint: disable=eval-used
-        result = eval(condition, context)
+        try:
+            result = eval(condition, context)
+        except Exception as e:  # Catch all the things, pylint: disable=broad-except
+            self.sysexit(
+                "Error, failed to evaluate condition '%s' in this context, exception was '%s'."
+                % (condition, e)
+            )
 
         if type(result) is not bool:
             self.sysexit(
@@ -1011,8 +1121,14 @@ except ImportError:
         return result
 
     @classmethod
-    def warning(cls, message):
-        plugins_logger.warning(cls.plugin_name + ": " + message)
+    def warning(cls, message, **kwargs):
+        # Doing keyword only arguments manually, to keep older Python compatibility, and avoid
+        # user errors still.
+        mnemonic = kwargs.pop("mnemonic", None)
+        if kwargs:
+            plugins_logger.sysexit("Illegal arguments for self.warning")
+
+        plugins_logger.warning(cls.plugin_name + ": " + message, mnemonic=mnemonic)
 
     @classmethod
     def info(cls, message):
