@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -29,37 +29,44 @@ The base class in PluginBase will serve as documentation of available.
 
 import inspect
 import os
-import shutil
 from optparse import OptionConflictError, OptionGroup
 
-import nuitka.plugins.commercial
-import nuitka.plugins.standard
 from nuitka import Options, OutputDirectories
 from nuitka.__past__ import basestring, iter_modules
 from nuitka.build.DataComposerInterface import deriveModuleConstantsBlobName
-from nuitka.containers.odict import OrderedDict
-from nuitka.containers.oset import OrderedSet
-from nuitka.freezer.IncludedEntryPoints import makeDllEntryPointOld
+from nuitka.containers.OrderedDicts import OrderedDict
+from nuitka.containers.OrderedSets import OrderedSet
+from nuitka.freezer.IncludedDataFiles import IncludedDataFile
+from nuitka.freezer.IncludedEntryPoints import IncludedEntryPoint
 from nuitka.ModuleRegistry import addUsedModule
 from nuitka.Tracing import plugins_logger, printLine
-from nuitka.utils.FileOperations import (
-    addFileExecutablePermission,
-    makePath,
-    putTextFileContents,
-)
+from nuitka.utils.FileOperations import makePath, putTextFileContents
 from nuitka.utils.Importing import importFileAsModule
-from nuitka.utils.ModuleNames import ModuleName, checkModuleName
+from nuitka.utils.ModuleNames import (
+    ModuleName,
+    checkModuleName,
+    makeTriggerModuleName,
+    post_module_load_trigger_name,
+    pre_module_load_trigger_name,
+)
 
-from .PluginBase import NuitkaPluginBase, post_modules, pre_modules
+from .PluginBase import NuitkaPluginBase
 
 # Maps plugin name to plugin instances.
 active_plugins = OrderedDict()
 plugin_name2plugin_classes = {}
 plugin_options = {}
-plugin_datatag2pluginclasses = {}
 plugin_values = {}
 user_plugins = OrderedSet()
 
+# Trigger modules
+pre_modules = {}
+pre_modules_reasons = {}
+post_modules = {}
+post_modules_reasons = {}
+fake_modules = {}
+has_active_gui_toolkit_plugin = False
+
 
 def _addActivePlugin(plugin_class, args, force=False):
     plugin_name = plugin_class.plugin_name
@@ -76,11 +83,21 @@ def _addActivePlugin(plugin_class, args, force=False):
     else:
         plugin_args = {}
 
-    plugin_instance = plugin_class(**plugin_args)
+    try:
+        plugin_instance = plugin_class(**plugin_args)
+    except TypeError as e:
+        plugin_class.sysexit("Problem initializing plugin: %s" % e)
+
     assert isinstance(plugin_instance, NuitkaPluginBase), plugin_instance
 
     active_plugins[plugin_name] = plugin_instance
 
+    # Singleton, pylint: disable=global-statement
+    global has_active_gui_toolkit_plugin
+    has_active_gui_toolkit_plugin = has_active_gui_toolkit_plugin or getattr(
+        plugin_class, "plugin_gui_toolkit", False
+    )
+
 
 def getActivePlugins():
     """Return list of active plugins.
@@ -94,8 +111,6 @@ def getActivePlugins():
 
 
 def getActiveQtPlugin():
-    from .standard.PySidePyQtPlugin import getQtPluginNames
-
     for plugin_name in getQtPluginNames():
         if hasActivePlugin(plugin_name):
             if hasActivePlugin(plugin_name):
@@ -104,6 +119,26 @@ def getActiveQtPlugin():
     return None
 
 
+def getQtBindingNames():
+    return ("PySide", "PySide2", "PySide6", "PyQt4", "PyQt5", "PyQt6")
+
+
+def getOtherGUIBindingNames():
+    return ("wx", "tkinter", "Tkinter")
+
+
+def getQtPluginNames():
+    return tuple(qt_binding_name.lower() for qt_binding_name in getQtBindingNames())
+
+
+def getOtherGuiPluginNames():
+    return ("tk-inter",)
+
+
+def getGuiPluginNames():
+    return getQtPluginNames() + getOtherGuiPluginNames()
+
+
 def hasActivePlugin(plugin_name):
     """Decide if a plugin is active.
 
@@ -125,6 +160,19 @@ def hasActivePlugin(plugin_name):
     return not hasattr(plugin_instance, "detector_for")
 
 
+def getUserActivatedPluginNames():
+    for plugin_name, plugin_instance in active_plugins.items():
+        # Detectors do not count.
+        if hasattr(plugin_instance, "detector_for"):
+            continue
+
+        # Always activated does not count either
+        if plugin_instance.isAlwaysEnabled():
+            continue
+
+        yield plugin_name
+
+
 def getPluginClass(plugin_name):
     # First, load plugin classes, to know what we are talking about.
     loadPlugins()
@@ -157,7 +205,7 @@ def _loadPluginClassesFromPackage(scan_package):
     scan_path = scan_package.__path__
 
     for item in iter_modules(scan_path):
-        if item.ispkg:
+        if item.ispkg:  # spell-checker: ignore ispkg
             continue
 
         module_loader = item.module_finder.find_module(item.name)
@@ -174,9 +222,9 @@ def _loadPluginClassesFromPackage(scan_package):
         try:
             plugin_module = module_loader.load_module(item.name)
         except Exception:
-            if Options.is_nondebug:
+            if Options.is_non_debug:
                 plugins_logger.warning(
-                    "Problem loading plugin %r (%s), ignored. Use --debug to make it visible."
+                    "Problem loading plugin %r ('%s'), ignored. Use '--debug' to make it visible."
                     % (item.name, module_loader.get_filename())
                 )
                 continue
@@ -237,14 +285,21 @@ def loadStandardPluginClasses():
     """Load plugin files located in 'standard' folder.
 
     Notes:
-        Scan through the 'standard' and 'commercial' sub-folder of the folder
-        where this module resides. Import each valid Python module (but not
-        packages) and process it as a plugin.
+        Scan through the 'standard' and 'commercial' plugins. Import each valid
+        Python module (but not packages) and process it as a plugin.
     Returns:
         None
     """
+    import nuitka.plugins.standard
+
     _loadPluginClassesFromPackage(nuitka.plugins.standard)
-    _loadPluginClassesFromPackage(nuitka.plugins.commercial)
+
+    try:
+        import nuitka.plugins.commercial
+    except ImportError:
+        pass
+    else:
+        _loadPluginClassesFromPackage(nuitka.plugins.commercial)
 
 
 class Plugins(object):
@@ -278,12 +333,23 @@ class Plugins(object):
 
                 yield ModuleName(v)
 
+        seen = set()
+
         for full_name in iterateModuleNames(plugin.getImplicitImports(module)):
+            if full_name in seen:
+                continue
+            seen.add(full_name)
+
+            # Ignore dependencies on self. TODO: Make this an error for the
+            # plugin.
+            if full_name == module.getFullName():
+                continue
+
             try:
                 _module_name, module_filename, _finding = Importing.locateModule(
                     module_name=full_name,
                     parent_package=None,
-                    level=-1,
+                    level=0,
                 )
             except Exception:
                 plugin.warning(
@@ -303,7 +369,7 @@ class Plugins(object):
 
             result.append((full_name, module_filename))
 
-        if result:
+        if result and Options.isShowInclusion():
             plugin.info(
                 "Implicit dependencies of module '%s' added '%s'."
                 % (module.getFullName(), ",".join(r[0] for r in result))
@@ -345,6 +411,13 @@ class Plugins(object):
                     source_ref=module.source_ref,
                 )
 
+    @classmethod
+    def getPackageExtraScanPaths(cls, package_name, package_dir):
+        for plugin in getActivePlugins():
+            for path in plugin.getPackageExtraScanPaths(package_name, package_dir):
+                if os.path.isdir(path):
+                    yield path
+
     @classmethod
     def considerImplicitImports(cls, module, signal_change):
         for plugin in getActivePlugins():
@@ -370,7 +443,7 @@ class Plugins(object):
                 pre_modules[full_name],
                 using_module=module,
                 usage_tag="plugins",
-                reason="Not yet propagated by plugins.",
+                reason=" ".join(pre_modules_reasons[full_name]),
                 source_ref=module.source_ref,
             )
 
@@ -379,22 +452,59 @@ class Plugins(object):
                 module=post_modules[full_name],
                 using_module=module,
                 usage_tag="plugins",
-                reason="Not yet propagated by plugins.",
+                reason=" ".join(post_modules_reasons[full_name]),
                 source_ref=module.source_ref,
             )
 
+        if full_name in fake_modules:
+            for fake_module, plugin, reason in fake_modules[full_name]:
+                addUsedModule(
+                    module=fake_module,
+                    using_module=module,
+                    usage_tag="plugins",
+                    reason=reason,
+                    source_ref=module.source_ref,
+                )
+
+    @staticmethod
+    def onCopiedDLLs(dist_dir, standalone_entry_points):
+        """Lets the plugins modify entry points on disk."""
+        for entry_point in standalone_entry_points:
+            if entry_point.kind.endswith("_ignored"):
+                continue
+
+            for plugin in getActivePlugins():
+                plugin.onCopiedDLL(os.path.join(dist_dir, entry_point.dest_path))
+
+    @staticmethod
+    def onBeforeCodeParsing():
+        """Let plugins prepare for code parsing"""
+        for plugin in getActivePlugins():
+            plugin.onBeforeCodeParsing()
+
     @staticmethod
     def onStandaloneDistributionFinished(dist_dir):
-        """Let plugins postprocess the distribution folder in standalone mode"""
+        """Let plugins post-process the distribution folder in standalone mode"""
         for plugin in getActivePlugins():
             plugin.onStandaloneDistributionFinished(dist_dir)
 
+        standalone_binary = OutputDirectories.getResultFullpath(onefile=False)
+
+        for plugin in getActivePlugins():
+            plugin.onStandaloneBinary(standalone_binary)
+
     @staticmethod
     def onOnefileFinished(filename):
-        """Let plugins postprocess the onefile executable in onefile mode"""
+        """Let plugins post-process the onefile executable in onefile mode"""
         for plugin in getActivePlugins():
             plugin.onStandaloneDistributionFinished(filename)
 
+    @staticmethod
+    def onBootstrapBinary(filename):
+        """Let plugins add to bootstrap binary in some way"""
+        for plugin in getActivePlugins():
+            plugin.onBootstrapBinary(filename)
+
     @staticmethod
     def onFinalResult(filename):
         """Let plugins add to final binary in some way"""
@@ -402,49 +512,38 @@ class Plugins(object):
             plugin.onFinalResult(filename)
 
     @staticmethod
-    def considerExtraDlls(dist_dir, module):
+    def considerExtraDlls(module):
         """Ask plugins to provide extra DLLs.
 
         Notes:
-            These will be of type nuitka.freezer.IncludedEntryPoints.IncludedEntryPoint
-            and currently there is a backward compatibility for old style plugins that do
-            provide tuples of 3 elements. But plugins are really supposed to provide the
-            stuff created from factory functions for that type.
+            These will be of type IncludedEntryPoint or generators providing them, so
+            we get "yield from" effective with simple yield.
 
         """
 
         result = []
 
+        # TODO: This is probably something generic that only different
+        def _iterateExtraBinaries(plugin, value):
+            if value is None:
+                pass
+            elif isinstance(value, IncludedEntryPoint):
+                yield value
+            elif isinstance(value, (tuple, list)) or inspect.isgenerator(value):
+                for val in value:
+                    for v in _iterateExtraBinaries(plugin, val):
+                        yield v
+            else:
+                plugin.sysexit(
+                    "Returned object '%s' for module '%s' but should do 'IncludedEntryPoint' or generator."
+                    % (repr(value), module.asString())
+                )
+
         for plugin in getActivePlugins():
-            for extra_dll in plugin.considerExtraDlls(dist_dir, module):
-                # Backward compatibility with plugins not yet migrated to getExtraDlls usage.
-                if len(extra_dll) == 3:
-                    extra_dll = makeDllEntryPointOld(
-                        source_path=extra_dll[0],
-                        dest_path=extra_dll[1],
-                        package_name=extra_dll[2],
-                    )
-
-                    if not os.path.isfile(extra_dll.dest_path):
-                        plugin.sysexit(
-                            "Error, copied filename %r for module %r that is not a file."
-                            % (extra_dll.dest_path, module.getFullName())
-                        )
-                else:
-                    if not os.path.isfile(extra_dll.source_path):
-                        plugin.sysexit(
-                            "Error, attempting to copy plugin determined filename %r for module %r that is not a file."
-                            % (extra_dll.source_path, module.getFullName())
-                        )
-
-                    makePath(os.path.dirname(extra_dll.dest_path))
-
-                    shutil.copyfile(extra_dll.source_path, extra_dll.dest_path)
-
-                    if extra_dll.executable:
-                        addFileExecutablePermission(extra_dll.dest_path)
-
-                result.append(extra_dll)
+            for entry_point in _iterateExtraBinaries(
+                plugin, plugin.getExtraDlls(module)
+            ):
+                result.append(entry_point)
 
         return result
 
@@ -504,10 +603,39 @@ class Plugins(object):
             Data file description pairs, either (source, dest) or (func, dest)
             where the func will be called to create the content dynamically.
         """
+
+        def _iterateIncludedDataFiles(plugin, value):
+            if value is None:
+                pass
+            elif isinstance(value, IncludedDataFile):
+                yield value
+            elif inspect.isgenerator(value):
+                for val in value:
+                    for v in _iterateIncludedDataFiles(plugin, val):
+                        yield v
+            else:
+                plugin.sysexit("Plugin return non-datafile '%s'" % repr(value))
+
         for plugin in getActivePlugins():
             for value in plugin.considerDataFiles(module):
-                if value:
-                    yield plugin, value
+                for included_datafile in _iterateIncludedDataFiles(plugin, value):
+                    yield included_datafile
+
+    @staticmethod
+    def getDataFileTags(included_datafile):
+        tags = OrderedSet([included_datafile.kind])
+
+        tags.update(Options.getDataFileTags(tags))
+
+        for plugin in getActivePlugins():
+            plugin.updateDataFileTags(included_datafile)
+
+        return tags
+
+    @staticmethod
+    def onDataFileTags(included_datafile):
+        for plugin in getActivePlugins():
+            plugin.onDataFileTags(included_datafile)
 
     @classmethod
     def _createTriggerLoadedModule(cls, module, trigger_name, code, flags):
@@ -519,7 +647,7 @@ class Plugins(object):
 
         Args:
             module: the module object (serves as dict key)
-            trigger_name: string ("-preload"/"-postload")
+            trigger_name: string ("preLoad"/"postLoad")
             code: the code string
 
         Returns
@@ -528,7 +656,7 @@ class Plugins(object):
 
         from nuitka.tree.Building import buildModule
 
-        module_name = ModuleName(module.getFullName() + trigger_name)
+        module_name = makeTriggerModuleName(module.getFullName(), trigger_name)
 
         # In debug mode, put the files in the build folder, so they can be looked up easily.
         if Options.is_debug and "HIDE_SOURCE" not in flags:
@@ -548,7 +676,7 @@ class Plugins(object):
                 source_code=code,
                 is_top=False,
                 is_main=False,
-                is_shlib=False,
+                is_extension=False,
                 is_fake=module_name,
                 hide_syntax_error=False,
             )
@@ -564,18 +692,23 @@ class Plugins(object):
 
     @classmethod
     def onModuleDiscovered(cls, module):
+        # We offer plugins many ways to provide extra stuff
+        # pylint: disable=too-many-branches,too-many-locals,too-many-statements
+
         full_name = module.getFullName()
 
-        def _untangleLoadDesc(descs):
-            if descs and inspect.isgenerator(descs):
-                descs = tuple(descs)
+        def _untangleLoadDescription(description):
+            if description and inspect.isgenerator(description):
+                description = tuple(description)
 
-            if descs:
-                if type(descs[0]) not in (tuple, list):
-                    descs = [descs]
+            if description:
+                if type(description[0]) not in (tuple, list):
+                    description = [description]
 
-                for desc in descs:
-                    if len(desc) == 2:
+                for desc in description:
+                    if desc is None:
+                        pass
+                    elif len(desc) == 2:
                         code, reason = desc
                         flags = ()
                     else:
@@ -585,24 +718,45 @@ class Plugins(object):
 
                     yield plugin, code, reason, flags
 
-        preload_descs = []
-        postload_descs = []
+        def _untangleFakeDesc(description):
+            if description and inspect.isgenerator(description):
+                description = tuple(description)
+
+            if description:
+                if type(description[0]) not in (tuple, list):
+                    description = [description]
+
+                for desc in description:
+                    assert len(desc) == 4, desc
+                    yield plugin, desc[0], desc[1], desc[2], desc[3]
+
+        pre_module_load_descriptions = []
+        post_module_load_descriptions = []
+        fake_module_descriptions = []
 
         for plugin in getActivePlugins():
             plugin.onModuleDiscovered(module)
 
-            preload_descs.extend(
-                _untangleLoadDesc(descs=plugin.createPreModuleLoadCode(module))
+            pre_module_load_descriptions.extend(
+                _untangleLoadDescription(
+                    description=plugin.createPreModuleLoadCode(module)
+                )
             )
-            postload_descs.extend(
-                _untangleLoadDesc(descs=plugin.createPostModuleLoadCode(module))
+            post_module_load_descriptions.extend(
+                _untangleLoadDescription(
+                    description=plugin.createPostModuleLoadCode(module)
+                )
+            )
+            fake_module_descriptions.extend(
+                _untangleFakeDesc(description=plugin.createFakeModuleDependency(module))
             )
 
-        if preload_descs:
+        if pre_module_load_descriptions:
             total_code = []
             total_flags = OrderedSet()
+            reasons = []
 
-            for plugin, pre_code, reason, flags in preload_descs:
+            for plugin, pre_code, reason, flags in pre_module_load_descriptions:
                 if pre_code:
                     plugin.info(
                         "Injecting pre-module load code for module '%s':" % full_name
@@ -612,22 +766,25 @@ class Plugins(object):
 
                     total_code.append(pre_code)
                     total_flags.update(flags)
+                    reasons.append(reason)
 
             if total_code:
                 assert full_name not in pre_modules
 
                 pre_modules[full_name] = cls._createTriggerLoadedModule(
                     module=module,
-                    trigger_name="-preLoad",
+                    trigger_name=pre_module_load_trigger_name,
                     code="\n\n".join(total_code),
                     flags=total_flags,
                 )
+                pre_modules_reasons[full_name] = tuple(reasons)
 
-        if postload_descs:
+        if post_module_load_descriptions:
             total_code = []
             total_flags = OrderedSet()
+            reasons = []
 
-            for plugin, post_code, reason, flags in postload_descs:
+            for plugin, post_code, reason, flags in post_module_load_descriptions:
                 if post_code:
                     plugin.info(
                         "Injecting post-module load code for module '%s':" % full_name
@@ -637,30 +794,63 @@ class Plugins(object):
 
                     total_code.append(post_code)
                     total_flags.update(flags)
+                    reasons.append(reason)
 
             if total_code:
                 assert full_name not in post_modules
 
                 post_modules[full_name] = cls._createTriggerLoadedModule(
                     module=module,
-                    trigger_name="-postLoad",
+                    trigger_name=post_module_load_trigger_name,
                     code="\n\n".join(total_code),
                     flags=total_flags,
                 )
+                post_modules_reasons[full_name] = reasons
+
+        if fake_module_descriptions:
+            fake_modules[full_name] = []
+
+            from nuitka.tree.Building import buildModule
+
+            for (
+                plugin,
+                fake_module_name,
+                source_code,
+                fake_filename,
+                reason,
+            ) in fake_module_descriptions:
+                fake_module, _added = buildModule(
+                    module_filename=fake_filename,
+                    module_name=fake_module_name,
+                    source_code=source_code,
+                    is_top=False,
+                    is_main=False,
+                    is_extension=False,
+                    is_fake=fake_module_name,
+                    hide_syntax_error=False,
+                )
+
+                if fake_module.getCompilationMode() == "bytecode":
+                    fake_module.setSourceCode(source_code)
+
+                fake_modules[full_name].append((fake_module, plugin, reason))
 
     @staticmethod
     def onModuleSourceCode(module_name, source_code):
         assert type(module_name) is ModuleName
         assert type(source_code) is str
 
+        contributing_plugins = OrderedSet()
+
         for plugin in getActivePlugins():
             new_source_code = plugin.onModuleSourceCode(module_name, source_code)
-            if new_source_code is not None:
+            if new_source_code is not None and new_source_code != source_code:
                 source_code = new_source_code
+                contributing_plugins.add(plugin)
 
             assert type(source_code) is str
 
-        return source_code
+        return source_code, contributing_plugins
 
     @staticmethod
     def onFrozenModuleSourceCode(module_name, is_package, source_code):
@@ -687,12 +877,15 @@ class Plugins(object):
         return bytecode
 
     @staticmethod
-    def onModuleEncounter(module_filename, module_name, module_kind):
+    def onModuleEncounter(module_name, module_filename, module_kind):
         result = None
+        deciding_plugins = []
 
         for plugin in getActivePlugins():
             must_recurse = plugin.onModuleEncounter(
-                module_filename, module_name, module_kind
+                module_name=module_name,
+                module_filename=module_filename,
+                module_kind=module_kind,
             )
 
             if must_recurse is None:
@@ -705,12 +898,26 @@ class Plugins(object):
 
             if result is not None:
                 # false alarm, pylint: disable=unsubscriptable-object
-                assert result[0] == must_recurse[0]
+                if result[0] != must_recurse[0]:
+                    plugin.sysexit(
+                        "Error, decision %s does not match other plugin '%s' decision."
+                        % (must_recurse[0], ".".join(deciding_plugins))
+                    )
 
+            deciding_plugins.append(plugin.plugin_name)
             result = must_recurse
 
         return result
 
+    @staticmethod
+    def onModuleRecursion(module_name, module_filename, module_kind):
+        for plugin in getActivePlugins():
+            plugin.onModuleRecursion(
+                module_name=module_name,
+                module_filename=module_filename,
+                module_kind=module_kind,
+            )
+
     @staticmethod
     def onModuleInitialSet():
         """The initial set of root modules is complete, plugins may add more."""
@@ -787,6 +994,8 @@ class Plugins(object):
             i.e. "-Dkey=value" vs. "-Dkey"
         """
 
+        # spell-checker: ignore -Dkey
+
         if cls.preprocessor_symbols is None:
             cls.preprocessor_symbols = OrderedDict()
 
@@ -797,15 +1006,71 @@ class Plugins(object):
                     assert type(value) is dict, value
 
                     # We order per plugin, but from the plugins, lets just take a dict
-                    # and achieve determism by ordering the defines by name.
+                    # and achieve determinism by ordering the defines by name.
                     for key, value in sorted(value.items()):
                         # False alarm, pylint: disable=I0021,unsupported-assignment-operation
                         cls.preprocessor_symbols[key] = value
 
         return cls.preprocessor_symbols
 
+    build_definitions = None
+
+    @classmethod
+    def getBuildDefinitions(cls):
+        """Let plugins provide C source defines to be used in compilation.
+
+        Notes:
+            The plugins can each contribute, but are hopefully using
+            a namespace for their defines. Only specific code sees these
+            if it chooses to include "build_definitions.h" file.
+
+        Returns:
+            OrderedDict() with keys and values."
+        """
+
+        if cls.build_definitions is None:
+            cls.build_definitions = OrderedDict()
+
+            for plugin in getActivePlugins():
+                value = plugin.getBuildDefinitions()
+
+                if value is not None:
+                    assert type(value) is dict, value
+
+                    # We order per plugin, but from the plugins themselves, lets just assume
+                    # unordered dict and achieve determinism by ordering the defines by name.
+                    for key, value in sorted(value.items()):
+                        # False alarm, pylint: disable=I0021,unsupported-assignment-operation
+                        cls.build_definitions[key] = value
+
+        return cls.build_definitions
+
+    extra_include_directories = None
+
+    @classmethod
+    def getExtraIncludeDirectories(cls):
+        """Let plugins extra directories to use for C includes in compilation.
+
+        Notes:
+            The plugins can each contribute, but are hopefully not colliding,
+            order will be plugin order.
+
+        Returns:
+            OrderedSet() of paths to include as well.
+        """
+        if cls.extra_include_directories is None:
+            cls.extra_include_directories = OrderedSet()
+
+            for plugin in getActivePlugins():
+                value = plugin.getExtraIncludeDirectories()
+
+                if value:
+                    cls.extra_include_directories.update(value)
+
+        return cls.extra_include_directories
+
     @staticmethod
-    def getExtraCodeFiles():
+    def _getExtraCodeFiles(for_onefile):
         result = OrderedDict()
 
         for plugin in getActivePlugins():
@@ -815,8 +1080,14 @@ class Plugins(object):
                 assert type(value) is dict
 
                 # We order per plugin, but from the plugins, lets just take a dict
-                # and achieve determism by ordering the files by name.
+                # and achieve determinism by ordering the files by name.
                 for key, value in sorted(value.items()):
+
+                    if (for_onefile and not "onefile_" in key) or (
+                        not for_onefile and "onefile_" in key
+                    ):
+                        continue
+
                     if not key.startswith("nuitka_"):
                         key = "plugin." + plugin.plugin_name + "." + key
 
@@ -825,6 +1096,23 @@ class Plugins(object):
 
         return result
 
+    @staticmethod
+    def writeExtraCodeFiles(onefile):
+        # Circular dependency.
+        from nuitka.tree.SourceHandling import writeSourceCode
+
+        source_dir = OutputDirectories.getSourceDirectoryPath(onefile=onefile)
+
+        for filename, source_code in Plugins._getExtraCodeFiles(onefile).items():
+            target_dir = os.path.join(source_dir, "plugins")
+
+            if not os.path.isdir(target_dir):
+                makePath(target_dir)
+
+            writeSourceCode(
+                filename=os.path.join(target_dir, filename), source_code=source_code
+            )
+
     extra_link_libraries = None
 
     @classmethod
@@ -837,10 +1125,10 @@ class Plugins(object):
 
                 if value is not None:
                     if isinstance(value, basestring):
-                        cls.extra_link_libraries.add(value)
+                        cls.extra_link_libraries.add(os.path.normcase(value))
                     else:
                         for library_name in value:
-                            cls.extra_link_libraries.add(library_name)
+                            cls.extra_link_libraries.add(os.path.normcase(library_name))
 
         return cls.extra_link_libraries
 
@@ -863,6 +1151,11 @@ class Plugins(object):
 
         return cls.extra_link_directories
 
+    @classmethod
+    def onDataComposerRun(cls):
+        for plugin in getActivePlugins():
+            plugin.onDataComposerRun()
+
     @classmethod
     def onDataComposerResult(cls, blob_filename):
         for plugin in getActivePlugins():
@@ -900,6 +1193,12 @@ class Plugins(object):
                 body=body,
             )
 
+    @classmethod
+    def getCacheContributionValues(cls, module_name):
+        for plugin in getActivePlugins():
+            for value in plugin.getCacheContributionValues(module_name):
+                yield value
+
 
 def listPlugins():
     """Print available standard plugins."""
@@ -998,6 +1297,19 @@ def loadPlugins():
         loadStandardPluginClasses()
 
 
+def addStandardPluginCommandLineOptions(parser):
+    loadPlugins()
+
+    for (_plugin_name, (plugin_class, _plugin_detector)) in sorted(
+        plugin_name2plugin_classes.items()
+    ):
+        if plugin_class.isAlwaysEnabled():
+            _addPluginCommandLineOptions(
+                parser=parser,
+                plugin_class=plugin_class,
+            )
+
+
 def activatePlugins():
     """Activate selected plugin classes
 
@@ -1014,6 +1326,8 @@ def activatePlugins():
     Returns:
         None
     """
+    # Many cases, often with UI related decisions, pylint: disable=too-many-branches
+
     loadPlugins()
 
     # ensure plugin is known and not both, enabled and disabled
@@ -1031,15 +1345,23 @@ def activatePlugins():
                 "Error, conflicting enable/disable of plug-in '%s'." % plugin_name
             )
 
+    plugin_detectors = OrderedSet()
+
     for (plugin_name, (plugin_class, plugin_detector)) in sorted(
         plugin_name2plugin_classes.items()
     ):
         if plugin_name in Options.getPluginsEnabled():
             if plugin_class.isAlwaysEnabled():
-                plugin_class.warning("Plugin is defined as always enabled.")
+                plugin_class.warning(
+                    "Plugin is defined as always enabled, no need to enable it."
+                )
 
             if plugin_class.isRelevant():
                 _addActivePlugin(plugin_class, args=True)
+            elif plugin_class.isDeprecated():
+                plugin_class.warning(
+                    "This plugin has been deprecated, do not enable it anymore."
+                )
             else:
                 plugin_class.warning(
                     "Not relevant with this OS, or Nuitka arguments given, not activated."
@@ -1053,27 +1375,25 @@ def activatePlugins():
             and Options.shallDetectMissingPlugins()
             and plugin_detector.isRelevant()
         ):
-            _addActivePlugin(plugin_detector, args=False)
+            plugin_detectors.add(plugin_detector)
 
     for plugin_class in user_plugins:
         _addActivePlugin(plugin_class, args=True)
 
-
-def lateActivatePlugin(plugin_name, option_values):
-    """Activate plugin after the command line parsing, expects options to be set."""
-
-    values = getPluginClass(plugin_name).getPluginDefaultOptionValues()
-    values.update(option_values)
-    setPluginOptions(plugin_name, values)
-
-    _addActivePlugin(getPluginClass(plugin_name), args=True, force=True)
+    # Suppress GUI toolkit detectors automatically.
+    for plugin_detector in plugin_detectors:
+        if (
+            not has_active_gui_toolkit_plugin
+            or plugin_detector.plugin_name not in getGuiPluginNames()
+        ):
+            _addActivePlugin(plugin_detector, args=False)
 
 
-def _addPluginCommandLineOptions(parser, plugin_class, data_files_tags):
+def _addPluginCommandLineOptions(parser, plugin_class):
     plugin_name = plugin_class.plugin_name
 
     if plugin_name not in plugin_options:
-        option_group = OptionGroup(parser, "Plugin %s" % plugin_name)
+        option_group = OptionGroup(parser, "Plugin options of '%s'" % plugin_name)
         try:
             plugin_class.addPluginCommandLineOptions(option_group)
         except OptionConflictError as e:
@@ -1095,26 +1415,12 @@ def _addPluginCommandLineOptions(parser, plugin_class, data_files_tags):
         else:
             plugin_options[plugin_name] = ()
 
-        plugin_data_files_tags = plugin_class.getTagDataFileTagOptions()
 
-        if plugin_data_files_tags:
-            for tag_name, tag_desc in plugin_data_files_tags:
-                if tag_name in (tag for tag, _desc in data_files_tags):
-                    plugins_logger.sysexit(
-                        "Plugin '%s' provides data files tag handling '%s' already provided."
-                        % (plugin_name, tag_name)
-                    )
-
-                data_files_tags.append((tag_name, tag_desc))
-
-                plugin_datatag2pluginclasses[tag_name] = plugin_class
-
-
-def addPluginCommandLineOptions(parser, plugin_names, data_files_tags):
+def addPluginCommandLineOptions(parser, plugin_names):
     """Add option group for the plugin to the parser.
 
     Notes:
-        This is exclusively for use in the commandline parsing. Not all
+        This is exclusively for use in the command line parsing. Not all
         plugins have to have options. But this will add them to the
         parser in a first pass, so they can be recognized in a second
         pass with them included.
@@ -1124,16 +1430,12 @@ def addPluginCommandLineOptions(parser, plugin_names, data_files_tags):
     """
     for plugin_name in plugin_names:
         plugin_class = getPluginClass(plugin_name)
-        _addPluginCommandLineOptions(
-            parser=parser, plugin_class=plugin_class, data_files_tags=data_files_tags
-        )
+        _addPluginCommandLineOptions(parser=parser, plugin_class=plugin_class)
 
 
-def addUserPluginCommandLineOptions(parser, filename, data_files_tags):
+def addUserPluginCommandLineOptions(parser, filename):
     plugin_class = loadUserPlugin(filename)
-    _addPluginCommandLineOptions(
-        parser=parser, plugin_class=plugin_class, data_files_tags=data_files_tags
-    )
+    _addPluginCommandLineOptions(parser=parser, plugin_class=plugin_class)
 
     user_plugins.add(plugin_class)
 
@@ -1174,10 +1476,37 @@ def getPluginOptions(plugin_name):
         if "[REQUIRED]" in option.help:
             if not arg_value:
                 plugins_logger.sysexit(
-                    "Error, required plugin argument %r of Nuitka plugin %s not given."
+                    "Error, required plugin argument '%s' of Nuitka plugin '%s' not given."
                     % (option_name, plugin_name)
                 )
 
         result[option.dest] = arg_value
 
     return result
+
+
+def replaceTriggerModule(old, new):
+    """Replace a trigger module with another form if it. For use in bytecode demotion."""
+
+    found = None
+    for key, value in pre_modules.items():
+        if value is old:
+            found = key
+            break
+
+    if found is not None:
+        pre_modules[found] = new
+
+    found = None
+    for key, value in post_modules.items():
+        if value is old:
+            found = key
+            break
+
+    if found is not None:
+        post_modules[found] = new
+
+
+def isTriggerModule(module):
+    """Decide of a module is a trigger module."""
+    return module in pre_modules.values() or module in post_modules.values()
