@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -153,13 +153,12 @@ void registerDillPluginTables(char const *module_name, PyMethodDef *reduce_compi
 
     if (function_tables == NULL) {
         DROP_ERROR_OCCURRED();
-        function_tables = PyDict_New();
+
+        function_tables = MAKE_DICT_EMPTY();
         PyObject_SetAttrString((PyObject *)builtin_module, "compiled_function_tables", function_tables);
     }
 
-    PyObject *funcs = PyTuple_New(2);
-    PyTuple_SET_ITEM(funcs, 0, PyCFunction_New(reduce_compiled_function, NULL));
-    PyTuple_SET_ITEM(funcs, 1, PyCFunction_New(create_compiled_function, NULL));
+    PyObject *funcs = MAKE_TUPLE2_0(PyCFunction_New(reduce_compiled_function, NULL), PyCFunction_New(create_compiled_function, NULL));
 
     PyDict_SetItemString(function_tables, module_name, funcs);
 }
