@@ -17,6 +17,7 @@
 #
 """ Common helper functions for specializing code."""
 
+from nuitka.Constants import the_empty_unicode
 from nuitka.tools.quality.auto_format.AutoFormat import (  # For import from here, pylint: disable=unused-import
     withFileOpenedAndAutoFormatted,
 )
@@ -32,32 +33,34 @@ def writeLine(output, *args):
 
 
 # Python2 dict methods:
-python2_dict_methods = (
-    "clear",  # has full dict coverage
-    "copy",  # has full dict coverage
-    "fromkeys",
-    "get",  # has full dict coverage
-    "has_key",  # has full dict coverage
-    "items",  # has full dict coverage
-    "iteritems",  # has full dict coverage
-    "iterkeys",  # has full dict coverage
-    "itervalues",  # has full dict coverage
-    "keys",  # has full dict coverage
-    "pop",  # has full dict coverage
-    "popitem",  # has full dict coverage
-    "setdefault",  # has full dict coverage
-    "update",  # has full dict coverage
-    "values",  # has full dict coverage
-    "viewitems",  # has full dict coverage
-    "viewkeys",  # has full dict coverage
-    "viewvalues",  # has full dict coverage
+python2_dict_methods = (  # we have full coverage for all methods
+    "clear",
+    "copy",
+    # This is actually a static method, not useful to call on on instance
+    # "fromkeys",
+    "get",
+    "has_key",
+    "items",
+    "iteritems",
+    "iterkeys",
+    "itervalues",
+    "keys",
+    "pop",
+    "popitem",
+    "setdefault",
+    "update",
+    "values",
+    "viewitems",
+    "viewkeys",
+    "viewvalues",
 )
 
 python3_dict_methods = (
     # see Python2 methods, these are only less
     "clear",
     "copy",
-    "fromkeys",
+    # This is actually a static method, not useful to call on on instance
+    # "fromkeys",
     "get",
     "items",
     "keys",
@@ -68,48 +71,48 @@ python3_dict_methods = (
     "values",
 )
 
-python2_str_methods = (
-    "capitalize",  # has full str coverage
+python2_str_methods = (  # we have full coverage for all methods
+    "capitalize",
     "center",
-    "count",  # has full str coverage
-    "decode",  # has full str coverage
-    "encode",  # has full str coverage
-    "endswith",  # has full str coverage
+    "count",
+    "decode",
+    "encode",
+    "endswith",
     "expandtabs",
-    "find",  # has full str coverage
+    "find",
     "format",
-    "index",  # has full str coverage
-    "isalnum",  # has full str coverage
-    "isalpha",  # has full str coverage
-    "isdigit",  # has full str coverage
-    "islower",  # has full str coverage
-    "isspace",  # has full str coverage
-    "istitle",  # has full str coverage
-    "isupper",  # has full str coverage
-    "join",  # has full str coverage
+    "index",
+    "isalnum",
+    "isalpha",
+    "isdigit",
+    "islower",
+    "isspace",
+    "istitle",
+    "isupper",
+    "join",
     "ljust",
-    "lower",  # has full str coverage
-    "lstrip",  # has full str coverage
-    "partition",  # has full str coverage
+    "lower",
+    "lstrip",
+    "partition",
     "replace",
-    "rfind",  # has full str coverage
-    "rindex",  # has full str coverage
+    "rfind",
+    "rindex",
     "rjust",
-    "rpartition",  # has full str coverage
-    "rsplit",  # has full str coverage
-    "rstrip",  # has full str coverage
-    "split",  # has full str coverage
+    "rpartition",
+    "rsplit",
+    "rstrip",
+    "split",
     "splitlines",
-    "startswith",  # has full str coverage
-    "strip",  # has full str coverage
-    "swapcase",  # has full str coverage
-    "title",  # has full str coverage
+    "startswith",
+    "strip",
+    "swapcase",
+    "title",
     "translate",
-    "upper",  # has full str coverage
+    "upper",
     "zfill",
 )
 
-python3_str_methods = (
+python3_str_methods = (  # we have full coverage for all methods
     "capitalize",
     "casefold",
     "center",
@@ -207,7 +210,7 @@ python3_bytes_methods = (
     "capitalize",
     "center",
     "count",
-    "decode",
+    "decode",  # has full coverage
     "endswith",
     "expandtabs",
     "find",
@@ -298,6 +301,12 @@ def getMethodVariations(spec_module, shape_name, method_name, must_exist=False):
     return present, arg_names, arg_tests, arg_name_mapping, arg_counts
 
 
+def getSpecs(spec_module):
+    for name in sorted(dir(spec_module)):
+        if name.endswith("_spec"):
+            yield name, getattr(spec_module, name)
+
+
 def formatArgs(args, starting=True, finishing=True):
     result = []
     if args:
@@ -318,7 +327,7 @@ def check():
         for method_name in python2_str_methods:
             assert hasattr("", method_name), method_name
         for method_name in python2_unicode_methods:
-            assert hasattr(u"", method_name), method_name
+            assert hasattr(the_empty_unicode, method_name), method_name
         for method_name in python2_dict_methods:
             assert hasattr({}, method_name), method_name
     else:
