@@ -29,7 +29,7 @@ import tempfile
 import threading
 import time
 from contextlib import contextmanager
-from optparse import OptionGroup, OptionParser
+from optparse import OptionParser
 
 from nuitka.__past__ import subprocess
 from nuitka.PythonVersions import (
@@ -40,21 +40,26 @@ from nuitka.PythonVersions import (
 from nuitka.Tracing import OurLogger, my_print
 from nuitka.tree.SourceHandling import readSourceCodeFromFilename
 from nuitka.utils.AppDirs import getCacheDir
-from nuitka.utils.Execution import check_output, getNullInput, getNullOutput
+from nuitka.utils.Execution import (
+    check_output,
+    createProcess,
+    getNullInput,
+    getNullOutput,
+)
 from nuitka.utils.FileOperations import (
     areSamePaths,
     getExternalUsePath,
     getFileContentByLine,
     getFileContents,
     getFileList,
-    isPathBelowOrSameAs,
+    isFilenameSameAsOrBelowPath,
     makePath,
     openTextFile,
     removeDirectory,
 )
 from nuitka.utils.InstalledPythons import findInstalledPython
 from nuitka.utils.Jinja2 import getTemplate
-from nuitka.utils.Utils import getOS
+from nuitka.utils.Utils import getOS, isMacOS, isWin32Windows
 
 from .SearchModes import (
     SearchModeByPattern,
@@ -161,7 +166,7 @@ def setup(suite="", needs_io_encoding=False, silent=False, go_main=True):
     if (
         len(os.environ["PYTHON"]) == 2
         and os.environ["PYTHON"].isdigit()
-        and os.name != "nt"
+        and not isWin32Windows()
     ):
 
         os.environ["PYTHON"] = "python%s.%s" % (
@@ -253,7 +258,7 @@ def convertUsing2to3(path, force=False):
 
     if use_binary:
         # On Windows, we cannot rely on 2to3 to be in the path.
-        if os.name == "nt":
+        if isWin32Windows():
             command = [
                 sys.executable,
                 os.path.join(os.path.dirname(sys.executable), "Tools/Scripts/2to3.py"),
@@ -269,7 +274,7 @@ def convertUsing2to3(path, force=False):
         check_output(command, stderr=getNullOutput())
 
     except subprocess.CalledProcessError:
-        if os.name == "nt":
+        if isWin32Windows():
             raise
 
         command[0:3] = ["2to3"]
@@ -390,7 +395,7 @@ def _removeCPythonTestSuiteDir():
         # call as last resort could be a good idea.
 
         # This seems to work for broken "lnk" files.
-        if os.name == "nt":
+        if isWin32Windows():
             os.system("rmdir /S /Q @test")
 
         if os.path.exists("@test"):
@@ -748,7 +753,7 @@ def createSearchMode():
 
     parser = OptionParser()
 
-    select_group = OptionGroup(parser, "Select Tests")
+    select_group = parser.add_option_group("Select Tests")
 
     select_group.add_option(
         "--pattern",
@@ -767,9 +772,9 @@ Execute only tests matching the pattern. Defaults to all tests.""",
 Execute all tests, continue execution even after failure of one.""",
     )
 
-    parser.add_option_group(select_group)
+    del select_group
 
-    debug_group = OptionGroup(parser, "Test features")
+    debug_group = parser.add_option_group("Test features")
 
     debug_group.add_option(
         "--debug",
@@ -790,7 +795,7 @@ Defaults to off.""",
 Defaults to off.""",
     )
 
-    parser.add_option_group(debug_group)
+    del debug_group
 
     options, positional_args = parser.parse_args()
 
@@ -1292,10 +1297,10 @@ def setupCacheHashSalt(test_code_path):
 
 
 def displayFolderContents(name, path):
-    test_logger.info("Listing of %s %r:" % (name, path))
+    test_logger.info("Listing of %s '%s':" % (name, path))
 
     if os.path.exists(path):
-        if os.name == "nt":
+        if isWin32Windows():
             command = "dir /b /s /a:-D %s" % path
         else:
             command = "ls -Rla %s" % path
@@ -1373,23 +1378,25 @@ class DelayedExecutionThread(threading.Thread):
         self.func()
 
 
-def executeAfterTimePassed(timeout, func):
+def executeAfterTimePassed(message, timeout, func):
+    test_logger.info(message % timeout)
+
     alarm = DelayedExecutionThread(timeout=timeout, func=func)
     alarm.start()
 
 
-def killProcess(name, pid):
+def killProcess(process_name, pid):
     """Kill a process in a portable way.
 
     Right now SIGINT is used, unclear what to do on Windows
     with Python2 or non-related processes.
     """
 
-    if str is bytes and os.name == "nt":
-        test_logger.info("Using taskkill on test process %r." % name)
+    if str is bytes and isWin32Windows():
+        test_logger.info("Using taskkill on test process '%s'." % process_name)
         os.system("taskkill.exe /PID %d" % pid)
     else:
-        test_logger.info("Killing test process %r." % name)
+        test_logger.info("Killing test process '%s'." % process_name)
         os.kill(pid, signal.SIGINT)
 
 
@@ -1409,7 +1416,7 @@ def checkLoadedFileAccesses(loaded_filenames, current_dir):
         loaded_filename = os.path.normcase(loaded_filename)
         loaded_basename = os.path.basename(loaded_filename)
 
-        if os.name == "nt":
+        if isWin32Windows():
             if areSamePaths(
                 os.path.dirname(loaded_filename),
                 os.path.normpath(os.path.join(os.environ["SYSTEMROOT"], "System32")),
@@ -1465,7 +1472,7 @@ def checkLoadedFileAccesses(loaded_filenames, current_dir):
             # Terminal info files are OK too.
             "/lib/terminfo",
         ):
-            if isPathBelowOrSameAs(ignored_dir, loaded_filename):
+            if isFilenameSameAsOrBelowPath(ignored_dir, loaded_filename):
                 ignore = False
                 break
         if not ignore:
@@ -1774,6 +1781,34 @@ def checkLoadedFileAccesses(loaded_filenames, current_dir):
         if loaded_basename.upper() in ("MSVCRT.DLL", "MSVCR90.DLL"):
             continue
 
+        if isMacOS():
+            ignore = True
+            for ignored_dir in (
+                "/System/Library/PrivateFrameworks",
+                "/System/Library/CoreService",
+                "/System/Library/Frameworks/CoreFoundation.framework",
+                "/System/Library/dyld",
+                "/usr/lib/system/",
+            ):
+                if isFilenameSameAsOrBelowPath(ignored_dir, loaded_filename):
+                    ignore = False
+                    break
+            if not ignore:
+                continue
+
+            if loaded_filename == "/usr/libexec/rosetta/runtime":
+                continue
+
+            if loaded_filename in (
+                "/usr/lib/libSystem.B.dylib",
+                "/usr/lib/libc++.1.dylib",
+                "/usr/lib/libc++abi.dylib",
+                "/usr/lib/libfakelink.dylib",
+                "/usr/lib/liboah.dylib",
+                "/usr/lib/libobjc.A.dylib",
+            ):
+                continue
+
         illegal_accesses.append(orig_loaded_filename)
 
     return illegal_accesses
@@ -1826,3 +1861,58 @@ def getPythonSysPath():
         _sys_path_path = _sys_path_path.strip()
 
     return _sys_path_path
+
+
+_web_server_process = None
+_web_server_port = 27272
+_web_server_hostname = "localhost"
+
+
+def getLocalWebServerUrl():
+    return "http://%s:%d" % (_web_server_hostname, _web_server_port)
+
+
+def getLocalWebServerDir(base_dir):
+    global _web_server_process  # singleton, pylint: disable=global-statement
+
+    web_dir = os.path.join(getTempDir(), "local-web-server", base_dir)
+
+    if _web_server_process is None:
+        web_server_directory_supporting_pythons = ("3.10", "3.9", "3.8", "3.7")
+
+        web_server_python = findInstalledPython(
+            python_versions=web_server_directory_supporting_pythons,
+            module_name=None,
+            module_version=None,
+        )
+
+        if web_server_python is None:
+            return None
+
+        os.makedirs(web_dir)
+
+        command = [
+            web_server_python.getPythonExe(),
+            "-m",
+            "http.server",
+            "--bind",
+            _web_server_hostname,
+            "--dir",
+            web_dir,
+            str(_web_server_port),
+        ]
+
+        test_logger.my_print(" ".join(command))
+
+        _web_server_process = createProcess(
+            command,
+            stdout=sys.stderr,
+            stderr=sys.stderr,
+        )
+
+        def killWebServerProcess():
+            _web_server_process.kill()
+
+        atexit.register(killWebServerProcess)
+
+    return web_dir
