@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -19,6 +19,7 @@
 
 """
 
+
 def generateConstructCases(construct_source_code):
     inside = False
     case = 0
@@ -47,4 +48,4 @@ def generateConstructCases(construct_source_code):
             inside = True
             case = 1
 
-    return '\n'.join(case_1), '\n'.join(case_2)
+    return "\n".join(case_1), "\n".join(case_2)
