@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -73,8 +73,11 @@ tempfile_re = re.compile(r"/tmp/tmp[a-z0-9_]*")
 logging_info_re = re.compile(r"^Nuitka.*?:INFO")
 logging_warning_re = re.compile(r"^Nuitka.*?:WARNING")
 
+# Python3.11 style traceback carets are not done by Nuitka (yet?)
+syntax_error_caret_re = re.compile(r"^\s*~*\^*~*$")
 
-def normalizeTimeDiff(outputStr):
+
+def _normalizeTimeDiff(outputStr):
     """
     use regular expression to normalize the time output
 
@@ -98,7 +101,7 @@ def makeDiffable(output, ignore_warnings, syntax_errors):
     # pylint: disable=too-many-branches,too-many-statements
 
     # do not compare time differences
-    output = normalizeTimeDiff(output)
+    output = _normalizeTimeDiff(output)
 
     result = []
 
@@ -140,7 +143,18 @@ def makeDiffable(output, ignore_warnings, syntax_errors):
         if logging_info_re.match(line):
             continue
 
-        if line.startswith("Nuitka-Recursion:WARNING: Cannot follow import to module"):
+        if line.startswith("Nuitka-Inclusion:WARNING: Cannot follow import to module"):
+            continue
+        if line.startswith("Nuitka:WARNING: Cannot detect Linux distribution"):
+            continue
+        if line.startswith(
+            "Nuitka-Options:WARNING: You did not specify to follow or include"
+        ):
+            continue
+        if line.startswith("Nuitka:WARNING: Using very slow fallback for ordered sets"):
+            continue
+
+        if syntax_error_caret_re.match(line):
             continue
 
         line = instance_re.sub(r"at 0xxxxxxxxx\1", line)
@@ -217,6 +231,15 @@ exceeded while calling a Python object' in \
         if "clcache: persistent json file" in line or "clcache: manifest file" in line:
             continue
 
+        # Ignore manual error message of CPython 3.11 that is different from the generic one for super
+        line = line.replace(
+            "super() argument 1 must be a type, not NoneType",
+            "super() argument 1 must be type, not None",
+        )
+        line = line.replace(
+            "super() argument 1 must be a type", "super() argument 1 must be type"
+        )
+
         result.append(line)
 
     return result
@@ -225,16 +248,16 @@ exceeded while calling a Python object' in \
 def compareOutput(
     kind, out_cpython, out_nuitka, ignore_warnings, syntax_errors, trace_result=True
 ):
-    fromdate = ""
-    todate = ""
+    from_date = ""
+    to_date = ""
 
     diff = difflib.unified_diff(
         makeDiffable(out_cpython, ignore_warnings, syntax_errors),
         makeDiffable(out_nuitka, ignore_warnings, syntax_errors),
         "{program} ({detail})".format(program=os.environ["PYTHON"], detail=kind),
         "{program} ({detail})".format(program="nuitka", detail=kind),
-        fromdate,
-        todate,
+        from_date,
+        to_date,
         n=3,
     )
 
