@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -33,6 +33,7 @@ from nuitka.PythonVersions import python_version
 from nuitka.tools.testing.Common import (
     addToPythonPath,
     executeAfterTimePassed,
+    getDebugPython,
     getTempDir,
     getTestingCPythonOutputsCacheDir,
     killProcess,
@@ -41,6 +42,7 @@ from nuitka.tools.testing.Common import (
 from nuitka.tools.testing.OutputComparison import compareOutput
 from nuitka.Tracing import my_print
 from nuitka.utils.Execution import (
+    callProcess,
     check_output,
     executeProcess,
     wrapCommandForDebuggerForSubprocess,
@@ -69,7 +71,7 @@ def checkNoPermissionError(output):
         if candidate in output:
             return False
 
-    # These are localized it seems.
+    # These are localized it seems, spell-checker: ignore totest
     if re.search(
         b"(WindowsError|FileNotFoundError|FileExistsError|WinError 145):"
         b".*(@test|totest|xx|Error 145)",
@@ -87,7 +89,7 @@ def checkNoPermissionError(output):
 def _getCPythonResults(cpython_cmd, send_kill):
     stop_watch = StopWatch()
 
-    # Try a coupile of times for permission denied, on Windows it can
+    # Try a compile of times for permission denied, on Windows it can
     # be transient.
     for _i in range(5):
         stop_watch.start()
@@ -101,7 +103,9 @@ def _getCPythonResults(cpython_cmd, send_kill):
         if send_kill:
             # Doing it per loop iteration hopefully, pylint: disable=cell-var-from-loop
             executeAfterTimePassed(
-                1.0, lambda: killProcess("Uncompiled Python program", process.pid)
+                message="Scheduling process kill %f",
+                timeout=2.0,
+                func=lambda: killProcess("Uncompiled Python program", process.pid),
             )
 
         stdout_cpython, stderr_cpython = process.communicate()
@@ -147,7 +151,7 @@ def getCPythonResults(cpython_cmd, cpython_cached, force_update, send_kill):
         command_hash.update(hash_salt)
 
         if os.name == "nt" and python_version < 0x300:
-            curdir = os.getcwdu()
+            curdir = os.getcwdu()  # spell-checker: ignore getcwdu
         else:
             curdir = os.getcwd()
 
@@ -225,8 +229,9 @@ def main():
     ignore_warnings = hasArg("ignore_warnings")
     expect_success = hasArg("expect_success")
     expect_failure = hasArg("expect_failure")
-    python_debug = hasArg("python_debug")
+    python_debug = hasArg("python_debug") or hasArg("--python-debug")
     module_mode = hasArg("--module")
+    module_entry_point = hasArgValue("--module-entry-point")
     coverage_mode = hasArg("coverage")
     two_step_execution = hasArg("two_step_execution")
     binary_python_path = hasArg("binary_python_path")
@@ -251,9 +256,9 @@ def main():
     full_compat = not hasArg("improved")
     cpython_cached = hasArg("cpython_cache")
     syntax_errors = hasArg("syntax_errors")
-    noprefer_source = hasArg("noprefer_source")
-    noverbose_log = hasArg("noverbose_log")
-    noinclusion_log = hasArg("noinclusion_log")
+    no_prefer_source = hasArg("--no-prefer-source")
+    no_verbose_log = hasArg("no_verbose_log")
+    no_inclusion_log = hasArg("no_inclusion_log")
     send_kill = hasArg("--send-ctrl-c")
     output_dir = hasArgValue("--output-dir", None)
     include_packages = hasArgValues("--include-package")
@@ -278,18 +283,11 @@ def main():
             user_plugins.append(arg[len("user_plugin:") :])
             del args[count]
 
-    recurse_not = []
+    nowarn_mnemonics = []
 
     for count, arg in reversed(tuple(enumerate(args))):
-        if arg.startswith("recurse_not:"):
-            recurse_not.append(arg[len("recurse_not:") :])
-            del args[count]
-
-    recurse_to = []
-
-    for count, arg in reversed(tuple(enumerate(args))):
-        if arg.startswith("recurse_to:"):
-            recurse_to.append(arg[len("recurse_to:") :])
+        if arg.startswith("--nowarn-mnemonic="):
+            nowarn_mnemonics.append(arg[len("--nowarn-mnemonic=") :])
             del args[count]
 
     if args:
@@ -324,22 +322,13 @@ def main():
 
     extra_options = os.environ.get("NUITKA_EXTRA_OPTIONS", "").split()
 
-    if "--python-debug" in extra_options or "--python-dbg" in extra_options:
+    if os.path.normcase(os.environ["PYTHON"]).endswith(("-dbg", "-debug", "_d.exe")):
+        python_debug = True
+    elif "--python-debug" in extra_options or "--python-dbg" in extra_options:
         python_debug = True
 
     if python_debug:
-        if os.path.exists(os.path.join("/usr/bin/", os.environ["PYTHON"] + "-dbg")):
-            os.environ["PYTHON"] += "-dbg"
-
-        if os.name == "nt":
-            if os.path.exists(os.environ["PYTHON"][:-4] + "_d.exe"):
-                os.environ["PYTHON"] = os.environ["PYTHON"][:-4] + "_d.exe"
-
-    if os.environ["PYTHON"].endswith("-dbg"):
-        python_debug = True
-
-    if os.environ["PYTHON"].lower().endswith("_d.exe"):
-        python_debug = True
+        os.environ["PYTHON"] = getDebugPython() or os.environ["PYTHON"]
 
     if comparison_mode:
         my_print(
@@ -374,11 +363,15 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".format
         if module_name.endswith(".py"):
             module_name = module_name[:-3]
 
+        mini_script = "import %s" % module_name
+        if module_entry_point:
+            mini_script += "; %s.%s()" % (module_name, module_entry_point)
+
         cpython_cmd = [
             os.environ["PYTHON"],
             "-c",
-            "import sys; sys.path.append(%s); import %s"
-            % (repr(os.path.dirname(filename)), module_name),
+            "import sys; sys.path.append(%s); %s"
+            % (repr(os.path.dirname(filename)), mini_script),
         ]
 
         if no_warnings:
@@ -408,7 +401,7 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".format
         # Would need to extract which "python" this is going to use.
         assert not coverage_mode, "Not implemented for binaries."
 
-        nuitka_call = [os.environ["NUITKA"]]
+        nuitka_call = [os.environ["PYTHON"], os.environ["NUITKA"]]
     else:
         if comparison_mode:
             nuitka_call = [
@@ -419,6 +412,8 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".format
         else:
             assert coverage_mode
 
+            # spell-checker: ignore rcfile
+
             nuitka_call = [
                 os.environ["PYTHON"],
                 "-S",
@@ -450,7 +445,7 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".format
     if full_compat:
         extra_options.append("--full-compat")
 
-    if noprefer_source:
+    if no_prefer_source:
         extra_options.append("--no-prefer-source")
 
     if python_flag_m:
@@ -475,7 +470,7 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".format
     if binary_python_path:
         addToPythonPath(os.path.dirname(os.path.abspath(filename)))
 
-    if keep_python_path or binary_python_path:
+    if (keep_python_path or binary_python_path) and not coverage_mode:
         extra_options.append("--execute-with-pythonpath")
 
     if report:
@@ -487,8 +482,8 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".format
     if follow_imports:
         extra_options.append("--follow-imports")
 
-    if recurse_not:
-        extra_options.extend("--nofollow-import-to=" + v for v in recurse_not)
+    if nowarn_mnemonics:
+        extra_options.extend("--nowarn-mnemonic=" + v for v in nowarn_mnemonics)
 
     if coverage_mode:
         extra_options.append("--must-not-re-execute")
@@ -503,10 +498,10 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".format
     for user_plugin in user_plugins:
         extra_options.append("--user-plugin=" + user_plugin)
 
-    if not noverbose_log:
+    if not no_verbose_log:
         extra_options.append("--verbose-output=%s.optimization.log" % filename)
 
-    if not noinclusion_log:
+    if not no_inclusion_log:
         extra_options.append("--show-modules-output=%s.inclusion.log" % filename)
 
     if output_dir is not None:
@@ -529,6 +524,9 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".format
     for include_module in include_modules:
         extra_options.append("--include-module=%s" % include_module)
 
+    # Progress bar is not used.
+    extra_options.append("--no-progressbar")
+
     # Now build the command to run Nuitka.
     if not two_step_execution:
         if module_mode:
@@ -562,13 +560,7 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".format
         if module_name.endswith(".py"):
             module_name = module_name[:-3]
 
-        nuitka_cmd2 = [
-            os.environ["PYTHON"],
-            "-W",
-            "ignore",
-            "-c",
-            "import %s" % module_name,
-        ]
+        nuitka_cmd2 = [os.environ["PYTHON"], "-W", "ignore", "-c", mini_script]
     else:
         exe_filename = os.path.basename(filename)
 
@@ -651,6 +643,8 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".format
                     nuitka_cmd1
                 )
 
+                python_path_used = os.environ["PYTHONPATH"]
+
             if exit_nuitka1 != 0:
                 if (
                     not expect_failure
@@ -659,12 +653,12 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".format
                 ):
                     sys.exit(
                         """\
-Error, failed to take coverage with '%s'.
+Error, failed to take coverage with '%s' (PYTHONPATH was '%s').
 
 Stderr was:
 %s
 """
-                        % (os.environ["PYTHON"], stderr_nuitka1)
+                        % (nuitka_cmd1, python_path_used, stderr_nuitka1)
                     )
 
                 exit_nuitka = exit_nuitka1
@@ -689,8 +683,11 @@ Stderr was:
                     if send_kill:
                         # Lambda is used immediately in same loop, pylint: disable=cell-var-from-loop
                         executeAfterTimePassed(
-                            1.0,
-                            lambda: killProcess("Nuitka compiled program", process.pid),
+                            message="Scheduling process kill %f",
+                            timeout=2.0,
+                            func=lambda: killProcess(
+                                "Nuitka compiled program", process.pid
+                            ),
                         )
 
                     stdout_nuitka2, stderr_nuitka2 = process.communicate()
@@ -702,7 +699,7 @@ Stderr was:
                     if exit_nuitka in (-11, -6) and sys.platform != "nt":
                         nuitka_cmd2 = wrapCommandForDebuggerForSubprocess(*nuitka_cmd2)
 
-                        executeProcess(nuitka_cmd2)
+                        callProcess(nuitka_cmd2, shell=False)
                 else:
                     exit_nuitka = exit_nuitka1
                     stdout_nuitka, stderr_nuitka = stdout_nuitka1, stderr_nuitka1
@@ -810,7 +807,7 @@ Stderr was:
             nuitka_cmd.insert(len(nuitka_cmd) - 1, "--debugger")
 
             with withPythonPathChange(nuitka_package_dir):
-                executeProcess(command=nuitka_cmd, needs_stdin=True)
+                callProcess(nuitka_cmd, shell=False)
 
         exit_code = exit_code_stdout or exit_code_stderr or exit_code_return
 
