@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,11 +22,6 @@ source code comments with Developer Manual sections.
 
 """
 
-from nuitka.nodes.AssignNodes import (
-    StatementAssignmentVariable,
-    StatementAssignmentVariableName,
-    StatementReleaseVariable,
-)
 from nuitka.nodes.AttributeNodes import makeExpressionAttributeLookup
 from nuitka.nodes.BuiltinRefNodes import ExpressionBuiltinAnonymousRef
 from nuitka.nodes.CallNodes import makeExpressionCall
@@ -51,13 +46,16 @@ from nuitka.nodes.ModuleAttributeNodes import ExpressionModuleAttributeNameRef
 from nuitka.nodes.NodeMakingHelpers import mergeStatements
 from nuitka.nodes.OutlineNodes import ExpressionOutlineBody
 from nuitka.nodes.ReturnNodes import StatementReturn
-from nuitka.nodes.SubscriptNodes import ExpressionSubscriptLookup
+from nuitka.nodes.SubscriptNodes import makeExpressionIndexLookup
 from nuitka.nodes.TryNodes import StatementTry
 from nuitka.nodes.TypeNodes import ExpressionBuiltinType1
-from nuitka.nodes.VariableRefNodes import (
-    ExpressionTempVariableRef,
+from nuitka.nodes.VariableAssignNodes import makeStatementAssignmentVariable
+from nuitka.nodes.VariableNameNodes import (
     ExpressionVariableNameRef,
+    StatementAssignmentVariableName,
 )
+from nuitka.nodes.VariableRefNodes import ExpressionTempVariableRef
+from nuitka.nodes.VariableReleaseNodes import makeStatementReleaseVariable
 from nuitka.PythonVersions import python_version
 
 from .ReformulationClasses3 import buildClassNode3
@@ -86,6 +84,7 @@ def buildClassNode2(provider, node, source_ref):
 
     code_object = CodeObjectSpec(
         co_name=node.name,
+        co_qualname=provider.getChildQualname(node.name),
         co_kind="Class",
         co_varnames=(),
         co_freevars=(),
@@ -181,15 +180,13 @@ def buildClassNode2(provider, node, source_ref):
         tmp_base = select_metaclass.allocateTempVariable(temp_scope=None, name="base")
 
         statements = (
-            StatementAssignmentVariable(
+            makeStatementAssignmentVariable(
                 variable=tmp_base,
-                source=ExpressionSubscriptLookup(
+                source=makeExpressionIndexLookup(
                     expression=ExpressionTempVariableRef(
                         variable=tmp_bases, source_ref=source_ref
                     ),
-                    subscript=makeConstantRefNode(
-                        constant=0, source_ref=source_ref, user_provided=True
-                    ),
+                    index_value=0,
                     source_ref=source_ref,
                 ),
                 source_ref=source_ref,
@@ -225,7 +222,7 @@ def buildClassNode2(provider, node, source_ref):
                     return_handler=None,
                     source_ref=source_ref,
                 ),
-                final=StatementReleaseVariable(
+                final=makeStatementReleaseVariable(
                     variable=tmp_base, source_ref=source_ref
                 ),
                 source_ref=source_ref,
@@ -269,7 +266,7 @@ def buildClassNode2(provider, node, source_ref):
     )
 
     statements = [
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=tmp_bases,
             source=makeExpressionMakeTupleOrConstant(
                 elements=buildNodeList(
@@ -280,10 +277,10 @@ def buildClassNode2(provider, node, source_ref):
             ),
             source_ref=source_ref,
         ),
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=tmp_class_dict, source=function_body, source_ref=source_ref
         ),
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=tmp_metaclass,
             source=ExpressionConditional(
                 condition=ExpressionDictOperationIn(
@@ -313,7 +310,7 @@ def buildClassNode2(provider, node, source_ref):
             ),
             source_ref=source_ref,
         ),
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=tmp_class,
             source=makeExpressionCall(
                 called=ExpressionTempVariableRef(
@@ -344,7 +341,7 @@ def buildClassNode2(provider, node, source_ref):
 
     for decorator in buildNodeList(provider, reversed(node.decorator_list), source_ref):
         statements.append(
-            StatementAssignmentVariable(
+            makeStatementAssignmentVariable(
                 variable=tmp_class,
                 source=makeExpressionCall(
                     called=decorator,
@@ -373,10 +370,10 @@ def buildClassNode2(provider, node, source_ref):
     )
 
     final = (
-        StatementReleaseVariable(variable=tmp_class, source_ref=source_ref),
-        StatementReleaseVariable(variable=tmp_bases, source_ref=source_ref),
-        StatementReleaseVariable(variable=tmp_class_dict, source_ref=source_ref),
-        StatementReleaseVariable(variable=tmp_metaclass, source_ref=source_ref),
+        makeStatementReleaseVariable(variable=tmp_class, source_ref=source_ref),
+        makeStatementReleaseVariable(variable=tmp_bases, source_ref=source_ref),
+        makeStatementReleaseVariable(variable=tmp_class_dict, source_ref=source_ref),
+        makeStatementReleaseVariable(variable=tmp_metaclass, source_ref=source_ref),
     )
 
     return makeTryFinallyStatement(
