@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,10 +22,6 @@ source code comments with Developer Manual sections.
 
 """
 
-from nuitka.nodes.AssignNodes import (
-    StatementAssignmentVariable,
-    StatementReleaseVariable,
-)
 from nuitka.nodes.BuiltinRefNodes import ExpressionBuiltinExceptionRef
 from nuitka.nodes.ComparisonNodes import ExpressionComparisonIs
 from nuitka.nodes.ConditionalNodes import (
@@ -40,7 +36,9 @@ from nuitka.nodes.ExceptionNodes import StatementRaiseException
 from nuitka.nodes.ExecEvalNodes import StatementExec, StatementLocalsDictSync
 from nuitka.nodes.GlobalsLocalsNodes import ExpressionBuiltinGlobals
 from nuitka.nodes.NodeMakingHelpers import makeExpressionBuiltinLocals
+from nuitka.nodes.VariableAssignNodes import makeStatementAssignmentVariable
 from nuitka.nodes.VariableRefNodes import ExpressionTempVariableRef
+from nuitka.nodes.VariableReleaseNodes import makeStatementReleaseVariable
 
 from .ReformulationTryFinallyStatements import makeTryFinallyStatement
 from .TreeHelpers import (
@@ -91,10 +89,10 @@ def wrapEvalGlobalsAndLocals(
         )
 
     post_statements += (
-        StatementReleaseVariable(
+        makeStatementReleaseVariable(
             variable=globals_keeper_variable, source_ref=source_ref
         ),
-        StatementReleaseVariable(
+        makeStatementReleaseVariable(
             variable=locals_keeper_variable, source_ref=source_ref
         ),
     )
@@ -119,10 +117,10 @@ def wrapEvalGlobalsAndLocals(
 
     pre_statements = [
         # First assign globals and locals temporary the values given.
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=globals_keeper_variable, source=globals_node, source_ref=source_ref
         ),
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=locals_keeper_variable, source=locals_node, source_ref=source_ref
         ),
         makeStatementConditional(
@@ -133,7 +131,7 @@ def wrapEvalGlobalsAndLocals(
                 right=ExpressionConstantNoneRef(source_ref=source_ref),
                 source_ref=source_ref,
             ),
-            yes_branch=StatementAssignmentVariable(
+            yes_branch=makeStatementAssignmentVariable(
                 variable=locals_keeper_variable,
                 source=locals_default,
                 source_ref=source_ref,
@@ -149,7 +147,7 @@ def wrapEvalGlobalsAndLocals(
                 right=ExpressionConstantNoneRef(source_ref=source_ref),
                 source_ref=source_ref,
             ),
-            yes_branch=StatementAssignmentVariable(
+            yes_branch=makeStatementAssignmentVariable(
                 variable=globals_keeper_variable,
                 source=ExpressionBuiltinGlobals(source_ref=source_ref),
                 source_ref=source_ref,
@@ -246,22 +244,22 @@ exec: arg 1 must be a string, file, or code object""",
 
     tried = (
         # First evaluate the source code expressions.
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=source_variable, source=source_code, source_ref=source_ref
         ),
         # Assign globals and locals temporary the values given, then fix it
         # up, taking note in the "plain" temporary variable, if it was an
         # "exec" statement with None arguments, in which case the copy back
         # will be necessary.
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=globals_keeper_variable,
             source=globals_value,
             source_ref=source_ref,
         ),
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=locals_keeper_variable, source=locals_value, source_ref=source_ref
         ),
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=plain_indicator_variable,
             source=makeConstantRefNode(constant=False, source_ref=source_ref),
             source_ref=source_ref,
@@ -275,7 +273,7 @@ exec: arg 1 must be a string, file, or code object""",
                 source_ref=source_ref,
             ),
             yes_branch=makeStatementsSequenceFromStatements(
-                StatementAssignmentVariable(
+                makeStatementAssignmentVariable(
                     variable=globals_keeper_variable,
                     source=ExpressionBuiltinGlobals(source_ref=source_ref),
                     source_ref=source_ref,
@@ -289,7 +287,7 @@ exec: arg 1 must be a string, file, or code object""",
                         source_ref=source_ref,
                     ),
                     yes_branch=makeStatementsSequenceFromStatements(
-                        StatementAssignmentVariable(
+                        makeStatementAssignmentVariable(
                             variable=locals_keeper_variable,
                             source=makeExpressionBuiltinLocals(
                                 locals_scope=provider.getLocalsScope(),
@@ -297,7 +295,7 @@ exec: arg 1 must be a string, file, or code object""",
                             ),
                             source_ref=source_ref,
                         ),
-                        StatementAssignmentVariable(
+                        makeStatementAssignmentVariable(
                             variable=plain_indicator_variable,
                             source=makeConstantRefNode(
                                 constant=True, source_ref=source_ref
@@ -319,7 +317,7 @@ exec: arg 1 must be a string, file, or code object""",
                         source_ref=source_ref,
                     ),
                     yes_branch=makeStatementsSequenceFromStatement(
-                        statement=StatementAssignmentVariable(
+                        statement=makeStatementAssignmentVariable(
                             variable=locals_keeper_variable,
                             source=ExpressionTempVariableRef(
                                 variable=globals_keeper_variable, source_ref=source_ref
@@ -370,14 +368,14 @@ exec: arg 1 must be a string, file, or code object""",
     )
 
     final = (
-        StatementReleaseVariable(variable=source_variable, source_ref=source_ref),
-        StatementReleaseVariable(
+        makeStatementReleaseVariable(variable=source_variable, source_ref=source_ref),
+        makeStatementReleaseVariable(
             variable=globals_keeper_variable, source_ref=source_ref
         ),
-        StatementReleaseVariable(
+        makeStatementReleaseVariable(
             variable=locals_keeper_variable, source_ref=source_ref
         ),
-        StatementReleaseVariable(
+        makeStatementReleaseVariable(
             variable=plain_indicator_variable, source_ref=source_ref
         ),
     )
