@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -25,7 +25,7 @@ import os
 from nuitka import Tracing
 from nuitka.__past__ import raw_input, urlretrieve
 
-from .AppDirs import getAppDir
+from .AppDirs import getCacheDir
 from .FileOperations import addFileExecutablePermission, deleteFile, makePath
 
 
@@ -34,29 +34,30 @@ def getCachedDownload(
     binary,
     flatten,
     is_arch_specific,
-    specifity,
+    specificity,
     message,
     reject,
     assume_yes_for_downloads,
 ):
-    # Many branches to deal with, pylint: disable=too-many-branches,too-many-statements
+    # Many branches to deal with.
+    # pylint: disable=too-many-branches,too-many-locals,too-many-statements
 
-    nuitka_app_dir = getAppDir()
+    nuitka_download_dir = os.path.join(getCacheDir(), "downloads")
 
-    nuitka_app_dir = os.path.join(
-        nuitka_app_dir, os.path.basename(binary).replace(".exe", "")
+    nuitka_download_dir = os.path.join(
+        nuitka_download_dir, os.path.basename(binary).replace(".exe", "")
     )
 
     if is_arch_specific:
-        nuitka_app_dir = os.path.join(nuitka_app_dir, is_arch_specific)
+        nuitka_download_dir = os.path.join(nuitka_download_dir, is_arch_specific)
 
-    if specifity:
-        nuitka_app_dir = os.path.join(nuitka_app_dir, specifity)
+    if specificity:
+        nuitka_download_dir = os.path.join(nuitka_download_dir, specificity)
 
-    download_path = os.path.join(nuitka_app_dir, os.path.basename(url))
-    exe_path = os.path.join(nuitka_app_dir, binary)
+    download_path = os.path.join(nuitka_download_dir, os.path.basename(url))
+    exe_path = os.path.join(nuitka_download_dir, binary)
 
-    makePath(nuitka_app_dir)
+    makePath(nuitka_download_dir)
 
     if not os.path.isfile(download_path) and not os.path.isfile(exe_path):
         if assume_yes_for_downloads:
@@ -71,7 +72,7 @@ Is it OK to download and put it in '%s'.
 No installer needed, cached, one time question.
 
 Proceed and download? [Yes]/No """
-                % (message, nuitka_app_dir)
+                % (message, nuitka_download_dir)
             )
             Tracing.flushStandardOutputs()
 
@@ -88,13 +89,13 @@ Proceed and download? [Yes]/No """
 
             try:
                 urlretrieve(url, download_path)
-            except Exception:  # Any kind of error, pylint: disable=broad-except
+            except Exception as e:  # Any kind of error, pylint: disable=broad-except
                 try:
                     urlretrieve(url.replace("https://", "http://"), download_path)
                 except Exception:  # Any kind of error, pylint: disable=broad-except
                     Tracing.general.sysexit(
-                        "Failed to download '%s'. Contents should manually be copied to '%s'."
-                        % (url, download_path)
+                        "Failed to download '%s' due to '%s'. Contents should manually be copied to '%s'."
+                        % (url, e, download_path)
                     )
 
     if not os.path.isfile(exe_path) and os.path.isfile(download_path):
@@ -113,7 +114,7 @@ Proceed and download? [Yes]/No """
                 if flatten:
                     zip_info.filename = os.path.basename(zip_info.filename)
 
-                zip_file.extract(zip_info, nuitka_app_dir)
+                zip_file.extract(zip_info, nuitka_download_dir)
 
         except Exception:  # Catching anything zip throws, pylint: disable=broad-except
             Tracing.general.info("Problem with the downloaded zip file, deleting it.")
@@ -122,7 +123,7 @@ Proceed and download? [Yes]/No """
             deleteFile(download_path, must_exist=True)
 
             Tracing.general.sysexit(
-                "Error, need %r as extracted from %r." % (binary, url)
+                "Error, need '%s' as extracted from '%s'." % (binary, url)
             )
 
     # Check success here, and make sure it's executable.
@@ -141,16 +142,16 @@ def getCachedDownloadedMinGW64(target_arch, assume_yes_for_downloads):
     # Large URLs, pylint: disable=line-too-long
 
     if target_arch == "x86_64":
-        url = "https://github.com/brechtsanders/winlibs_mingw/releases/download/11.2.0-12.0.1-9.0.0-r1/winlibs-x86_64-posix-seh-gcc-11.2.0-llvm-12.0.1-mingw-w64-9.0.0-r1.zip"
+        url = "https://github.com/brechtsanders/winlibs_mingw/releases/download/11.3.0-14.0.3-10.0.0-msvcrt-r3/winlibs-x86_64-posix-seh-gcc-11.3.0-llvm-14.0.3-mingw-w64msvcrt-10.0.0-r3.zip"
         binary = r"mingw64\bin\gcc.exe"
     else:
-        url = "https://github.com/brechtsanders/winlibs_mingw/releases/download/11.2.0-12.0.1-9.0.0-r1/winlibs-i686-posix-dwarf-gcc-11.2.0-llvm-12.0.1-mingw-w64-9.0.0-r1.zip"
+        url = "https://github.com/brechtsanders/winlibs_mingw/releases/download/11.3.0-14.0.3-10.0.0-msvcrt-r3/winlibs-i686-posix-dwarf-gcc-11.3.0-llvm-14.0.3-mingw-w64msvcrt-10.0.0-r3.zip"
         binary = r"mingw32\bin\gcc.exe"
 
     gcc_binary = getCachedDownload(
         url=url,
         is_arch_specific=target_arch,
-        specifity=url.rsplit("/", 2)[1],
+        specificity=url.rsplit("/", 2)[1],
         binary=binary,
         flatten=False,
         message="Nuitka will use gcc from MinGW64 of winlibs to compile on Windows.",
