@@ -44,7 +44,7 @@ from nuitka.Tracing import general, my_print, options_logger
 
 from .Importing import importFromInlineCopy
 from .ThreadedExecutor import RLock, getThreadIdent
-from .Utils import getOS, isMacOS, isWin32Windows
+from .Utils import isMacOS, isWin32OrPosixWindows, isWin32Windows
 
 # Locking seems to be only required for Windows mostly, but we can keep
 # it for all.
@@ -136,7 +136,7 @@ def relpath(path, start="."):
     except ValueError:
         # On Windows, paths on different devices prevent it to work. Use that
         # full path then.
-        if getOS() == "Windows":
+        if isWin32OrPosixWindows():
             return os.path.abspath(path)
         raise
 
@@ -206,12 +206,7 @@ def _getRealPathWindows(path):
                 [
                     _powershell_path,
                     "-NoProfile",
-                    "Get-Item",
-                    path,
-                    "|",
-                    "Select-Object",
-                    "-ExpandProperty",
-                    "Target",
+                    'Get-Item "%s" | Select-Object -ExpandProperty Target' % path,
                 ],
                 shell=False,
             )
@@ -243,9 +238,10 @@ def getDirectoryRealPath(path):
     """
     path = os.path.realpath(path)
 
-    # Attempt to resolve Windows symlinks on Python2
-    if os.name == "nt" and not os.path.isdir(path) and os.path.exists(path):
-        path = _getRealPathWindows(path)
+    # Attempt to resolve Windows symlinks older Python
+    if os.name == "nt":
+        if os.path.islink(path) or (not os.path.isdir(path) and os.path.exists(path)):
+            path = _getRealPathWindows(path)
 
     return path
 
@@ -305,7 +301,7 @@ def getFileList(
 
     result = []
 
-    # Normalize ignoredirs for better matching.
+    # Normalize "ignore_dirs" for better matching.
     ignore_dirs = [os.path.normcase(ignore_dir) for ignore_dir in ignore_dirs]
     ignore_filenames = [
         os.path.normcase(ignore_filename) for ignore_filename in ignore_filenames
@@ -380,12 +376,13 @@ def getSubDirectories(path, ignore_dirs=()):
     return result
 
 
-def listDllFilesFromDirectory(path, prefix=""):
+def listDllFilesFromDirectory(path, prefix=None, suffixes=None):
     """Give a sorted listing of DLLs filenames in a path.
 
     Args:
         path: directory to create a DLL listing from
         prefix: shell pattern to match filename start against, can be None
+        suffixes: shell patch to match filename end against, defaults to all platform ones
 
     Returns:
         Sorted list of tuples of full filename, and basename of
@@ -400,7 +397,9 @@ def listDllFilesFromDirectory(path, prefix=""):
     # Accept None value as well.
     prefix = prefix or ""
 
-    pattern_list = [prefix + "*." + suffix for suffix in ("dll", "so.*", "so", "dylib")]
+    suffixes = suffixes or ("dll", "so.*", "so", "dylib")
+
+    pattern_list = [prefix + "*." + suffix for suffix in suffixes]
 
     for fullpath, filename in listDir(path):
         for pattern in pattern_list:
@@ -409,6 +408,47 @@ def listDllFilesFromDirectory(path, prefix=""):
                 break
 
 
+def listExeFilesFromDirectory(path, prefix=None, suffixes=None):
+    """Give a sorted listing of EXE filenames in a path.
+
+    Args:
+        path: directory to create a DLL listing from
+        prefix: shell pattern to match filename start against, can be None
+        suffixes: shell patch to match filename end against, can be None
+
+    Returns:
+        Sorted list of tuples of full filename, and basename of
+        DLLs in that directory.
+
+    Notes:
+        Typically the full name and the basename are both needed
+        so this function simply does both, for ease of use on the
+        calling side.
+    """
+
+    # Accept None value as well.
+    prefix = prefix or ""
+
+    # On Windows, we check exe suffixes, on other platforms we shell all filenames,
+    # matching the prefix, but they have to the executable bit set.
+    if suffixes is None and isWin32OrPosixWindows():
+        suffixes = "exe", "bin"
+
+    if suffixes:
+        pattern_list = [prefix + "*." + suffix for suffix in suffixes]
+    else:
+        pattern_list = [prefix + "*"]
+
+    for fullpath, filename in listDir(path):
+        for pattern in pattern_list:
+            if fnmatch.fnmatch(filename, pattern):
+                if not isWin32OrPosixWindows() and not os.access(fullpath, os.X_OK):
+                    continue
+
+                yield fullpath, filename
+                break
+
+
 def getSubDirectoriesWithDlls(path):
     """Get all directories below a given path.
 
@@ -452,6 +492,28 @@ def _isMacOSFramework(path):
     return isMacOS() and os.path.isdir(path) and path.endswith(".framework")
 
 
+def isLink(path):
+    result = os.path.islink(path)
+
+    # Special handling for Junctions.
+    if not result and isWin32Windows():
+        import ctypes.wintypes
+
+        GetFileAttributesW = ctypes.windll.kernel32.GetFileAttributesW
+        GetFileAttributesW.restype = ctypes.wintypes.DWORD
+        GetFileAttributesW.argtypes = (ctypes.wintypes.LPCWSTR,)
+
+        INVALID_FILE_ATTRIBUTES = 0xFFFFFFFF
+        FILE_ATTRIBUTE_REPARSE_POINT = 0x00400
+
+        result = GetFileAttributesW(path)
+
+        if result != INVALID_FILE_ATTRIBUTES:
+            result = bool(result & FILE_ATTRIBUTE_REPARSE_POINT)
+
+    return result
+
+
 def deleteFile(path, must_exist):
     """Delete a file, potentially making sure it exists.
 
@@ -463,7 +525,7 @@ def deleteFile(path, must_exist):
         possible.
     """
     with withFileLock("deleting file %s" % path):
-        if os.path.islink(path) or os.path.isfile(path):
+        if isLink(path) or os.path.isfile(path):
             try:
                 os.unlink(path)
             except OSError:
@@ -771,8 +833,8 @@ def getWindowsDrive(path):
     return os.path.normcase(drive)
 
 
-def isPathBelow(path, filename):
-    """Is a path inside of a given directory path
+def isFilenameBelowPath(path, filename):
+    """Is a filename inside of a given directory path
 
     Args:
         path: location to be below
@@ -780,7 +842,7 @@ def isPathBelow(path, filename):
     """
     if type(path) in (tuple, list):
         for p in path:
-            if isPathBelow(path=p, filename=filename):
+            if isFilenameBelowPath(path=p, filename=filename):
                 return True
 
         return False
@@ -795,9 +857,9 @@ def isPathBelow(path, filename):
     return os.path.relpath(filename, path).split(os.path.sep)[0] != ".."
 
 
-def isPathBelowOrSameAs(path, filename):
-    """Is a path inside of a given directory path or the same path as that directory."""
-    return isPathBelow(path, filename) or areSamePaths(path, filename)
+def isFilenameSameAsOrBelowPath(path, filename):
+    """Is a filename inside of a given directory path or the same path as that directory."""
+    return isFilenameBelowPath(path, filename) or areSamePaths(path, filename)
 
 
 def getWindowsShortPathName(filename):
@@ -813,11 +875,11 @@ def getWindowsShortPathName(filename):
     import ctypes.wintypes
 
     GetShortPathNameW = ctypes.windll.kernel32.GetShortPathNameW
-    GetShortPathNameW.argtypes = [
+    GetShortPathNameW.argtypes = (
         ctypes.wintypes.LPCWSTR,
         ctypes.wintypes.LPWSTR,
         ctypes.wintypes.DWORD,
-    ]
+    )
     GetShortPathNameW.restype = ctypes.wintypes.DWORD
 
     output_buf_size = 0
@@ -857,7 +919,7 @@ def getExternalUsePath(filename, only_dirname=False):
     Returns:
         Path that is a absolute and (on Windows) short filename pointing at the same file.
     Notes:
-        This is only os.path.abspath except on Windows, where is coverts
+        This is only "os.path.abspath" except on Windows, where is converts
         to a short path too.
     """
 
@@ -874,6 +936,17 @@ def getExternalUsePath(filename, only_dirname=False):
     return filename
 
 
+def getReportPath(filename):
+    """Convert filename into a path suitable for reporting, avoiding home directory paths."""
+    if os.path.isabs(os.path.expanduser(filename)):
+        abs_filename = os.path.abspath(os.path.expanduser(filename))
+        home_path = os.path.expanduser("~")
+        if isFilenameBelowPath(path=home_path, filename=abs_filename):
+            return os.path.join("~", relpath(path=abs_filename, start=home_path))
+
+    return filename
+
+
 def getLinkTarget(filename):
     """Return the path a link is pointing too, if any.
 
@@ -931,7 +1004,7 @@ def resolveShellPatternToFilenames(pattern):
 
             if glob2 is None:
                 options_logger.sysexit(
-                    "Using pattern with ** is not supported before Python 3.5 unless glob2 is installed."
+                    "Using pattern with '**' is not supported before Python 3.5 unless glob2 is installed."
                 )
 
             result = glob2.glob(pattern)
