@@ -1,4 +1,4 @@
-#     Copyright 2019, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -23,52 +23,59 @@ stub that does not thread at all.
 
 from threading import RLock, current_thread
 
+# Set this to false, to enable actual use of threads. This was found no longer
+# useful with dependency walker, but might be true in other cases.
 _use_threaded_executor = False
 
-try:
-    if _use_threaded_executor:
-        from concurrent.futures import (
-            ThreadPoolExecutor,
-            wait,
-            as_completed,
-            FIRST_EXCEPTION,
-        )  # pylint: disable=I0021,import-error,no-name-in-module,unused-import
 
+class NonThreadedPoolExecutor(object):
+    def __init__(self, max_workers=None):
+        # This stub ignores max_workers, pylint: disable=unused-argument
+
+        self.results = []
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_value, exc_traceback):
+        return False
+
+    def submit(self, function, *args):
+        # Submitted jobs are simply done immediately.
+        self.results.append(function(*args))
+
+        return self
+
+
+def waitWorkers(workers):
+    if workers:
+        return iter(workers[0].results)
+
+
+ThreadPoolExecutor = NonThreadedPoolExecutor
+
+
+if _use_threaded_executor:
+    try:
+        from concurrent.futures import (  # pylint: disable=I0021,import-error,no-name-in-module,unused-import
+            FIRST_EXCEPTION,
+            ThreadPoolExecutor,
+            as_completed,
+            wait,
+        )
+
+        # We overwrite this if wanted, pylint: disable=function-redefined
         def waitWorkers(workers):
             wait(workers, return_when=FIRST_EXCEPTION)
 
             for future in as_completed(workers):
                 yield future.result()
 
-
-except ImportError:
-    _use_threaded_executor = False
-
-if not _use_threaded_executor:
-    # No backport installed, use stub for at least Python 2.6, and potentially
-    # also Python 2.7, we might want to tell the user about it though, that
-    # we think it should be installed. pylint: disable=function-redefined
-    class ThreadPoolExecutor(object):
-        def __init__(self, max_workers=None):
-            # This stub ignores max_workers, pylint: disable=unused-argument
-
-            self.results = []
-
-        def __enter__(self):
-            return self
-
-        def __exit__(self, exc_type, exc_value, exc_traceback):
-            return False
-
-        def submit(self, function, *args):
-            # Submitted jobs are simply done immediately.
-            self.results.append(function(*args))
-
-            return self
-
-    def waitWorkers(workers):
-        if workers:
-            return iter(workers[0].results)
+    except ImportError:
+        # No backport installed, use stub for at least Python 2.6, and potentially
+        # also Python 2.7, we might want to tell the user about it though, that
+        # we think it should be installed.
+        pass
 
 
 def getThreadIdent():
