@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -41,7 +41,7 @@ from nuitka.tools.testing.Common import (
     compareWithCPython,
     createSearchMode,
     decideNeeds2to3,
-    hasDebugPython,
+    getDebugPython,
     scanDirectoryForTestCases,
     setup,
 )
@@ -75,28 +75,27 @@ def main():
 
         # This test should be run with the debug Python, and makes outputs to
         # standard error that might be ignored.
-        if filename.startswith("Referencing"):
-            if hasDebugPython():
-                extra_flags.append("python_debug")
+        if filename.startswith("Referencing") and getDebugPython() is not None:
+            extra_flags.append("--python-debug")
 
         # This tests warns about __import__() used.
-        if filename == "OrderChecks.py":
+        if filename == "OrderChecksTest.py":
             extra_flags.append("ignore_warnings")
 
         # This tests warns about an package relative import despite
         # being in no package.
-        if filename == "Importing.py":
+        if filename == "ImportingTest.py":
             extra_flags.append("ignore_warnings")
 
         # TODO: Nuitka does not give output for ignored exception in dtor, this is
         # not fully compatible and potentially an error.
-        if filename == "YieldFrom33.py":
+        if filename == "YieldFromTest33.py":
             extra_flags.append("ignore_stderr")
 
         # For Python2 there is a "builtins" package that gives warnings. TODO: We
         # ought to NOT import that package and detect statically that __builtins__
         # import won't raise ImportError.
-        if filename == "BuiltinOverload.py":
+        if filename == "BuiltinOverloadTest.py":
             extra_flags.append("ignore_warnings")
 
         active = search_mode.consider(dirname=None, filename=filename)
