@@ -22,7 +22,6 @@ from __future__ import print_function
 # nuitka-skip-unless-imports: numpy
 
 # nuitka-project: --standalone
-# nuitka-project: --enable-plugin=numpy
 
 # Make sure, the usual bad ones are not included with anti-bloat.
 
@@ -40,4 +39,9 @@ print("An array", a)
 
 import numpy.random._bounded_integers
 
+import numpy.lib.recfunctions
+
+a = numpy.zeros(4, dtype=[('a', 'i4'), ('b', 'f4,u2'), ('c', 'f4', 2)])
+print("Another numpy usage", numpy.lib.recfunctions.structured_to_unstructured(a))
+
 print("OK.")
\ No newline at end of file
