@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python test originally created or extracted from other peoples work. The
 #     parts from me are licensed as below. It is at least Free Software where
@@ -43,7 +43,6 @@ sys.path.insert(
 
 # isort:start
 
-from nuitka.freezer.RuntimeTracing import getRuntimeTraceOfLoadedFiles
 from nuitka.tools.testing.Common import (
     checkLoadedFileAccesses,
     checkRequirements,
@@ -57,9 +56,10 @@ from nuitka.tools.testing.Common import (
     setup,
     test_logger,
 )
+from nuitka.tools.testing.RuntimeTracing import getRuntimeTraceOfLoadedFiles
 from nuitka.utils.FileOperations import removeDirectory
 from nuitka.utils.Timing import TimerReport
-from nuitka.utils.Utils import getOS
+from nuitka.utils.Utils import isMacOS, isWin32Windows
 
 
 def displayError(dirname, filename):
@@ -101,6 +101,8 @@ def main():
             "cpython_cache",
             # To understand what is slow.
             "timing",
+            # Don't care here, this is mostly for coverage.
+            "--nowarn-mnemonic=debian-dist-packages",
         ]
 
         # skip each test if their respective requirements are not met
@@ -141,11 +143,11 @@ def main():
                 continue
 
         if filename == "TkInterUsing.py":
-            if getOS() == "Darwin":
+            if isMacOS():
                 reportSkip("Not working macOS yet", ".", filename)
                 continue
 
-            if getOS() == "Windows":
+            if isWin32Windows() == "Windows":
                 reportSkip("Can hang on Windows CI.", ".", filename)
                 continue
 
@@ -156,7 +158,7 @@ def main():
             # For the warnings.
             extra_flags.append("ignore_warnings")
 
-        # TODO: Once we have a noqt Plugin, we should use that.
+        # TODO: Once we have a no-Qt Plugin, we should use that.
         if filename == "MatplotlibUsing.py":
             # For the plugin warnings.
             extra_flags.append("ignore_warnings")
@@ -168,10 +170,8 @@ def main():
 
         if filename == "PandasUsing.py":
             extra_flags.append("plugin_enable:numpy")
+            extra_flags.append("plugin_enable:no-qt")
             extra_flags.append("plugin_disable:pylint-warnings")
-            extra_flags.append("plugin_disable:pyqt5")
-            extra_flags.append("plugin_disable:pyside2")
-            extra_flags.append("plugin_disable:pyside6")
 
         if filename == "PmwUsing.py":
             extra_flags.append("plugin_enable:pmw-freezer")
@@ -199,16 +199,6 @@ def main():
                 reportSkip("irrelevant Python version", ".", filename)
                 continue
 
-            # For the plug-in information
-            if filename.startswith("PySide2"):
-                extra_flags.append("plugin_enable:pyside6")
-            elif filename.startswith("PySide6"):
-                extra_flags.append("plugin_enable:pyside6")
-            elif filename.startswith("PyQt5"):
-                extra_flags.append("plugin_enable:pyqt5")
-            elif filename.startswith("PyQt6"):
-                extra_flags.append("plugin_enable:pyqt6")
-
         test_logger.info(
             "Consider output of standalone mode compiled program: %s" % filename
         )
